/*
 * Text.h
 *
 * Description:
 * 	Allows for drawing text in a bitmap environment.
 *
 * Authors:
 * 	Tim Li
 */

//represent a character as a 5x3 array of dots (use squares for dots)
#define CHAR_ROWS 5
#define CHAR_COLS 3

//currently only supports numbers, but characters could be added easily
const uchar NumbersBMPs[10][15] = 
	{ { 0, 1, 0,  1, 0, 1,  1, 0, 1,  1, 0, 1,  0, 1, 0},	//0
	  { 0, 1, 0,  1, 1, 0,  0, 1, 0,  0, 1, 0,  1, 1, 1},	//1
	  { 0, 1, 0,  1, 0, 1,  0, 0, 1,  0, 1, 0,  1, 1, 1},	//2
	  { 1, 1, 1,  0, 0, 1,  0, 1, 1,  0, 0, 1,  1, 1, 1},	//3
	  { 1, 0, 1,  1, 0, 1,  1, 1, 1,  0, 0, 1,  0, 0, 1},	//4
	  { 1, 1, 1,  1, 0, 0,  1, 1, 1,  0, 0, 1,  1, 1, 1},	//5
	  { 1, 1, 1,  1, 0, 0,  1, 1, 1,  1, 0, 1,  1, 1, 1},	//6
	  { 1, 1, 1,  0, 0, 1,  0, 0, 1,  0, 0, 1,  0, 0, 1},	//7
	  { 1, 1, 1,  1, 0, 1,  1, 1, 1,  1, 0, 1,  1, 1, 1},	//8
	  { 1, 1, 1,  1, 0, 1,  1, 1, 1,  0, 0, 1,  0, 0, 1}	//9
	};
/* E.g. 0 is represented as, and A could be represented as:
 *	 0				 A
 *	0 0				A A
 *	0 0				AAA
 *	0 0				A A
 *	 0				A A
*/

//draw a digit (number) at (X,Y), with dotSize and color 
void drawNumber(int X, int Y, int number, int dotSize, long color)
{
	int x, y, dot=0;	//dot = iterator for bitmap index
	
	if (number < 0 || number > 9)
		return;		//only draws single numbers

	for (y = 0; y < CHAR_ROWS; y++)
	{
		for (x = 0; x < CHAR_COLS; x++)
		{
			if (NumbersBMPs[number][dot++])
			{	//if that digit's bitmap had a 1, draw a dot at that point
				rect(X+x*dotSize, Y+y*dotSize, X+(x+1)*dotSize, Y+(y+1)*dotSize, color);
			}
		}
	}
}

