//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*
-------------------------------------------------------------------------------
   Title      : Pipeline ZBT RAM Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : ZBT_CONTROL.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : defines.v ADDR_BUS_INTERFACE.v CTRL_BUS_INTERFACE.v 
			DATA_BUS_INTERFACE.v PIPELINES.v
-------------------------------------------------------------------------------
   Used by    : MEMORY_BANK0_RAM_CTRL.v  MEMORY_BANK1_RAM_CTRL.v
-------------------------------------------------------------------------------
   Description: This module instantiates the submodules that make up the ZBT controller.

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
*/

//`include "W:\ece532\svga_dbl_buf\pcores\bm_mode_svga_ctrl\hdl\verilog\defines.v"

module zbt_control (
ram_clock,
ram_clocken_n,
ram_we_n,
ram_oe_n,
ram_ce0_n,
ram_ce1,
ram_ce2_n,
ram_adv_ld,
ram_wea_n,
ram_web_n,
ram_wec_n,
ram_wed_n,
ram_lbo,
ram_zz,
ram_addr,
user_read_data,

//ram_data_bus,
ram_data_bus_i,
ram_data_bus_o,
ram_data_bus_t,

user_ram_clock,
user_clocken,
user_read_writez,
user_oe,
user_ce0,
user_ce1,
user_ce2,
user_adv_ld,
user_wea,
user_web,
user_wec,
user_wed,
user_lbo,
user_zz,
clock,
user_write_data,
user_ram_addr
);

parameter ADDR_BITS=19;
parameter DATA_BITS=32;
  
//inout  [(DATA_BITS-1):0]ram_data_bus; 	// bidirectional data bus to RAM
input   [(DATA_BITS-1):0]ram_data_bus_i;
output  [(DATA_BITS-1):0]ram_data_bus_o;
output  [(DATA_BITS-1):0]ram_data_bus_t;

output [(ADDR_BITS-1):0]ram_addr;     	// addr goes out to the RAM
output     			ram_lbo;      	// Burst Mode (0 = Linear, 1 = Interleaved)
output     			ram_clock;	// clock to the RAM
output     			ram_clocken_n;	// Synchronous Clock Enable
output     			ram_adv_ld;	// Synchronous address Adv/LD
output    			ram_wea_n;	// Synchronous Byte Write Enable A
output     			ram_web_n;	// Synchronous Byte Write Enable B
output     			ram_wec_n;	// Synchronous Byte Write Enable C
output     			ram_wed_n;	// Synchronous Byte Write Enable D
output     			ram_we_n;		// Read write control goes out to the RAM
output     			ram_oe_n;		// Output Enable
output   			   	ram_ce0_n;	// Synchronous Chip Enable
output   			   	ram_ce1;	  	// Synchronous Chip Enable
output     		  	ram_ce2_n;	// Synchronous Chip Enable
output     			ram_zz;	  	// Snooze Mode

   
input [(ADDR_BITS-1):0] user_ram_addr;    // addr comes in from the user design
input [(DATA_BITS-1):0] user_write_data;  // data comes in from the user design
input 		     	user_read_writez; // Read write control comes from the user design
output [(DATA_BITS-1):0]user_read_data;   // data read from SRAM
   
input user_ram_clock;
input user_clocken;
input user_oe;
input user_ce0;
input user_ce1;
input user_ce2;
input user_adv_ld;
input user_wea;
input user_web;
input user_wec;
input user_wed;
input user_lbo;
input user_zz;
input clock;

   
wire [(DATA_BITS-1):0]   	write_data;
wire [(DATA_BITS-1):0]   	read_data;
wire	[(DATA_BITS-1):0]   	tristate_control;

//tri 	[(DATA_BITS-1):0]  	ram_data_bus;
wire	[(DATA_BITS-1):0]   	ram_data_bus_i;
wire	[(DATA_BITS-1):0]   	ram_data_bus_o;
wire	[(DATA_BITS-1):0]   	ram_data_bus_t;  

wire	 ram_lbo;    		// Burst Mode (0 = Linear, 1 = Interleaved)
wire  ram_clock;      	// clock to the RAM
wire  ram_clocken_n;   	// Synchronous Clock Enable
wire  ram_adv_ld;     	// Synchronous address Adv/LD
wire  ram_wea_n;    	// Synchronous Byte Write Enable A
wire  ram_web_n;    	// Synchronous Byte Write Enable B
wire  ram_wec_n;    	// Synchronous Byte Write Enable C
wire  ram_wed_n;    	// Synchronous Byte Write Enable D
wire  ram_we_n;     	// Read write control goes out to the RAM
wire  ram_oe_n;     	// Output Enable
wire  ram_ce0_n;     	// Synchronous Chip Enable
wire  ram_ce1;      	// Synchronous Chip Enable
wire  ram_ce2_n;    	// Synchronous Chip Enable
wire  ram_zz;       	// Snooze Mode


//OBUF_F_12 I_obuf_ram_clock (.O(ram_clock), .I(user_ram_clock));
assign	ram_clock = user_ram_clock;
/*
assign  write_data[(DATA_BITS-1):0] = ram_data_bus_i[(DATA_BITS-1):0];
assign  ram_data_bus_o[(DATA_BITS-1):0] = read_data[(DATA_BITS-1):0];
assign  ram_data_bus_t[(DATA_BITS-1):0] = latched_tristate_control[(DATA_BITS-1):0];
*/
pipelines pipelines(
user_read_data, 
write_data, 
tristate_control, 
user_write_data, 
user_read_writez, 
read_data, 
clock
);

data_bus_interface data_bus_interface(
read_data, 
//ram_data_bus, 
write_data, 
tristate_control, 
clock,
ram_data_bus_i,
ram_data_bus_o,
ram_data_bus_t
);
   
addr_bus_interface addr_bus_interface(
ram_addr, 
user_ram_addr, 
clock
);

ctrl_bus_interface ctrl_bus_interface(
ram_clocken_n,
ram_we_n,
ram_oe_n,
ram_ce0_n,
ram_ce1,
ram_ce2_n,
ram_adv_ld,
ram_wea_n,
ram_web_n,
ram_wec_n,
ram_wed_n,
ram_lbo,
ram_zz,
user_clocken,
user_read_writez,
user_oe,
user_ce0,
user_ce1,
user_ce2,
user_adv_ld,
user_wea,
user_web,
user_wec,
user_wed,
user_lbo,
user_zz,
clock
); 
endmodule // ZBT_CONTROL




