// Provides a buffer switcher for two sets on ZBT memory control inputs.
//   They can either go straight through (INA->OUT1 / INB->OUT2),
//   or after switch_pedge is trigger (INA->OUT2 / INB->OUT1).
//
// Timothy Li, 2005-03-11

module zbt_buffer_mux
(
//switch signal (flips on posedge)
  switch_pedge,
//input A
  MEMORY_INA_CLK_P,
  MEMORY_INA_CLKEN_N,
  MEMORY_INA_WEN_N,
  MEMORY_INA_WENA_N,
  MEMORY_INA_WENB_N,
  MEMORY_INA_WENC_N,
  MEMORY_INA_WEND_N,
  MEMORY_INA_ADV_LD_N,
  MEMORY_INA_OEN_N,
  MEMORY_INA_CEN_N,
  MEMORY_INA_ADDR_P,
  MEMORY_INA_DATA_A_P_I,
  MEMORY_INA_DATA_A_P_O,
  MEMORY_INA_DATA_A_P_T,
  MEMORY_INA_DATA_B_P_I,
  MEMORY_INA_DATA_B_P_O,
  MEMORY_INA_DATA_B_P_T,
  MEMORY_INA_DATA_C_P_I,
  MEMORY_INA_DATA_C_P_O,
  MEMORY_INA_DATA_C_P_T,
  MEMORY_INA_DATA_D_P_I,
  MEMORY_INA_DATA_D_P_O,
  MEMORY_INA_DATA_D_P_T,
//input B
  MEMORY_INB_CLK_P,
  MEMORY_INB_CLKEN_N,
  MEMORY_INB_WEN_N,
  MEMORY_INB_WENA_N,
  MEMORY_INB_WENB_N,
  MEMORY_INB_WENC_N,
  MEMORY_INB_WEND_N,
  MEMORY_INB_ADV_LD_N,
  MEMORY_INB_OEN_N,
  MEMORY_INB_CEN_N,
  MEMORY_INB_ADDR_P,
  MEMORY_INB_DATA_A_P_I,
  MEMORY_INB_DATA_A_P_O,
  MEMORY_INB_DATA_A_P_T,
  MEMORY_INB_DATA_B_P_I,
  MEMORY_INB_DATA_B_P_O,
  MEMORY_INB_DATA_B_P_T,
  MEMORY_INB_DATA_C_P_I,
  MEMORY_INB_DATA_C_P_O,
  MEMORY_INB_DATA_C_P_T,
  MEMORY_INB_DATA_D_P_I,
  MEMORY_INB_DATA_D_P_O,
  MEMORY_INB_DATA_D_P_T,
//output to first bank
  MEMORY_OUT1_CLK_P,
  MEMORY_OUT1_CLKEN_N,
  MEMORY_OUT1_WEN_N,
  MEMORY_OUT1_WENA_N,
  MEMORY_OUT1_WENB_N,
  MEMORY_OUT1_WENC_N,
  MEMORY_OUT1_WEND_N,
  MEMORY_OUT1_ADV_LD_N,
  MEMORY_OUT1_OEN_N,
  MEMORY_OUT1_CEN_N,
  MEMORY_OUT1_ADDR_P,
  MEMORY_OUT1_DATA_A_P_I,
  MEMORY_OUT1_DATA_A_P_O,
  MEMORY_OUT1_DATA_A_P_T,
  MEMORY_OUT1_DATA_B_P_I,
  MEMORY_OUT1_DATA_B_P_O,
  MEMORY_OUT1_DATA_B_P_T,
  MEMORY_OUT1_DATA_C_P_I,
  MEMORY_OUT1_DATA_C_P_O,
  MEMORY_OUT1_DATA_C_P_T,
  MEMORY_OUT1_DATA_D_P_I,
  MEMORY_OUT1_DATA_D_P_O,
  MEMORY_OUT1_DATA_D_P_T,
//output to second bank
  MEMORY_OUT2_CLK_P,
  MEMORY_OUT2_CLKEN_N,
  MEMORY_OUT2_WEN_N,
  MEMORY_OUT2_WENA_N,
  MEMORY_OUT2_WENB_N,
  MEMORY_OUT2_WENC_N,
  MEMORY_OUT2_WEND_N,
  MEMORY_OUT2_ADV_LD_N,
  MEMORY_OUT2_OEN_N,
  MEMORY_OUT2_CEN_N,
  MEMORY_OUT2_ADDR_P,
  MEMORY_OUT2_DATA_A_P_I,
  MEMORY_OUT2_DATA_A_P_O,
  MEMORY_OUT2_DATA_A_P_T,
  MEMORY_OUT2_DATA_B_P_I,
  MEMORY_OUT2_DATA_B_P_O,
  MEMORY_OUT2_DATA_B_P_T,
  MEMORY_OUT2_DATA_C_P_I,
  MEMORY_OUT2_DATA_C_P_O,
  MEMORY_OUT2_DATA_C_P_T,
  MEMORY_OUT2_DATA_D_P_I,
  MEMORY_OUT2_DATA_D_P_O,
  MEMORY_OUT2_DATA_D_P_T
);

input switch_pedge;		//triggers buffer switch on posedge

input  [18:0]	MEMORY_INA_ADDR_P;		// external memory address bus
input 			MEMORY_INA_CLK_P;		// external memory clock
input 			MEMORY_INA_CLKEN_N;		// external memory clock enable
input 			MEMORY_INA_WEN_N;		// external memory read/write
input 			MEMORY_INA_WENA_N;		// external memory write enable for byte A
input 			MEMORY_INA_WENB_N;		// external memory write enable for byte B
input 			MEMORY_INA_WENC_N;		// external memory write enable for byte C
input 			MEMORY_INA_WEND_N;		// external memory write enable for byte D
input 			MEMORY_INA_OEN_N;		// external memory input enable
input 			MEMORY_INA_CEN_N;		// external memory chip enable
input 			MEMORY_INA_ADV_LD_N;	// external memory load/advance
output [7:0]	MEMORY_INA_DATA_A_P_I;	//OUR output,  but O on the IOBUF at the system level
input  [7:0]	MEMORY_INA_DATA_A_P_O;	//OUR input, but I on the IOBUF at the system level
input  [7:0]	MEMORY_INA_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
output [7:0]	MEMORY_INA_DATA_B_P_I;
input  [7:0]	MEMORY_INA_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
input  [7:0]	MEMORY_INA_DATA_B_P_T;
output [7:0]	MEMORY_INA_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
input  [7:0]	MEMORY_INA_DATA_C_P_O;
input  [7:0]	MEMORY_INA_DATA_C_P_T;
output [7:0]	MEMORY_INA_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
input  [7:0]	MEMORY_INA_DATA_D_P_O;
input  [7:0]	MEMORY_INA_DATA_D_P_T;

input  [18:0]	MEMORY_INB_ADDR_P;		// external memory address bus
input 			MEMORY_INB_CLK_P;		// external memory clock
input 			MEMORY_INB_CLKEN_N;		// external memory clock enable
input 			MEMORY_INB_WEN_N;		// external memory read/write
input 			MEMORY_INB_WENA_N;		// external memory write enable for byte A
input 			MEMORY_INB_WENB_N;		// external memory write enable for byte B
input 			MEMORY_INB_WENC_N;		// external memory write enable for byte C
input 			MEMORY_INB_WEND_N;		// external memory write enable for byte D
input 			MEMORY_INB_OEN_N;		// external memory input enable
input 			MEMORY_INB_CEN_N;		// external memory chip enable
input 			MEMORY_INB_ADV_LD_N;	// external memory load/advance
output [7:0]	MEMORY_INB_DATA_A_P_I;	//OUR output,  but O on the IOBUF at the system level
input  [7:0]	MEMORY_INB_DATA_A_P_O;	//OUR input, but I on the IOBUF at the system level
input  [7:0]	MEMORY_INB_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
output [7:0]	MEMORY_INB_DATA_B_P_I;
input  [7:0]	MEMORY_INB_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
input  [7:0]	MEMORY_INB_DATA_B_P_T;
output [7:0]	MEMORY_INB_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
input  [7:0]	MEMORY_INB_DATA_C_P_O;
input  [7:0]	MEMORY_INB_DATA_C_P_T;
output [7:0]	MEMORY_INB_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
input  [7:0]	MEMORY_INB_DATA_D_P_O;
input  [7:0]	MEMORY_INB_DATA_D_P_T;

output [18:0]	MEMORY_OUT1_ADDR_P;		// external memory address bus
output 			MEMORY_OUT1_CLK_P;		// external memory clock
output 			MEMORY_OUT1_CLKEN_N;	// external memory clock enable
output 			MEMORY_OUT1_WEN_N;		// external memory read/write
output 			MEMORY_OUT1_WENA_N;		// external memory write enable for byte A
output 			MEMORY_OUT1_WENB_N;		// external memory write enable for byte B
output 			MEMORY_OUT1_WENC_N;		// external memory write enable for byte C
output 			MEMORY_OUT1_WEND_N;		// external memory write enable for byte D
output 			MEMORY_OUT1_OEN_N;		// external memory output enable
output 			MEMORY_OUT1_CEN_N;		// external memory chip enable
output 			MEMORY_OUT1_ADV_LD_N;	// external memory load/advance
input  [7:0]	MEMORY_OUT1_DATA_A_P_I;	//OUR input,  but O on the IOBUF at the system level
output [7:0]	MEMORY_OUT1_DATA_A_P_O;	//OUR output, but I on the IOBUF at the system level
output [7:0]	MEMORY_OUT1_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
input  [7:0]	MEMORY_OUT1_DATA_B_P_I;
output [7:0]	MEMORY_OUT1_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
output [7:0]	MEMORY_OUT1_DATA_B_P_T;
input  [7:0]	MEMORY_OUT1_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
output [7:0]	MEMORY_OUT1_DATA_C_P_O;
output [7:0]	MEMORY_OUT1_DATA_C_P_T;
input  [7:0]	MEMORY_OUT1_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
output [7:0]	MEMORY_OUT1_DATA_D_P_O;
output [7:0]	MEMORY_OUT1_DATA_D_P_T;

output [18:0]	MEMORY_OUT2_ADDR_P;		// external memory address bus
output 			MEMORY_OUT2_CLK_P;		// external memory clock
output 			MEMORY_OUT2_CLKEN_N;	// external memory clock enable
output 			MEMORY_OUT2_WEN_N;		// external memory read/write
output 			MEMORY_OUT2_WENA_N;		// external memory write enable for byte A
output 			MEMORY_OUT2_WENB_N;		// external memory write enable for byte B
output 			MEMORY_OUT2_WENC_N;		// external memory write enable for byte C
output 			MEMORY_OUT2_WEND_N;		// external memory write enable for byte D
output 			MEMORY_OUT2_OEN_N;		// external memory output enable
output 			MEMORY_OUT2_CEN_N;		// external memory chip enable
output 			MEMORY_OUT2_ADV_LD_N;	// external memory load/advance
input  [7:0]	MEMORY_OUT2_DATA_A_P_I;	//OUR input,  but O on the IOBUF at the system level
output [7:0]	MEMORY_OUT2_DATA_A_P_O;	//OUR output, but I on the IOBUF at the system level
output [7:0]	MEMORY_OUT2_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
input  [7:0]	MEMORY_OUT2_DATA_B_P_I;
output [7:0]	MEMORY_OUT2_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
output [7:0]	MEMORY_OUT2_DATA_B_P_T;
input  [7:0]	MEMORY_OUT2_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
output [7:0]	MEMORY_OUT2_DATA_C_P_O;
output [7:0]	MEMORY_OUT2_DATA_C_P_T;
input  [7:0]	MEMORY_OUT2_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
output [7:0]	MEMORY_OUT2_DATA_D_P_O;
output [7:0]	MEMORY_OUT2_DATA_D_P_T;

//holds either 0/1 to determine if IN->OUT goes straight through, or switched
reg buffer_reg;

//-----------------------------------------

always @ (posedge switch_pedge)
begin
	buffer_reg <= ~buffer_reg;
end

assign MEMORY_INA_DATA_A_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_A_P_I[7:0]	: MEMORY_OUT2_DATA_A_P_I[7:0];
assign MEMORY_INA_DATA_B_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_B_P_I[7:0]	: MEMORY_OUT2_DATA_B_P_I[7:0];
assign MEMORY_INA_DATA_C_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_C_P_I[7:0]	: MEMORY_OUT2_DATA_C_P_I[7:0];
assign MEMORY_INA_DATA_D_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_D_P_I[7:0]	: MEMORY_OUT2_DATA_D_P_I[7:0];

assign MEMORY_INB_DATA_A_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT2_DATA_A_P_I[7:0]	: MEMORY_OUT1_DATA_A_P_I[7:0];
assign MEMORY_INB_DATA_B_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT2_DATA_B_P_I[7:0]	: MEMORY_OUT1_DATA_B_P_I[7:0];
assign MEMORY_INB_DATA_C_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT2_DATA_C_P_I[7:0]	: MEMORY_OUT1_DATA_C_P_I[7:0];
assign MEMORY_INB_DATA_D_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT2_DATA_D_P_I[7:0]	: MEMORY_OUT1_DATA_D_P_I[7:0];

assign MEMORY_OUT1_ADDR_P[18:0]	= (buffer_reg) ? MEMORY_INA_ADDR_P[18:0] : MEMORY_INB_ADDR_P[18:0];
assign MEMORY_OUT1_CLK_P	= (buffer_reg) ? MEMORY_INA_CLK_P	: MEMORY_INB_CLK_P;
assign MEMORY_OUT1_CLKEN_N	= (buffer_reg) ? MEMORY_INA_CLKEN_N	: MEMORY_INB_CLKEN_N;
assign MEMORY_OUT1_WEN_N	= (buffer_reg) ? MEMORY_INA_WEN_N	: MEMORY_INB_WEN_N;
assign MEMORY_OUT1_WENA_N	= (buffer_reg) ? MEMORY_INA_WENA_N	: MEMORY_INB_WENA_N;
assign MEMORY_OUT1_WENB_N	= (buffer_reg) ? MEMORY_INA_WENB_N	: MEMORY_INB_WENB_N;
assign MEMORY_OUT1_WENC_N	= (buffer_reg) ? MEMORY_INA_WENC_N	: MEMORY_INB_WENC_N;
assign MEMORY_OUT1_WEND_N	= (buffer_reg) ? MEMORY_INA_WEND_N	: MEMORY_INB_WEND_N;
assign MEMORY_OUT1_OEN_N	= (buffer_reg) ? MEMORY_INA_OEN_N	: MEMORY_INB_OEN_N;
assign MEMORY_OUT1_CEN_N	= (buffer_reg) ? MEMORY_INA_CEN_N	: MEMORY_INB_CEN_N;
assign MEMORY_OUT1_ADV_LD_N	= (buffer_reg) ? MEMORY_INA_ADV_LD_N: MEMORY_INB_ADV_LD_N;
assign MEMORY_OUT1_DATA_A_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_A_P_O[7:0]	: MEMORY_INB_DATA_A_P_O[7:0];
assign MEMORY_OUT1_DATA_A_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_A_P_T[7:0]	: MEMORY_INB_DATA_A_P_T[7:0];
assign MEMORY_OUT1_DATA_B_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_B_P_O[7:0]	: MEMORY_INB_DATA_B_P_O[7:0];
assign MEMORY_OUT1_DATA_B_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_B_P_T[7:0]	: MEMORY_INB_DATA_B_P_T[7:0];
assign MEMORY_OUT1_DATA_C_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_C_P_O[7:0]	: MEMORY_INB_DATA_C_P_O[7:0];
assign MEMORY_OUT1_DATA_C_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_C_P_T[7:0]	: MEMORY_INB_DATA_C_P_T[7:0];
assign MEMORY_OUT1_DATA_D_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_D_P_O[7:0]	: MEMORY_INB_DATA_D_P_O[7:0];
assign MEMORY_OUT1_DATA_D_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_D_P_T[7:0]	: MEMORY_INB_DATA_D_P_T[7:0];

assign MEMORY_OUT2_ADDR_P[18:0]	= (buffer_reg) ? MEMORY_INB_ADDR_P[18:0] : MEMORY_INA_ADDR_P[18:0];
assign MEMORY_OUT2_CLK_P	= (buffer_reg) ? MEMORY_INB_CLK_P	: MEMORY_INA_CLK_P;
assign MEMORY_OUT2_CLKEN_N	= (buffer_reg) ? MEMORY_INB_CLKEN_N	: MEMORY_INA_CLKEN_N;
assign MEMORY_OUT2_WEN_N	= (buffer_reg) ? MEMORY_INB_WEN_N	: MEMORY_INA_WEN_N;
assign MEMORY_OUT2_WENA_N	= (buffer_reg) ? MEMORY_INB_WENA_N	: MEMORY_INA_WENA_N;
assign MEMORY_OUT2_WENB_N	= (buffer_reg) ? MEMORY_INB_WENB_N	: MEMORY_INA_WENB_N;
assign MEMORY_OUT2_WENC_N	= (buffer_reg) ? MEMORY_INB_WENC_N	: MEMORY_INA_WENC_N;
assign MEMORY_OUT2_WEND_N	= (buffer_reg) ? MEMORY_INB_WEND_N	: MEMORY_INA_WEND_N;
assign MEMORY_OUT2_OEN_N	= (buffer_reg) ? MEMORY_INB_OEN_N	: MEMORY_INA_OEN_N;
assign MEMORY_OUT2_CEN_N	= (buffer_reg) ? MEMORY_INB_CEN_N	: MEMORY_INA_CEN_N;
assign MEMORY_OUT2_ADV_LD_N	= (buffer_reg) ? MEMORY_INB_ADV_LD_N: MEMORY_INA_ADV_LD_N;
assign MEMORY_OUT2_DATA_A_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_A_P_O[7:0]	: MEMORY_INA_DATA_A_P_O[7:0];
assign MEMORY_OUT2_DATA_A_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_A_P_T[7:0]	: MEMORY_INA_DATA_A_P_T[7:0];
assign MEMORY_OUT2_DATA_B_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_B_P_O[7:0]	: MEMORY_INA_DATA_B_P_O[7:0];
assign MEMORY_OUT2_DATA_B_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_B_P_T[7:0]	: MEMORY_INA_DATA_B_P_T[7:0];
assign MEMORY_OUT2_DATA_C_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_C_P_O[7:0]	: MEMORY_INA_DATA_C_P_O[7:0];
assign MEMORY_OUT2_DATA_C_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_C_P_T[7:0]	: MEMORY_INA_DATA_C_P_T[7:0];
assign MEMORY_OUT2_DATA_D_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_D_P_O[7:0]	: MEMORY_INA_DATA_D_P_O[7:0];
assign MEMORY_OUT2_DATA_D_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_D_P_T[7:0]	: MEMORY_INA_DATA_D_P_T[7:0];

endmodule
