// Provides a buffer switcher for two sets on ZBT memory control inputs.
//   They can either go straight through (INA->OUT1 / INB->OUT2),
//   or after switch_pedge is trigger (INA->OUT2 / INB->OUT1).
//
// Timothy Li, 2005-03-13

module zbt_buffer_mux
(
//switching signals
  user_access_ok,
  buffer_counter,
  buffer_counter_out,
//input A
  MEMORY_INA_CLK_P,
  MEMORY_INA_CLKEN_N,
  MEMORY_INA_WEN_N,
  MEMORY_INA_WENA_N,
  MEMORY_INA_WENB_N,
  MEMORY_INA_WENC_N,
  MEMORY_INA_WEND_N,
  MEMORY_INA_ADV_LD_N,
  MEMORY_INA_OEN_N,
  MEMORY_INA_CEN_N,
  MEMORY_INA_ADDR_P,
  MEMORY_INA_DATA_A_P_I,
  MEMORY_INA_DATA_A_P_O,
  MEMORY_INA_DATA_A_P_T,
  MEMORY_INA_DATA_B_P_I,
  MEMORY_INA_DATA_B_P_O,
  MEMORY_INA_DATA_B_P_T,
  MEMORY_INA_DATA_C_P_I,
  MEMORY_INA_DATA_C_P_O,
  MEMORY_INA_DATA_C_P_T,
  MEMORY_INA_DATA_D_P_I,
  MEMORY_INA_DATA_D_P_O,
  MEMORY_INA_DATA_D_P_T,
//input B
  MEMORY_INB_CLK_P,
  MEMORY_INB_CLKEN_N,
  MEMORY_INB_WEN_N,
  MEMORY_INB_BEN_N,		//emc has this as a vector
  MEMORY_INB_ADV_LD_N,
  MEMORY_INB_OEN_N,
  MEMORY_INB_CEN_N,
  MEMORY_INB_ADDR_P,
  MEMORY_INB_DATA_P_I,	//emc groups all data bits into one stream
  MEMORY_INB_DATA_P_O,
  MEMORY_INB_DATA_P_T,
//output to first bank
  MEMORY_OUT1_CLK_P,
  MEMORY_OUT1_CLKEN_N,
  MEMORY_OUT1_WEN_N,
  MEMORY_OUT1_WENA_N,
  MEMORY_OUT1_WENB_N,
  MEMORY_OUT1_WENC_N,
  MEMORY_OUT1_WEND_N,
  MEMORY_OUT1_ADV_LD_N,
  MEMORY_OUT1_OEN_N,
  MEMORY_OUT1_CEN_N,
  MEMORY_OUT1_ADDR_P,
  MEMORY_OUT1_DATA_A_P_I,
  MEMORY_OUT1_DATA_A_P_O,
  MEMORY_OUT1_DATA_A_P_T,
  MEMORY_OUT1_DATA_B_P_I,
  MEMORY_OUT1_DATA_B_P_O,
  MEMORY_OUT1_DATA_B_P_T,
  MEMORY_OUT1_DATA_C_P_I,
  MEMORY_OUT1_DATA_C_P_O,
  MEMORY_OUT1_DATA_C_P_T,
  MEMORY_OUT1_DATA_D_P_I,
  MEMORY_OUT1_DATA_D_P_O,
  MEMORY_OUT1_DATA_D_P_T,
//output to second bank
  MEMORY_OUT2_CLK_P,
  MEMORY_OUT2_CLKEN_N,
  MEMORY_OUT2_WEN_N,
  MEMORY_OUT2_WENA_N,
  MEMORY_OUT2_WENB_N,
  MEMORY_OUT2_WENC_N,
  MEMORY_OUT2_WEND_N,
  MEMORY_OUT2_ADV_LD_N,
  MEMORY_OUT2_OEN_N,
  MEMORY_OUT2_CEN_N,
  MEMORY_OUT2_ADDR_P,
  MEMORY_OUT2_DATA_A_P_I,
  MEMORY_OUT2_DATA_A_P_O,
  MEMORY_OUT2_DATA_A_P_T,
  MEMORY_OUT2_DATA_B_P_I,
  MEMORY_OUT2_DATA_B_P_O,
  MEMORY_OUT2_DATA_B_P_T,
  MEMORY_OUT2_DATA_C_P_I,
  MEMORY_OUT2_DATA_C_P_O,
  MEMORY_OUT2_DATA_C_P_T,
  MEMORY_OUT2_DATA_D_P_I,
  MEMORY_OUT2_DATA_D_P_O,
  MEMORY_OUT2_DATA_D_P_T
);

`define COUNTER_BITS 1
`define BUFFERS_COUNT 2

input 						user_access_ok;		//can switch buffers when HI (vertical blanking interval)
input  [(`COUNTER_BITS-1):0]	buffer_counter;		//allows external setting of buffer rotation stage
output [(`COUNTER_BITS-1):0]	buffer_counter_out;	//allows external read of current buffer rotation stage

input  [18:0]	MEMORY_INA_ADDR_P;		// external memory address bus
input 			MEMORY_INA_CLK_P;		// external memory clock
input 			MEMORY_INA_CLKEN_N;		// external memory clock enable
input 			MEMORY_INA_WEN_N;		// external memory read/write
input 			MEMORY_INA_WENA_N;		// external memory write enable for byte A
input 			MEMORY_INA_WENB_N;		// external memory write enable for byte B
input 			MEMORY_INA_WENC_N;		// external memory write enable for byte C
input 			MEMORY_INA_WEND_N;		// external memory write enable for byte D
input 			MEMORY_INA_OEN_N;		// external memory input enable
input 			MEMORY_INA_CEN_N;		// external memory chip enable
input 			MEMORY_INA_ADV_LD_N;	// external memory load/advance
output [7:0]	MEMORY_INA_DATA_A_P_I;	// external memory byte A bits [ 7: 0] BLUE DATA
input  [7:0]	MEMORY_INA_DATA_A_P_O;
input  [7:0]	MEMORY_INA_DATA_A_P_T;
output [7:0]	MEMORY_INA_DATA_B_P_I;	// external memory byte B bits [15: 8] GREEN DATA
input  [7:0]	MEMORY_INA_DATA_B_P_O;
input  [7:0]	MEMORY_INA_DATA_B_P_T;
output [7:0]	MEMORY_INA_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
input  [7:0]	MEMORY_INA_DATA_C_P_O;
input  [7:0]	MEMORY_INA_DATA_C_P_T;
output [7:0]	MEMORY_INA_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
input  [7:0]	MEMORY_INA_DATA_D_P_O;
input  [7:0]	MEMORY_INA_DATA_D_P_T;

input  [0:18]	MEMORY_INB_ADDR_P;		// external memory address bus
input 			MEMORY_INB_CLK_P;		// external memory clock
input 			MEMORY_INB_CLKEN_N;		// external memory clock enable
input 			MEMORY_INB_WEN_N;		// external memory read/write
input  [0:3]	MEMORY_INB_BEN_N;		// external memory write enable for byte A
input 			MEMORY_INB_OEN_N;		// external memory input enable
input 			MEMORY_INB_CEN_N;		// external memory chip enable
input 			MEMORY_INB_ADV_LD_N;	// external memory load/advance
output [0:31]	MEMORY_INB_DATA_P_I;	//
input  [0:31]	MEMORY_INB_DATA_P_O;	// external memory byte bits
input  [0:31]	MEMORY_INB_DATA_P_T;	// 

output [18:0]	MEMORY_OUT1_ADDR_P;		// external memory address bus
output 			MEMORY_OUT1_CLK_P;		// external memory clock
output 			MEMORY_OUT1_CLKEN_N;	// external memory clock enable
output 			MEMORY_OUT1_WEN_N;		// external memory read/write
output 			MEMORY_OUT1_WENA_N;		// external memory write enable for byte A
output 			MEMORY_OUT1_WENB_N;		// external memory write enable for byte B
output 			MEMORY_OUT1_WENC_N;		// external memory write enable for byte C
output 			MEMORY_OUT1_WEND_N;		// external memory write enable for byte D
output 			MEMORY_OUT1_OEN_N;		// external memory output enable
output 			MEMORY_OUT1_CEN_N;		// external memory chip enable
output 			MEMORY_OUT1_ADV_LD_N;	// external memory load/advance
input  [7:0]	MEMORY_OUT1_DATA_A_P_I;	//OUR input,  but O from the IOBUF at the system level
output [7:0]	MEMORY_OUT1_DATA_A_P_O;	//OUR output, but I to   the IOBUF at the system level
output [7:0]	MEMORY_OUT1_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
input  [7:0]	MEMORY_OUT1_DATA_B_P_I;
output [7:0]	MEMORY_OUT1_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
output [7:0]	MEMORY_OUT1_DATA_B_P_T;
input  [7:0]	MEMORY_OUT1_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
output [7:0]	MEMORY_OUT1_DATA_C_P_O;
output [7:0]	MEMORY_OUT1_DATA_C_P_T;
input  [7:0]	MEMORY_OUT1_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
output [7:0]	MEMORY_OUT1_DATA_D_P_O;
output [7:0]	MEMORY_OUT1_DATA_D_P_T;

output [18:0]	MEMORY_OUT2_ADDR_P;		// external memory address bus
output 			MEMORY_OUT2_CLK_P;		// external memory clock
output 			MEMORY_OUT2_CLKEN_N;	// external memory clock enable
output 			MEMORY_OUT2_WEN_N;		// external memory read/write
output 			MEMORY_OUT2_WENA_N;		// external memory write enable for byte A
output 			MEMORY_OUT2_WENB_N;		// external memory write enable for byte B
output 			MEMORY_OUT2_WENC_N;		// external memory write enable for byte C
output 			MEMORY_OUT2_WEND_N;		// external memory write enable for byte D
output 			MEMORY_OUT2_OEN_N;		// external memory output enable
output 			MEMORY_OUT2_CEN_N;		// external memory chip enable
output 			MEMORY_OUT2_ADV_LD_N;	// external memory load/advance
input  [7:0]	MEMORY_OUT2_DATA_A_P_I;	//OUR input,  but O from the IOBUF at the system level
output [7:0]	MEMORY_OUT2_DATA_A_P_O;	//OUR output, but I to   the IOBUF at the system level
output [7:0]	MEMORY_OUT2_DATA_A_P_T;	// external memory byte A bits [ 7: 0] BLUE DATA
input  [7:0]	MEMORY_OUT2_DATA_B_P_I;
output [7:0]	MEMORY_OUT2_DATA_B_P_O;	// external memory byte B bits [15: 8] GREEN DATA
output [7:0]	MEMORY_OUT2_DATA_B_P_T;
input  [7:0]	MEMORY_OUT2_DATA_C_P_I;	// external memory byte C bits [23:16] RED DATA
output [7:0]	MEMORY_OUT2_DATA_C_P_O;
output [7:0]	MEMORY_OUT2_DATA_C_P_T;
input  [7:0]	MEMORY_OUT2_DATA_D_P_I;	// external memory byte D bits [31:24] MISC DATA
output [7:0]	MEMORY_OUT2_DATA_D_P_O;
output [7:0]	MEMORY_OUT2_DATA_D_P_T;

//holds either 0/1 to determine if IN->OUT goes straight through, or switched
reg 						buffer_reg;
reg [(`COUNTER_BITS-1):0]	buffer_counter_prev;

//-----------------------------------------

always @ (posedge user_access_ok)
begin
	if (buffer_counter != buffer_counter_prev)
	begin
		buffer_reg <= ~buffer_reg;
		buffer_counter_prev <= buffer_counter;
	end
end

assign buffer_counter_out = buffer_counter_prev;

assign MEMORY_INA_DATA_A_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_A_P_I[7:0]	: MEMORY_OUT2_DATA_A_P_I[7:0];
assign MEMORY_INA_DATA_B_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_B_P_I[7:0]	: MEMORY_OUT2_DATA_B_P_I[7:0];
assign MEMORY_INA_DATA_C_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_C_P_I[7:0]	: MEMORY_OUT2_DATA_C_P_I[7:0];
assign MEMORY_INA_DATA_D_P_I[7:0]	=  (buffer_reg) ? MEMORY_OUT1_DATA_D_P_I[7:0]	: MEMORY_OUT2_DATA_D_P_I[7:0];

assign MEMORY_INB_DATA_P_I[24:31]	=  (buffer_reg) ? MEMORY_OUT2_DATA_A_P_I[7:0]	: MEMORY_OUT1_DATA_A_P_I[7:0];
assign MEMORY_INB_DATA_P_I[16:23]	=  (buffer_reg) ? MEMORY_OUT2_DATA_B_P_I[7:0]	: MEMORY_OUT1_DATA_B_P_I[7:0];
assign MEMORY_INB_DATA_P_I[8:15]	=  (buffer_reg) ? MEMORY_OUT2_DATA_C_P_I[7:0]	: MEMORY_OUT1_DATA_C_P_I[7:0];
assign MEMORY_INB_DATA_P_I[0:7] 	=  (buffer_reg) ? MEMORY_OUT2_DATA_D_P_I[7:0]	: MEMORY_OUT1_DATA_D_P_I[7:0];

assign MEMORY_OUT1_ADDR_P[18:0]	= (buffer_reg) ? MEMORY_INA_ADDR_P[18:0] : MEMORY_INB_ADDR_P[0:18];	//OR [11:29]?
assign MEMORY_OUT1_CLK_P	= (buffer_reg) ? MEMORY_INA_CLK_P	: MEMORY_INB_CLK_P;
assign MEMORY_OUT1_CLKEN_N	= (buffer_reg) ? MEMORY_INA_CLKEN_N	: MEMORY_INB_CLKEN_N;
assign MEMORY_OUT1_WEN_N	= (buffer_reg) ? MEMORY_INA_WEN_N	: MEMORY_INB_WEN_N;
assign MEMORY_OUT1_WENA_N	= (buffer_reg) ? MEMORY_INA_WENA_N	: MEMORY_INB_BEN_N[3];
assign MEMORY_OUT1_WENB_N	= (buffer_reg) ? MEMORY_INA_WENB_N	: MEMORY_INB_BEN_N[2];
assign MEMORY_OUT1_WENC_N	= (buffer_reg) ? MEMORY_INA_WENC_N	: MEMORY_INB_BEN_N[1];
assign MEMORY_OUT1_WEND_N	= (buffer_reg) ? MEMORY_INA_WEND_N	: MEMORY_INB_BEN_N[0];
assign MEMORY_OUT1_OEN_N	= (buffer_reg) ? MEMORY_INA_OEN_N	: MEMORY_INB_OEN_N;
assign MEMORY_OUT1_CEN_N	= (buffer_reg) ? MEMORY_INA_CEN_N	: MEMORY_INB_CEN_N;
assign MEMORY_OUT1_ADV_LD_N	= (buffer_reg) ? MEMORY_INA_ADV_LD_N: MEMORY_INB_ADV_LD_N;
assign MEMORY_OUT1_DATA_A_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_A_P_O[7:0]	: MEMORY_INB_DATA_P_O[24:31];
assign MEMORY_OUT1_DATA_A_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_A_P_T[7:0]	: MEMORY_INB_DATA_P_T[24:31];
assign MEMORY_OUT1_DATA_B_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_B_P_O[7:0]	: MEMORY_INB_DATA_P_O[16:23];
assign MEMORY_OUT1_DATA_B_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_B_P_T[7:0]	: MEMORY_INB_DATA_P_T[16:23];
assign MEMORY_OUT1_DATA_C_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_C_P_O[7:0]	: MEMORY_INB_DATA_P_O[8:15];
assign MEMORY_OUT1_DATA_C_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_C_P_T[7:0]	: MEMORY_INB_DATA_P_T[8:15];
assign MEMORY_OUT1_DATA_D_P_O[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_D_P_O[7:0]	: MEMORY_INB_DATA_P_O[0:7];
assign MEMORY_OUT1_DATA_D_P_T[7:0]	= (buffer_reg) ? MEMORY_INA_DATA_D_P_T[7:0]	: MEMORY_INB_DATA_P_T[0:7];

assign MEMORY_OUT2_ADDR_P[18:0]	= (buffer_reg) ? MEMORY_INB_ADDR_P[0:18] : MEMORY_INA_ADDR_P[18:0];
assign MEMORY_OUT2_CLK_P	= (buffer_reg) ? MEMORY_INB_CLK_P	: MEMORY_INA_CLK_P;
assign MEMORY_OUT2_CLKEN_N	= (buffer_reg) ? MEMORY_INB_CLKEN_N	: MEMORY_INA_CLKEN_N;
assign MEMORY_OUT2_WEN_N	= (buffer_reg) ? MEMORY_INB_WEN_N	: MEMORY_INA_WEN_N;
assign MEMORY_OUT2_WENA_N	= (buffer_reg) ? MEMORY_INB_BEN_N[3]: MEMORY_INA_WENA_N;
assign MEMORY_OUT2_WENB_N	= (buffer_reg) ? MEMORY_INB_BEN_N[2]: MEMORY_INA_WENB_N;
assign MEMORY_OUT2_WENC_N	= (buffer_reg) ? MEMORY_INB_BEN_N[1]: MEMORY_INA_WENC_N;
assign MEMORY_OUT2_WEND_N	= (buffer_reg) ? MEMORY_INB_BEN_N[0]: MEMORY_INA_WEND_N;
assign MEMORY_OUT2_OEN_N	= (buffer_reg) ? MEMORY_INB_OEN_N	: MEMORY_INA_OEN_N;
assign MEMORY_OUT2_CEN_N	= (buffer_reg) ? MEMORY_INB_CEN_N	: MEMORY_INA_CEN_N;
assign MEMORY_OUT2_ADV_LD_N	= (buffer_reg) ? MEMORY_INB_ADV_LD_N: MEMORY_INA_ADV_LD_N;
assign MEMORY_OUT2_DATA_A_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_O[24:31]	: MEMORY_INA_DATA_A_P_O[7:0];
assign MEMORY_OUT2_DATA_A_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_T[24:31]	: MEMORY_INA_DATA_A_P_T[7:0];
assign MEMORY_OUT2_DATA_B_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_O[16:23]	: MEMORY_INA_DATA_B_P_O[7:0];
assign MEMORY_OUT2_DATA_B_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_T[16:23]	: MEMORY_INA_DATA_B_P_T[7:0];
assign MEMORY_OUT2_DATA_C_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_O[8:15]	: MEMORY_INA_DATA_C_P_O[7:0];
assign MEMORY_OUT2_DATA_C_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_T[8:15]	: MEMORY_INA_DATA_C_P_T[7:0];
assign MEMORY_OUT2_DATA_D_P_O[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_O[0:7]	: MEMORY_INA_DATA_D_P_O[7:0];
assign MEMORY_OUT2_DATA_D_P_T[7:0]	= (buffer_reg) ? MEMORY_INB_DATA_P_T[0:7]	: MEMORY_INA_DATA_D_P_T[7:0];

endmodule
