module tvin2 (
  // inputs
  clk,
  rst,
  VIDEO_INPUT,
  led0_o,
  led1_o, // for debug
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P

);

// NOTE: Changing the number of states will probably require changing how 
// Hmid_prev is used
parameter  st_reset=0;  
parameter  st_ctrl=1;
parameter  st_ctrl2=2;
parameter  st_addr=3;
parameter  st_addr2=4;
parameter  st_data=5;
parameter  st_data2=6; // Because pixels come in at 1/2 clk rate (422->444)
//`define control_word 32'h00028005 //Used by the SIMPPL protocol
//`define control_word 32'h00500000 //Used in Nathalie's protocol
`define line_length 10'd640 
//`define control_word 32'h005a0000
//`define line_length 10'd720

input clk;
input rst;
input [7:0] VIDEO_INPUT; // Connect the MSB pins
output led0_o, led1_o; // for debug

output 			MEMORY_CLK_P;		// ZBT RAM clock
output 			MEMORY_CLKEN_N;	// ZBT RAM clock enable
output 			MEMORY_WEN_N;		// ZBT RAM write enable
output 			MEMORY_WENA_N;	// ZBT RAM byte A write enable
output 			MEMORY_WENB_N;	// ZBT RAM byte B write enable
output 			MEMORY_WENC_N;	// ZBT RAM byte C write enable
output 			MEMORY_WEND_N;	// ZBT RAM byte D write enable
output 			MEMORY_ADV_LD_N;	// ZBT RAM address load-advance
output 			MEMORY_OEN_N;		// ZBT RAM output enable
output 			MEMORY_CEN_N;		// ZBT RAM chip enable
output [18:0]MEMORY_ADDR_P;// ZBT RAM address bus
inout  [7:0] 		MEMORY_DATA_A_P;	// ZBT RAM data bus byte "A"
inout  [7:0] 		MEMORY_DATA_B_P;	// ZBT RAM data bus byte "B"
inout  [7:0] 		MEMORY_DATA_C_P;	// ZBT RAM data bus byte "C"
inout  [7:0] 		MEMORY_DATA_D_P;	// ZBT RAM data bus byte "D"

wire led0_o, led1_o;

wire user_memory_clock;
wire user_memory_ram_clock;
wire user_memory_clocken;
wire user_memory_load_addr;
wire user_memory_read;
reg user_memory_write;
wire user_memory_wea;
wire user_memory_web;
wire user_memory_wec;
wire user_memory_wed;
wire user_memory_ce;
reg [18:0]user_memory_addr;
wire [31:0]user_memory_read_data;
reg [31:0]user_memory_write_data;

//  clk, rst, // rst should be driven by FSL_Rst
//  VIDEO_INPUT,

wire [7:0] R_data;
wire [7:0] G_data;
wire [7:0] B_data;
wire Fmid;
wire Vmid;
wire Hmid;

reg [18:0] counter;
wire [31:0] rgbdata;

//wire Fmid, Vmid, Hmid;
reg [5:0] Hmid_prev;
reg [2:0] pres_state, next_state;
reg [9:0] pixel_ct; // for debugging


assign led0_o = 1;
assign led1_o = 0;
  
assign user_memory_clock = clk;
assign user_memory_ram_clock = clk;
assign user_memory_clocken = 1;
assign user_memory_load_addr = 1;
assign user_memory_read = 0;
//assign user_memory_write = 0;
assign user_memory_wea = 1;
assign user_memory_web = 1;
assign user_memory_wec = 1;
assign user_memory_wed = 1;
assign user_memory_ce = 1;
//assign user_memory_addr
//assign user_memory_read_data
//assign user_memory_write_data

assign rgbdata[31:24] = 8'b0;
assign rgbdata[23:16] = R_data;
assign rgbdata[15:8] = G_data;
assign rgbdata[7:0] = B_data;

/*always @ (posedge FSL_S_Clk) begin
	if (FSL_S_Exists == 1) begin
		FSL_S_Control <= 1;
		FSL_S_Read <= 1;
		//FSL_S_Data <= RGBdata;
	end
	else begin
		FSL_S_Control <= 0;
		FSL_S_Read <= 0;
	end
end*/

/*always @ (posedge clk) begin
	if (rst) begin
		counter <= 0;
	end
	user_memory_write <= 1;
	user_memory_addr <= counter;
	user_memory_write_data <= rgbdata;
	if (counter < 307200) begin
		counter <= counter + 1;
	end
	else begin
		counter <= 0;
	end
end*/


// Use Horizontal blanking signal to control state changes
// Also register RGB values to give delay
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1) begin
    Hmid_prev <= 6'b000000;
  end
  else begin
    Hmid_prev[5:1] <= Hmid_prev[4:0];
    Hmid_prev[0] <= Hmid;
  end
end
/*
// Determine next state
always @ (pres_state or rst or Vmid or Hmid_prev or Hmid or pixel_ct) begin
  case (pres_state) 
    st_reset:
	   // Don't attempt to write out this line unless we know for sure
	   // we have space to buffer it. i.e. the svga-line-sized buffer is 
	   // empty
        if (Vmid == 1'b0 && Hmid_prev[0] == 1'b1 && Hmid == 1'b0)  
	         next_state <= st_ctrl;
        else
	         next_state <= st_reset;
    st_ctrl:
	   next_state <= st_ctrl2;
    st_ctrl2:
        next_state <= st_addr;
    st_addr:
    	   next_state <= st_addr2;
    st_addr2:
        next_state <= st_data;
    st_data:
//      begin
//        if ( Hmid_prev[4] == 1'b0 && Hmid_prev[3] == 1'b1)
//	        next_state <= st_reset;
//        else
        next_state <= st_data2;
//	end
    st_data2:
//      if ( Hmid_prev[4] == 1'b0 && Hmid_prev[3] == 1'b1)
        if (pixel_ct == 640)//`line_length) // Camera must be giving wrong number of words or dropping TRS's
            	next_state <= st_reset;
        else
        	next_state <= st_data;
  endcase
end
  */
// FSM outputs
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1) begin
    //FSL_M_Dataval <= 32'h0;
    //FSL_M_Controlval <= 1'b0;
    ////release_priority <= 1'b1;
    //fifo_write <= 1'b0;
    pixel_ct <= 10'd0;
    user_memory_write <= 0;
  end
  else begin
    case (pres_state) 
	  st_reset:
        begin
          //FSL_M_Dataval <= 32'h0;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b1;
	  //fifo_write <= 1'b0;
	     pixel_ct <= 10'd0;
		user_memory_write <= 0;
	   end
      st_ctrl:
        begin
          //FSL_M_Dataval <= `control_word;
          //FSL_M_Controlval <= 1'b1;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	     pixel_ct <= pixel_ct;
		user_memory_write <= 1;
		user_memory_addr <= pixel_ct;
		user_memory_write_data <= rgbdata;
        end
      st_ctrl2:
        begin
          //FSL_M_Dataval <= FSL_M_Dataval;
          //FSL_M_Controlval <= 1'b1;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	     pixel_ct <= pixel_ct;
		user_memory_write <= 0;
        end
      st_addr:
        begin
          //FSL_M_Dataval <= {13'h0,Addr};
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	     pixel_ct <= pixel_ct;
		user_memory_write <= 1;
		user_memory_addr <= pixel_ct;
		user_memory_write_data <= rgbdata;
        end
      st_addr2:
        begin
          //FSL_M_Dataval <= FSL_M_Dataval;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	     pixel_ct <= pixel_ct;
		user_memory_write <= 0;
        end
      st_data:
        begin
          //FSL_M_Dataval <= RGBdata;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	     pixel_ct <= pixel_ct + 1;
		user_memory_write <= 1;
		user_memory_addr <= pixel_ct;
		user_memory_write_data <= rgbdata;
        end
      st_data2:
        begin
          //FSL_M_Dataval <= RGBdata;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	     pixel_ct <= pixel_ct;
		user_memory_write <= 0;
        end
    endcase
  end
end

MEMORY_CTRL_532 MEMORY_CTRL_532(
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P,

user_memory_clock,
user_memory_ram_clock,
user_memory_clocken,
user_memory_load_addr,
user_memory_read,
user_memory_write,
user_memory_wea,
user_memory_web,
user_memory_wec,
user_memory_wed,
user_memory_ce,
user_memory_addr,
user_memory_read_data,
user_memory_write_data
);

video_input video_input(
  // inputs
  clk, rst, // rst should be driven by FSL_Rst
  VIDEO_INPUT,
  
  R_data, 
  G_data, 
  B_data, 
  Fmid,
  Vmid,
  Hmid
);


endmodule