//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: fsl_video_input.v
//    Uses: coregen_fifo.xco (and .edn, but .xco goes in the project & ISE
//            will look for .edn during compilation)
//          addr_gen.v
//          input2rgb.v
//    Used by: top_dualzbt_dow.v
//    Works with ProjectNavigator 6.2.03i
//    Description: Wraps video input processing modules and provides an FSL
//    interface.  Requires the line-length FIFO to avoid losing data and
//    clogging up the FSL.  
//
//    Protocol for communication is:
//      control word:  upper 19 bits give # of words to write out
//                     20th bit is read/write_n
//      address word:  upper 19 bits give address (19 bits to address into
//                      a ZBT)
//      data:  must output exactly the number of data words you specified
//              you would in the control word, otherwise the FSLs get clogged
//
//    NCKC August 2004
//************************************************************************ 
//
 

module video_input (
  // inputs
  clk, rst, // rst should be driven by FSL_Rst
  VIDEO_INPUT,
  
  //added to "new" video input
  //FSL_M_Full,
  //fifo_read,
  //fifo_empty, 
  //DatatoFSL,

  // outputs
  //release_priority,
  //FSL_M_Clk,
  //FSL_M_Data,
  //FSL_M_Control,
  //FSL_M_Write,
  //led0_o, led1_o, // for debug
  //batch

  R_data, 
  G_data, 
  B_data, 
  Fmid,
  Vmid,
  Hmid
  );

//coregen_fifo coregen_fifo (
	//.clk(clk),
	//.sinit(rst),
	//.din(DataIn),
	//.wr_en(fifo_write),
	//.full(fifo_full),

  input clk, rst;
  input [7:0] VIDEO_INPUT; // Connect the MSB pins
  //input FSL_M_Full;
  //input fifo_read; 
  //output fifo_empty;
  //output [0:32] DatatoFSL; 
  //output release_priority; // Could be used to demand priority servicing. 1 => give up priority, 0 => demand priority
  //output FSL_M_Clk, 
 // output FSL_M_Control// , FSL_M_Write;
//  output [31:0] FSL_M_Data;
  //output [0:31] FSL_M_Data;
  //output batch;  // 1 => Still on the same frame
  //output led0_o, led1_o; // for debug
  output [7:0] R_data;
  output [7:0] G_data;
  output [7:0] B_data;
  output Fmid, Vmid, Hmid;
	  
  //wire clk, rst;
  //wire [7:0] VIDEO_INPUT;
  //wire [7:0] R,G,B;
  //wire release_priority;  // Do not use
  //wire FSL_M_Clk, 
  //wire FSL_M_Full;
  //wire FSL_M_Control;
  //reg FSL_M_Write, FSL_M_Writed;
  //wire [0:31] FSL_M_Data;
//  wire [31:0] FSL_M_Data;
  //reg batch;

  //reg FSL_M_Controlval; //FSL_M_Writeval;
  //reg [31:0] FSL_M_Dataval;
//  reg [0:23] FSL_M_Dataval;

  //wire Fmid, Vmid, Hmid;
  //wire [18:0] Addr;
  //reg [18:0] Addr_prev;
  //wire [31:0] RGBdata;
//  wire [23:0] RGBdata;

  //reg [2:0] pres_state, next_state;
  //reg [5:0] Hmid_prev;

  // For internal FIFO
  //reg fifo_write; //, fifo_read;
  //wire fifo_exists, fifo_full;
  //wire [24:0] DataIn, DatatoFSL; // Is this correct endianness?
  //wire [32:0] DataIn; //, DatatoFSL; // Is this correct endianness?
  //reg wrote_last;  // == 1 if wrote in last cycle, otherwise didn't write 
  
  //reg [9:0] pixel_ct; // for debugging

  //reg led0_o, led1_o; // for debug  
//  reg [31:0] RGBdata_reg;

//// For Coregen FIFO
//wire fifo_empty;
//assign fifo_exists = ~fifo_empty;


// NOTE: Changing the number of states will probably require changing how 
// Hmid_prev is used
//parameter  st_reset=0;  
//parameter  st_ctrl=1;
//parameter  st_ctrl2=2;
//parameter  st_addr=3;
//parameter  st_addr2=4;
//parameter  st_data=5;
//parameter  st_data2=6; // Because pixels come in at 1/2 clk rate (422->444)

//`define control_word 32'h00028005 //Used by the SIMPPL protocol
//`define control_word 32'h00500000 //Used in Nathalie's protocol
//`define line_length 10'd640 
//`define control_word 32'h005a0000
//`define line_length 10'd720

//assign release_priority = 1'b1;

////  assign FSL_M_Data[8:31] = DatatoFSL[23:0];
////  assign FSL_M_Data[0:7] = 8'h00;
////  assign FSL_M_Control = DatatoFSL[24];
////  assign DataIn[24] = FSL_M_Controlval;
////  assign DataIn[23:0] = FSL_M_Dataval;
  //assign FSL_M_Data = DatatoFSL[31:0];
  //assign FSL_M_Control = DatatoFSL[32];
  //assign DataIn[32] = FSL_M_Controlval;
  //assign DataIn[31:0] = FSL_M_Dataval;
  
    
//assign FSL_M_Clk = clk;
//assign RGBdata[31:24] = 8'b0;
//assign R = RGBdata[23:16];  
//assign G = RGBdata[15:8];  
//assign B = RGBdata[7:0];  

// Use Horizontal blanking signal to control state changes
// Also register RGB values to give delay
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) begin
    //Hmid_prev <= 6'b000000;
////    RGBdata_reg <= 32'h0;
  //end
  //else begin
    //Hmid_prev[5:1] <= Hmid_prev[4:0];
    //Hmid_prev[0] <= Hmid;
////    RGBdata_reg <= RGBdata;
  //end
//end

//// FSM register
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //pres_state <= st_reset;
  //else
    //pres_state <= next_state;
//end

//// Determine next state
//always @ (pres_state or rst or Vmid or Hmid_prev or Hmid or fifo_exists or pixel_ct) begin
  //case (pres_state) 
    //st_reset:
      //begin 
	  //// Don't attempt to write out this line unless we know for sure
	  //// we have space to buffer it. i.e. the svga-line-sized buffer is 
	  //// empty
        //if (Vmid == 1'b0 && Hmid_prev[0] == 1'b1 && Hmid == 1'b0 && fifo_exists == 1'b0)  
	         //next_state <= st_ctrl;
        //else
	         //next_state <= st_reset;
      //end
    //st_ctrl:
        //next_state <= st_ctrl2;
    //st_ctrl2:
       	//next_state <= st_addr;
    //st_addr:
	//next_state <= st_addr2;
    //st_addr2:
        //next_state <= st_data;
    //st_data:
////      begin
////        if ( Hmid_prev[4] == 1'b0 && Hmid_prev[3] == 1'b1)
////	        next_state <= st_reset;
////        else
	        //next_state <= st_data2;
////	end
    //st_data2:
////      if ( Hmid_prev[4] == 1'b0 && Hmid_prev[3] == 1'b1)
        //if (pixel_ct == `line_length) // Camera must be giving wrong number of words or dropping TRS's
            	//next_state <= st_reset;
        //else
        	//next_state <= st_data;
//
  //endcase
//end

//// FSM outputs
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) begin
    //FSL_M_Dataval <= 32'h0;
    //FSL_M_Controlval <= 1'b0;
    ////release_priority <= 1'b1;
    //fifo_write <= 1'b0;
    //pixel_ct <= 10'd0;
  //end
  //else begin
    //case (pres_state) 
	  //st_reset:
        //begin
          //FSL_M_Dataval <= 32'h0;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b1;
	  //fifo_write <= 1'b0;
	  //pixel_ct <= 10'd0;
	     //end
      //st_ctrl:
        //begin
          //FSL_M_Dataval <= `control_word;
          //FSL_M_Controlval <= 1'b1;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	  //pixel_ct <= pixel_ct;
        //end
      //st_ctrl2:
        //begin
          //FSL_M_Dataval <= FSL_M_Dataval;
          //FSL_M_Controlval <= 1'b1;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	  //pixel_ct <= pixel_ct;
        //end
      //st_addr:
        //begin
          //FSL_M_Dataval <= {13'h0,Addr};
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	  //pixel_ct <= pixel_ct;
        //end
      //st_addr2:
        //begin
          //FSL_M_Dataval <= FSL_M_Dataval;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	  //pixel_ct <= pixel_ct;
        //end
      //st_data:
        //begin
          //FSL_M_Dataval <= RGBdata;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b1;
	  //pixel_ct <= pixel_ct + 1;
       //end
      //st_data2:
        //begin
          //FSL_M_Dataval <= RGBdata;
          //FSL_M_Controlval <= 1'b0;
	  ////release_priority <= 1'b0;
	  //fifo_write <= 1'b0;
	  //pixel_ct <= pixel_ct;
       //end
//
    //endcase
  //end
//end

//// for debug
//// Light an LED if we go back to reset before we've written pixels 0-719
//// (or is it 1-720?)
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //led0_o <= 1'b1; // off
  //else
    //if ((pixel_ct < (`line_length - 1)) && (pres_state != st_reset) && (next_state == st_reset))
      //led0_o <= 1'b0; //bad
    //else
      //led0_o <= led0_o;
//end
//
//// Light an LED if we write to a full fifo
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //led1_o <= 1'b1; // off
  //else
    //if (fifo_full == 1'b1)
      //led1_o <= 1'b0; //bad
    //else
      //led1_o <= led1_o;
//end

//// Connect internal FIFO to FSL
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) begin
    //fifo_read <= 1'b0;
    //FSL_M_Writed <= 1'b0;
    //wrote_last <= 1'b0;
  //end
  //else begin
    //if ((wrote_last == 1'b0) && (fifo_exists == 1'b1) && (FSL_M_Full == 1'b0)) begin
      //// Write to FSL
      //fifo_read <= 1'b1;
      //FSL_M_Writed <= 1'b1;
      //wrote_last <= 1'b1;
    //end 
    //else begin
      //fifo_read <= 1'b0;
      //FSL_M_Writed <= 1'b0;
      //wrote_last <= 1'b0;
    //end
  //end
//end

//// Delay the write b/c the Coregen FIFO has latency at output
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) 
    //FSL_M_Write <= 1'b0;
  //else
    //FSL_M_Write <= FSL_M_Writed;
//end


//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) 
    //Addr_prev <= 19'b0;
  //else 
    //Addr_prev <= Addr;
//end
//
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1) 
    //batch <= 1'b0;
  //else begin
    ////if ((Addr == 19'h0) && (Addr_prev != 19'h0))  // frame address resetting
    //if ((Fmid == 1'b0) && (Vmid == 1'b1))
      //batch <= 1'b0;
    //else
      //batch <= 1'b1;
  //end
//end
	
// Add a FIFO so that data does not get lost if ZBT controller is busy
// servicing the SVGA controller when the video starts up.  This would
// hopefully avoid data loss due to FSL being full, but only for the
// duration of 1 SVGA line (`H_ACTIVE) 
// Has to carry the write & control bit too
/*Sync_FIFO_SIM 
 #(.C_IMPL_STYLE(0),
//   .WordSize(33),
//   .MemSize(`H_ACTIVE))  // impl style (0 => not BRAMs), width, depth
   .WordSize(25),  // Don't need the upper-most word
   .MemSize(730))  // impl style (0 => not BRAMs), width, depth

   sync_fifo (
    .Reset(rst), 
    .Clk(clk), 
    .WE(fifo_write), 
    .DataIn(DataIn), 
    .Full(fifo_full), 
    .RD(fifo_read), 
    .DataOut(DatatoFSL), 
    .Exists(fifo_exists)
    );
*/
//coregen_fifo coregen_fifo (
	//.clk(clk),
	//.sinit(rst),
	//.din(DataIn),
	//.wr_en(fifo_write),
	//.rd_en(fifo_read),
	//.dout(DatatoFSL),
	//.full(fifo_full),
	//.empty(fifo_empty)); 


//addr_gen addr_gen (
    //.clk(clk), 
    //.rst(rst), 
    //.Fin(Fmid), 
    //.Vin(Vmid), 
    //.Hin(Hmid), 
    //.Addr(Addr)
    //);

input2rgb input2rgb (
    .video_bits(VIDEO_INPUT), 
    .clk(clk), 
    .rst(rst), 
    .R(R_data), 
    .G(G_data), 
    .B(B_data), 
    .Fmid(Fmid), 
    .Vmid(Vmid), 
    .Hmid(Hmid)
    );

endmodule
