//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//


/*
-------------------------------------------------------------------------------
   Title      : Address bus intreface for ZBT Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : ADDR_BUS_INTERFACE.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : defines.v
-------------------------------------------------------------------------------
   Used by    : ZBT_CONTROL.v
-------------------------------------------------------------------------------
   Description: This module defines the address bus interface for the ZBT RAM 
		    controller.
-------------------------------------------------------------------------------
------------------------------------------------------------------------------
*/

//`include "W:\ece532\svga_dbl_buf\pcores\bm_mode_svga_ctrl\hdl\verilog\defines.v"

module addr_bus_interface (
ram_addr,
user_ram_addr, 
clock
)/* synthesis syn_useioff = 1*/;

parameter ADDR_BITS=19;
parameter DATA_BITS=32;

// interfaces to user logic
input [(ADDR_BITS-1):0]  user_ram_addr;
input 		     	clock;

// Interface to ZBT SRAM
output [(ADDR_BITS-1):0] ram_addr;

reg [(ADDR_BITS-1):0] 	latched_user_addr;
wire [(ADDR_BITS-1):0] 	ram_addr;

always @ (posedge clock) begin
	latched_user_addr <= user_ram_addr;
   end

// instantiate the output buffers
/*
//ORIGINAL VERSION
OBUF_F_12 I_obuf_ram_addr0	(.O(ram_addr[0]),  .I(latched_user_addr[0]));
OBUF_F_12 I_obuf_ram_addr1 	(.O(ram_addr[1]),  .I(latched_user_addr[1]));
OBUF_F_12 I_obuf_ram_addr2 	(.O(ram_addr[2]),  .I(latched_user_addr[2]));
OBUF_F_12 I_obuf_ram_addr3 	(.O(ram_addr[3]),  .I(latched_user_addr[3]));
OBUF_F_12 I_obuf_ram_addr4 	(.O(ram_addr[4]),  .I(latched_user_addr[4]));
OBUF_F_12 I_obuf_ram_addr5 	(.O(ram_addr[5]),  .I(latched_user_addr[5]));
OBUF_F_12 I_obuf_ram_addr6 	(.O(ram_addr[6]),  .I(latched_user_addr[6]));
OBUF_F_12 I_obuf_ram_addr7 	(.O(ram_addr[7]),  .I(latched_user_addr[7]));
OBUF_F_12 I_obuf_ram_addr8 	(.O(ram_addr[8]),  .I(latched_user_addr[8]));
OBUF_F_12 I_obuf_ram_addr9 	(.O(ram_addr[9]),  .I(latched_user_addr[9]));
OBUF_F_12 I_obuf_ram_addr10 	(.O(ram_addr[10]), .I(latched_user_addr[10]));
OBUF_F_12 I_obuf_ram_addr11 	(.O(ram_addr[11]), .I(latched_user_addr[11]));
OBUF_F_12 I_obuf_ram_addr12 	(.O(ram_addr[12]), .I(latched_user_addr[12]));
OBUF_F_12 I_obuf_ram_addr13 	(.O(ram_addr[13]), .I(latched_user_addr[13]));
OBUF_F_12 I_obuf_ram_addr14 	(.O(ram_addr[14]), .I(latched_user_addr[14]));
OBUF_F_12 I_obuf_ram_addr15 	(.O(ram_addr[15]), .I(latched_user_addr[15]));
OBUF_F_12 I_obuf_ram_addr16 	(.O(ram_addr[16]), .I(latched_user_addr[16]));
OBUF_F_12 I_obuf_ram_addr17 	(.O(ram_addr[17]), .I(latched_user_addr[17]));
OBUF_F_12 I_obuf_ram_addr18 	(.O(ram_addr[18]), .I(latched_user_addr[18]));
*/

//just assign
assign 	ram_addr[18:0]   = latched_user_addr[18:0];


endmodule // ADDR_BUS_INTERFACE

