module tvin2 (
  // inputs
  clk,
  rst,
  VIDEO_INPUT,
  led0_o,
  led1_o, // for debug
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
/*
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P
*/
MEMORY_DATA_A_P_I,
MEMORY_DATA_A_P_O,
MEMORY_DATA_A_P_T,
MEMORY_DATA_B_P_I,
MEMORY_DATA_B_P_O,
MEMORY_DATA_B_P_T,
MEMORY_DATA_C_P_I,
MEMORY_DATA_C_P_O,
MEMORY_DATA_C_P_T,
MEMORY_DATA_D_P_I,
MEMORY_DATA_D_P_O,
MEMORY_DATA_D_P_T
);

input clk;
input rst;
input [7:0] VIDEO_INPUT; // Connect the MSB pins
output led0_o, led1_o; // for debug

output 			MEMORY_CLK_P;		// ZBT RAM clock
output 			MEMORY_CLKEN_N;	// ZBT RAM clock enable
output 			MEMORY_WEN_N;		// ZBT RAM write enable
output 			MEMORY_WENA_N;	// ZBT RAM byte A write enable
output 			MEMORY_WENB_N;	// ZBT RAM byte B write enable
output 			MEMORY_WENC_N;	// ZBT RAM byte C write enable
output 			MEMORY_WEND_N;	// ZBT RAM byte D write enable
output 			MEMORY_ADV_LD_N;	// ZBT RAM address load-advance
output 			MEMORY_OEN_N;		// ZBT RAM output enable
output 			MEMORY_CEN_N;		// ZBT RAM chip enable
output [18:0]MEMORY_ADDR_P;// ZBT RAM address bus
/*
inout  [7:0] 		MEMORY_DATA_A_P;	// ZBT RAM data bus byte "A"
inout  [7:0] 		MEMORY_DATA_B_P;	// ZBT RAM data bus byte "B"
inout  [7:0] 		MEMORY_DATA_C_P;	// ZBT RAM data bus byte "C"
inout  [7:0] 		MEMORY_DATA_D_P;	// ZBT RAM data bus byte "D"
*/
input  [7:0]		MEMORY_DATA_A_P_I;
output [7:0]		MEMORY_DATA_A_P_O;
output [7:0]		MEMORY_DATA_A_P_T;
input  [7:0]		MEMORY_DATA_B_P_I;
output [7:0]		MEMORY_DATA_B_P_O;
output [7:0]		MEMORY_DATA_B_P_T;
input  [7:0]		MEMORY_DATA_C_P_I;
output [7:0]		MEMORY_DATA_C_P_O;
output [7:0]		MEMORY_DATA_C_P_T;
input  [7:0]		MEMORY_DATA_D_P_I;
output [7:0]		MEMORY_DATA_D_P_O;
output [7:0]		MEMORY_DATA_D_P_T;


wire led0_o, led1_o;

wire user_memory_clock;
wire user_memory_ram_clock;
wire user_memory_clocken;
wire user_memory_load_addr;
wire user_memory_read;
reg user_memory_write;
wire user_memory_wea;
wire user_memory_web;
wire user_memory_wec;
wire user_memory_wed;
wire user_memory_ce;
reg [18:0]user_memory_addr;
wire [31:0]user_memory_read_data;
reg [31:0]user_memory_write_data;

//  clk, rst, // rst should be driven by FSL_Rst
//  VIDEO_INPUT,

wire [7:0] R_data;
wire [7:0] G_data;
wire [7:0] B_data;
wire Fmid;
wire Vmid;
wire Hmid;

reg [18:0] y;
reg [18:0] x;
reg [18:0] counter;

wire [31:0] rgbdata;

//wire Fmid, Vmid, Hmid;
reg [5:0] Hmid_prev;
reg [2:0] pres_state, next_state;
reg [9:0] pixel_ct; // for debugging

reg delayer;

assign led0_o = 1;
assign led1_o = 0;
  
assign user_memory_clock = clk;
assign user_memory_ram_clock = clk;
assign user_memory_clocken = 1;
assign user_memory_load_addr = 1;
assign user_memory_read = 0;
//assign user_memory_write = 0;
assign user_memory_wea = 1;
assign user_memory_web = 1;
assign user_memory_wec = 1;
assign user_memory_wed = 1;
assign user_memory_ce = 1;
//assign user_memory_addr
//assign user_memory_read_data
//assign user_memory_write_data

assign rgbdata[31:24] = 8'b0;
assign rgbdata[23:16] = R_data;
assign rgbdata[15:8] = G_data;
assign rgbdata[7:0] = B_data;

/*always @ (posedge FSL_S_Clk) begin
	if (FSL_S_Exists == 1) begin
		FSL_S_Control <= 1;
		FSL_S_Read <= 1;
		//FSL_S_Data <= RGBdata;
	end
	else begin
		FSL_S_Control <= 0;
		FSL_S_Read <= 0;
	end
end*/


always @ (posedge clk) begin
	if (Vmid == 1) y <= -10;
	if (Hmid == 1) x <= 0;

	if (Hmid == 0 && Vmid == 0) begin
		if (delayer == 0) begin
			delayer <= 1;
			user_memory_write <= (x >= 40 && x < 680) ? 1 : 0;
			if (Fmid == 0) begin
				user_memory_addr <= y * 2 * 640 + (x - 40);
			end
			else begin
				user_memory_addr <= y * 2 * 640 + 640 + (x - 40);
			end
			user_memory_write_data <= rgbdata;

			x <= x + 1;
			if (x == 681) y <= y + 1;
		end
		else begin
			delayer <= 0;
			user_memory_write <= 0;
		end
	end
	else begin
		user_memory_write <= 0;
	end
end


memory_ctrl_532 memory_ctrl_532(
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
/*
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P,
*/
MEMORY_DATA_A_P_I,
MEMORY_DATA_A_P_O,
MEMORY_DATA_A_P_T,
MEMORY_DATA_B_P_I,
MEMORY_DATA_B_P_O,
MEMORY_DATA_B_P_T,
MEMORY_DATA_C_P_I,
MEMORY_DATA_C_P_O,
MEMORY_DATA_C_P_T,
MEMORY_DATA_D_P_I,
MEMORY_DATA_D_P_O,
MEMORY_DATA_D_P_T,

user_memory_clock,
user_memory_ram_clock,
user_memory_clocken,
user_memory_load_addr,
user_memory_read,
user_memory_write,
user_memory_wea,
user_memory_web,
user_memory_wec,
user_memory_wed,
user_memory_ce,
user_memory_addr,
user_memory_read_data,
user_memory_write_data
);

video_input video_input(
  // inputs
  clk, rst, // rst should be driven by FSL_Rst
  VIDEO_INPUT,
  
  R_data, 
  G_data, 
  B_data, 
  Fmid,
  Vmid,
  Hmid
);


endmodule