#include "xparameters.h"
#include "xutil.h"
#include "xbasic_types.h"


#define DELTA_RED_THRESHOLD	(100)
/*
#define MIN_RED_THRESHOLD	(200)	//on red channel only
#define MAX_GREEN_THRESHOLD	(100)	
#define MAX_BLUE_THRESHOLD	(100)
*/
#define RED_COLOR_MASK			(0x00FF0000)
#define RED_COLOR_DOWNSHIFT		(16)
#define GREEN_COLOR_MASK		(0x0000FF00)
#define GREEN_COLOR_DOWNSHIFT	(8)
#define BLUE_COLOR_MASK			(0x000000FF)
#define BLUE_COLOR_DOWNSHIFT	(0)

#define MIN_SIZE			(5)		//this many occurences with below spacing in both X/Y directions
#define CHECK_SPACING		(3)		//number of pixels to skip in X/Y scans, to speed things up!

#define QUAD_ACROSS			(8)
#define QUAD_HIGH			(6)
#define QUADRANT_COUNT		(48)	//MUST BE QUAD_ACROSS*QUAD_HIGH

	#define LEFT	(0)
	#define TOP		(1)
	#define RIGHT	(2)
	#define BOTTOM	(3)
	#define SIDES	(4)
long CHECK_QUAD_RECTS[QUADRANT_COUNT][SIDES];

const long MIN_QUAD_SCORE	= MIN_SIZE*MIN_SIZE;

#define getRED(col)		((col & RED_COLOR_MASK)   >> RED_COLOR_DOWNSHIFT)
#define getGREEN(col)	((col & GREEN_COLOR_MASK) >> GREEN_COLOR_DOWNSHIFT)
#define getBLUE(col)	((col & BLUE_COLOR_MASK)  >> BLUE_COLOR_DOWNSHIFT)


void SetupQuadrants()
{
	const int W = WIDTH  / QUAD_ACROSS;
	const int H = HEIGHT / QUAD_HIGH;
	int rect_num = 0;
	int x, y;
	int scrx=0, scry=0;

	for (y=0; y<QUAD_HIGH; y++)
	{
		scrx = 0;
		for (x=0; x<QUAD_ACROSS; x++)
		{
			CHECK_QUAD_RECTS[rect_num][LEFT] = scrx;
			CHECK_QUAD_RECTS[rect_num][TOP] = scry;
			CHECK_QUAD_RECTS[rect_num][RIGHT] = scrx + W;
			CHECK_QUAD_RECTS[rect_num][BOTTOM] = scry + H;
			
			rect_num++;
			scrx += W;
		}
		scry += H;
	}
}


//this function tries to identify the 9-sector "quadrant" a colored flag is located in
//  RETURNS: 0 for not found, 1-9 for the "quadrant" detected,
//										in the order	1 2 3
//														4 5 6
//														7 8 9
int LocateFlagSector()
{
	long *screen;
	long quadrant_score[QUADRANT_COUNT];	//record scores per quadrant here.
	int quadrant;
	int x, y;
	int color, red, green, blue;
	int best_quad;
	int best_score;

	//check each quadrant, adding one to the score for each pixel that meets the threshold
	for (quadrant=0; quadrant < QUADRANT_COUNT; quadrant++)
	{
		quadrant_score[quadrant] = 0;	//reset score
		
		for (y=CHECK_QUAD_RECTS[quadrant][TOP]; y<=CHECK_QUAD_RECTS[quadrant][BOTTOM]; y+=CHECK_SPACING)
		{
			screen = (long*)XPAR_ZBT_512KX32_MEM0_BASEADDR + y*640;
			for (x=CHECK_QUAD_RECTS[quadrant][LEFT]; x<=CHECK_QUAD_RECTS[quadrant][RIGHT]; x+=CHECK_SPACING)
			{
				color = screen[x];
				red = getRED(color);
				green = getGREEN(color);
				blue = getBLUE(color);
				//if (red >= MIN_RED_THRESHOLD && green <= MAX_GREEN_THRESHOLD && blue <= MAX_BLUE_THRESHOLD)
				if ((red-green) >= DELTA_RED_THRESHOLD && (red-blue) >= DELTA_RED_THRESHOLD)
					quadrant_score[quadrant]++;
			}
		}
	}

	//check scores
	best_quad = 0;	//none found
	best_score = 0;
	for (quadrant=QUADRANT_COUNT-1; quadrant >= 0; quadrant--)
	{
		if (quadrant_score[quadrant] > MIN_QUAD_SCORE && quadrant_score[quadrant] > best_score)
		{
			best_score = quadrant_score[quadrant];
			best_quad = quadrant+1;
		}
	}

	return best_quad;
}
