//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

 /*

-------------------------------------------------------------------------------
   Title      : SVGA Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : BM_MODE_SVGA_CTRL.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/20
   Last Update: 2001/06/20
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : SVGA_TIMING_GENERATION.v DRIVE_DAC_DATA.v MEMORY_CTRL.V
-------------------------------------------------------------------------------
   Used by    : 
-------------------------------------------------------------------------------
   Description: This is the top level module for the bit_mapped mode SVGA display.
   	This design supports screen resolutions up to 800 x 600 with 24 bit color.
	This module uses one ZBT RAM as the video memory. The user is granted access to
	the RAM during horizontal and vertical blanking. An additional ZBT RAM will be
	required if the resolution is increased to 1024 x 768 the maximum supported
	by the SVGA_TIMING_GENERATION module.

	Conventions:
		All external port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
*/
module BM_MODE_SVGA_CTRL (
// connections to the DAC
VGA_OUT_PIXEL_CLOCK_P,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH_N,
VGA_VSYNCH_N,
VGA_OUT_RED_P,
VGA_OUT_GREEN_P,
VGA_OUT_BLUE_P,

// connections to the ZBT RAM
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P,

// connections to the user interface
user_vga_addr,
user_vga_read,
user_vga_write,
user_vga_write_byte_a,
user_vga_write_byte_b,
user_vga_write_byte_c,
user_vga_write_byte_d,
user_vga_enable,
user_vga_read_data,
user_vga_write_data,
user_access_ok,
//user_clock,
user_ram_clock,
user_clock_enable,
user_load_vga_addr,
pixel_clock,
reset
);

output 		VGA_OUT_PIXEL_CLOCK_P;	// pixel clock for the video DAC
output		VGA_COMP_SYNCH_N;		// composite sync for the video DAC
output 		VGA_OUT_BLANK_N;		// composite blanking for the video DAC
output 		VGA_HSYNCH_N;		// horizontal sync for the VGA output connector
output		VGA_VSYNCH_N;		// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED_P;		// RED DAC data
output [7:0] 	VGA_OUT_GREEN_P;		// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE_P;		// BLUE DAC data

output [18:0]	MEMORY_ADDR_P;		// external memory address bus
output 		MEMORY_CLK_P;		// external memory clock
output 		MEMORY_CLKEN_N;		// external memory clock enable
output 		MEMORY_WEN_N;		// external memory read/write
output 		MEMORY_WENA_N;		// external memory write enable for byte A
output 		MEMORY_WENB_N;		// external memory write enable for byte B
output 		MEMORY_WENC_N;		// external memory write enable for byte C
output 		MEMORY_WEND_N;		// external memory write enable for byte D
output 		MEMORY_OEN_N;		// external memory output enable
output 		MEMORY_CEN_N;		// external memory chip enable
output 		MEMORY_ADV_LD_N;		// external memory load/advance
inout [7:0] 	MEMORY_DATA_A_P;		// external memory byte A bits [ 7: 0] BLUE DATA
inout [7:0] 	MEMORY_DATA_B_P;		// external memory byte B bits [15: 8] GREEN DATA
inout [7:0] 	MEMORY_DATA_C_P;		// external memory byte C bits [23:16] RED DATA
inout [7:0] 	MEMORY_DATA_D_P;		// external memory byte D bits [31:24] MISC DATA

input [18:0]	user_vga_addr;
input 		user_vga_read;
input 		user_vga_write;
input 		user_vga_write_byte_a;
input 		user_vga_write_byte_b;
input 		user_vga_write_byte_c;
input 		user_vga_write_byte_d;
input 		user_vga_enable;
output [31:0]	user_vga_read_data;
input  [31:0]	user_vga_write_data;
output		user_access_ok;
//input			user_clock				/* synthesis syn_noclockbuf = 1 */; // user clock input
input			user_ram_clock			/* synthesis syn_noclockbuf = 1 */; // user clock for the RAM input
input			user_clock_enable;
input			user_load_vga_addr;			// flag to load the address specified by the user_vga_add bus 

input	 		pixel_clock				/* synthesis syn_noclockbuf = 1 */; // pixel clock input 
input			reset;				// reset


wire			VGA_OUT_PIXEL_CLOCK_P;
reg 			VGA_COMP_SYNCH_N;
reg			VGA_OUT_BLANK_N;
reg			VGA_HSYNCH_N;
reg			VGA_VSYNCH_N;
reg	[7:0]		VGA_OUT_RED_P; 
reg	[7:0]		VGA_OUT_GREEN_P; 
reg	[7:0]		VGA_OUT_BLUE_P;

wire  [18:0]	MEMORY_ADDR_P;
wire	 		MEMORY_CLK_P;
wire	 	 	MEMORY_CLKEN_N;
wire	 		MEMORY_WEN_N;
wire	 	 	MEMORY_WENA_N;
wire	 	 	MEMORY_WENB_N;
wire	 	 	MEMORY_WENC_N;
wire	 	 	MEMORY_WEND_N;
wire	 	 	MEMORY_OEN_N;
wire	 	 	MEMORY_CEN_N;
wire	 	 	MEMORY_ADV_LD_N;
wire	 [7:0] 	MEMORY_DATA_A_P;
wire	 [7:0] 	MEMORY_DATA_B_P;
wire	 [7:0] 	MEMORY_DATA_C_P;
wire	 [7:0] 	MEMORY_DATA_D_P;

wire 	[18:0]	user_vga_addr;
wire 			user_vga_read;
wire 			user_vga_write;
wire 			user_vga_write_byte_a;
wire 			user_vga_write_byte_b;
wire 			user_vga_write_byte_c;
wire 			user_vga_write_byte_d;
wire 			user_vga_enable;
wire 	[31:0]	user_vga_read_data;
wire  [31:0]	user_vga_write_data;
wire			user_access_ok;
wire			user_clock;				
wire			user_ram_clock;	
wire			user_clock_enable;	

wire 	[31:0]	user_memory_read_data;
wire 	[31:0]	user_memory_write_data;

wire 		user_memory_clock;
wire 		user_memory_ram_clock;
wire 		user_memory_clocken;
wire 		user_memory_load_addr;
wire 		user_memory_read;
wire 		user_memory_write;
wire 		user_memory_wea;
wire 		user_memory_web;
wire 		user_memory_wec;
wire 		user_memory_wed;
wire 		user_memory_ce;
wire [18:0]		user_memory_addr;

wire [7:0]		bm_red_data;
wire [7:0]		bm_green_data; 
wire [7:0]		bm_blue_data; 
wire [7:0]		char_mode_data;
wire 			bm_display_active;
wire 			char_display_active;
wire [7:0]		dac_red_data;
wire [7:0]		dac_green_data;
wire [7:0]		dac_blue_data;


wire 			h_synch_delay;
wire 			v_synch_delay;
wire 			comp_synch;
wire 			blank;
wire [19:0]		vga_ram_read_address;
//wire 			user_access_ok;
wire 			select_user_clock;

wire 			pixel_clock;
wire 			reset;
//wire user_clock;

assign user_clock = pixel_clock;

// make the internal connections
assign bm_red_data = user_memory_read_data[23:16]; 
assign bm_green_data = user_memory_read_data[15:8];  
assign bm_blue_data = user_memory_read_data[7:0]; 
assign char_mode_data 	= 8'h00;
assign bm_display_active = 1'b1;
assign char_display_active = 1'b0;

// select the user inputs to drive the memory controller ONLY when the SVGA_TIMING_GENERATOR
// is not reading the RAM


assign	user_memory_clocken 	= user_access_ok ? user_clock_enable:1'b1;
assign	user_memory_load_addr 	= user_access_ok ? user_load_vga_addr	:1'b1;
assign	user_memory_read 		= user_access_ok ? user_vga_read	:1'b1;
assign	user_memory_write 	= user_access_ok ? user_vga_write	:1'b0;
assign	user_memory_wea 		= user_access_ok ? user_vga_write_byte_a :1'b0;
assign	user_memory_web 		= user_access_ok ? user_vga_write_byte_b :1'b0;
assign	user_memory_wec 		= user_access_ok ? user_vga_write_byte_c :1'b0;
assign	user_memory_wed 		= user_access_ok ? user_vga_write_byte_d :1'b0;
assign	user_memory_ce 		= user_access_ok ? user_vga_enable	:1'b1;
assign	user_memory_addr[18:0] 		= user_access_ok ? user_vga_addr[18:0]	:vga_ram_read_address[18:0];
assign	user_vga_read_data[31:0] 	= user_memory_read_data[31:0];
assign	user_memory_write_data[31:0] 	= user_vga_write_data[31:0];


// instantiate clock multiplexers
//create the clock mux for memory controller
BUFGMUX memory_ctrl_clk_mux (.I0(pixel_clock), .I1(user_clock), .S(select_user_clock), .O(user_memory_clock)) /* synthesis syn_noclockbuf = 1 */;

//create the clock mux for RAMs
BUFGMUX ram_clk_mux (.I0(pixel_clock), .I1(user_ram_clock), .S(select_user_clock), .O(user_memory_ram_clock)) /* synthesis syn_noclockbuf = 1 */;



// make the external connections
// shut down the video output during  reset
always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		VGA_HSYNCH_N	 	<= 1'b1;
		VGA_VSYNCH_N 	 	<= 1'b1;
		VGA_COMP_SYNCH_N 		<= 1'b1;
		VGA_OUT_BLANK_N 		<= 1'b0;
		VGA_OUT_RED_P		<= 8'h00;
		VGA_OUT_GREEN_P		<= 8'h00;
		VGA_OUT_BLUE_P		<= 8'h00;
		end
	else begin
		VGA_HSYNCH_N	 	<= !h_synch_delay;
		VGA_VSYNCH_N 	 	<= !v_synch_delay;
		VGA_COMP_SYNCH_N 		<= !comp_synch;
		VGA_OUT_BLANK_N 		<= !blank;
		VGA_OUT_RED_P		<= dac_red_data;
		VGA_OUT_GREEN_P		<= dac_green_data;
		VGA_OUT_BLUE_P		<= dac_blue_data;
		end
	end

assign  VGA_OUT_PIXEL_CLOCK_P	= pixel_clock;

// instantiate the SVGA timing generator
SVGA_TIMING_GENERATION  SVGA_TIMING_GENERATION (
pixel_clock,
reset,
h_synch_delay,
v_synch_delay,
comp_synch,
blank,
vga_ram_read_address,
user_access_ok,
select_user_clock);


// instantiate the DAC driver
DRIVE_DAC_DATA DRIVE_DAC_DATA(
bm_red_data, 
bm_green_data, 
bm_blue_data, 
char_mode_data,
bm_display_active,
char_display_active,
dac_red_data,
dac_green_data,
dac_blue_data
);



// instantiate the memory controller
MEMORY_CTRL MEMORY_CTRL(
MEMORY_CLK_P,
MEMORY_CLKEN_N,
MEMORY_WEN_N,
MEMORY_WENA_N,
MEMORY_WENB_N,
MEMORY_WENC_N,
MEMORY_WEND_N,
MEMORY_ADV_LD_N,
MEMORY_OEN_N,
MEMORY_CEN_N,
MEMORY_ADDR_P,
MEMORY_DATA_A_P,
MEMORY_DATA_B_P,
MEMORY_DATA_C_P,
MEMORY_DATA_D_P,

user_memory_clock,
user_memory_ram_clock,
user_memory_clocken,
user_memory_load_addr,
user_memory_read,
user_memory_write,
user_memory_wea,
user_memory_web,
user_memory_wec,
user_memory_wed,
user_memory_ce,
user_memory_addr,
user_memory_read_data,
user_memory_write_data
);


endmodule //BM_MODE_SVGA_CTRL


module IOBUF_F_12(O, IO, I, T); // synthesis syn_black_box
output O;
inout IO;
input I;
input T;
endmodule

module FD(Q, C, D); // synthesis syn_black_box
output Q;
input C;
input D;
endmodule


module BUFGMUX (O, I0, I1, S); // synthesis syn_black_box
input  I0;
input  I1;
input  S;
output O;
endmodule


module OBUF_F_12(O, I); // synthesis syn_black_box
output O;
input I;
endmodule


