//************************************************************************
//    This file is a modified version of the version provided by Xilinx
//    File: BM_MODE_SVGA_CTRL_532.v
//    Uses: SVGA_TIMING_GENERATION_532.v
//          DRIVE_DAC_DATA.v (Xilinx original)
//          MEMORY_CTRL_532.v
//          svga_defines_532.v
//    Used by: top_dualzbt_dow.v
//    Modified for: University of Toronto ECE532 project
//                  FSL SVGA controller module
//    Modifications:
//      Added FSL interface
//      Added delay_cycles parameter
//      Added counter to make sure we read everything off FSL
//      For MultiMedia board, using 25MHz pixel clock => 640x480
//
//    Protocol for communication is:
//      control word:  upper 19 bits give # of words to be read
//                     20th bit is read/write_n by SVGA
//      address word:  upper 19 bits give address (19 bits to address into
//                      a ZBT)
//      data:  must read exactly the number of data words you specified
//              you would in the control word, otherwise the FSLs get clogged
//
//    Below is the header provided by Xilinx:
//************************************************************************
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

 /*

-------------------------------------------------------------------------------
   Title      : SVGA Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : BM_MODE_SVGA_CTRL.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/20
   Last Update: 2001/06/20
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : SVGA_TIMING_GENERATION.v DRIVE_DAC_DATA.v MEMORY_CTRL.V
-------------------------------------------------------------------------------
   Used by    : 
-------------------------------------------------------------------------------
   Description: This is the top level module for the bit_mapped mode SVGA display.
   	This design supports screen resolutions up to 800 x 600 with 24 bit color.
	This module uses one ZBT RAM as the video memory. The user is granted access to
	the RAM during horizontal and vertical blanking. An additional ZBT RAM will be
	required if the resolution is increased to 1024 x 768 the maximum supported
	by the SVGA_TIMING_GENERATION module.

	Conventions:
		All external port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
*/

`include "svga_defines_532.v"

module fsl_BM_MODE_SVGA_CTRL_532 (
// connections to the DAC
VGA_OUT_PIXEL_CLOCK_P,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH_N,
VGA_VSYNCH_N,
VGA_OUT_RED_P,
VGA_OUT_GREEN_P,
VGA_OUT_BLUE_P,

user_access_ok,
select_user_clock,
pixel_clock,
pixel_clock_2x, // For FSL
reset,

  // Slave FSL Signals
  FSL_S_Clk,
  FSL_S_Read,
  FSL_S_Data,
  FSL_S_Control,
  FSL_S_Exists,

  // Master FSL Signals
  FSL_M_Clk,
  FSL_M_Write,
  FSL_M_Data,
  FSL_M_Control,
  FSL_M_Full

);

parameter base_address = 00000; 
parameter delay_cycles = 3;


output 		VGA_OUT_PIXEL_CLOCK_P;	// pixel clock for the video DAC
output		VGA_COMP_SYNCH_N;		// composite sync for the video DAC
output 		VGA_OUT_BLANK_N;		// composite blanking for the video DAC
output 		VGA_HSYNCH_N;		// horizontal sync for the VGA output connector
output		VGA_VSYNCH_N;		// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED_P;		// RED DAC data
output [7:0] 	VGA_OUT_GREEN_P;		// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE_P;		// BLUE DAC data

output		user_access_ok;
output select_user_clock;
input	 		pixel_clock;				/* synthesis syn_noclockbuf = 1 */ // pixel clock input 
input 	pixel_clock_2x;
input			reset;				// reset

// Slave FSL Signals
output FSL_S_Clk;
output FSL_S_Read;
input  [0:31] FSL_S_Data;
input  FSL_S_Control;
input  FSL_S_Exists;

  // Master FSL Signals
 output FSL_M_Clk;
 output FSL_M_Write;
 output [0:31] FSL_M_Data;
 output FSL_M_Control;
 input FSL_M_Full;


wire FSL_S_Clk;
wire FSL_S_Read;
reg FSL_S_Read_reg;
wire [0:31] FSL_S_Data;
wire FSL_S_Control;
wire FSL_S_Exists;
wire FSL_M_Clk;
 reg FSL_M_Write;
 reg [0:31] FSL_M_Data;
 reg FSL_M_Control;
 wire FSL_M_Full;

wire			VGA_OUT_PIXEL_CLOCK_P;
reg 			VGA_COMP_SYNCH_N;
reg			VGA_OUT_BLANK_N;
reg			VGA_HSYNCH_N;
reg			VGA_VSYNCH_N;
reg	[7:0]		VGA_OUT_RED_P; 
reg	[7:0]		VGA_OUT_GREEN_P; 
reg	[7:0]		VGA_OUT_BLUE_P;

wire			user_access_ok;
wire			svga_user_access_ok;
wire select_user_clock;
wire svga_select_user_clock;
wire 			pixel_clock;
wire 			pixel_clock_2x;
wire 			reset;

wire [7:0]		bm_red_data;
wire [7:0]		bm_green_data; 
wire [7:0]		bm_blue_data; 
wire [7:0]		dac_red_data;
wire [7:0]		dac_green_data;
wire [7:0]		dac_blue_data;


wire 			h_synch_delay;
wire 			v_synch_delay;
wire 			comp_synch;
wire 			blank;
wire [19:0]		vga_ram_read_address;

wire[18:0] num_pixels;

reg [2:0] pres_state;
reg [2:0] next_state;
reg [18:0]addr_reg0;
reg [18:0]addr_reg1;
reg [18:0]addr_reg2;
reg [18:0]addr_reg3;
wire [18:0] full_addr;
wire [18:0] base_addr_sig;

// Make sure we read off everything we requested to prevent FSL from
// getting backlogged & hanging the ZBT controller
reg [11:0] memdata_ctr; // Max pix are 1024, but don't want overflow errors

reg S_altlevs, M_altlevs;
 
// For State machine
`define st_reset 0
`define st_ctrl 1
`define st_addr 2
`define st_read 3
`define st_useracc_ok 4


assign FSL_S_Clk = pixel_clock_2x;
assign FSL_M_Clk = pixel_clock_2x;

assign FSL_S_Read = FSL_S_Read_reg;
assign base_addr_sig = base_address;
assign full_addr = base_addr_sig + addr_reg3;


// make the internal connections
// Should I be registering FSL_S_Data?

assign bm_red_data = FSL_S_Data[8:15]; 
assign bm_green_data = FSL_S_Data[16:23];  
assign bm_blue_data = FSL_S_Data[24:31]; 

assign num_pixels = `H_ACTIVE;


	
// Don't want to allow access between first request for data and
// last word of that data coming from memory
assign user_access_ok = svga_user_access_ok;
assign select_user_clock = svga_select_user_clock;


// FSM register
always @ (posedge pixel_clock or posedge reset) begin
  if (reset == 1'b1) begin
    pres_state <= `st_reset; 
  end
  else begin
    pres_state <= next_state;
  end
end

// Determine next state
always @ (negedge pixel_clock or posedge reset) begin
  if (reset == 1'b1) begin
    next_state <= `st_reset; 
  end
  else begin
    case (pres_state) 
      `st_reset:
	begin
          if (reset == 1'b0) begin
            next_state <= `st_ctrl;
	  end
	  else begin
            next_state <= `st_reset;
	  end
	end
      `st_ctrl:
	next_state <= `st_addr;
      `st_addr:
	next_state <= `st_read;
      `st_read:
	begin
          if (user_access_ok == 1'b1) begin  // Keep reading until all data is back
            next_state <= `st_useracc_ok;
	  end
	  else begin
            next_state <= `st_read;
	  end
	end
      `st_useracc_ok:
	begin
          if (user_access_ok == 1'b1) begin  
            next_state <= `st_useracc_ok;
	  end
	  else begin
            next_state <= `st_ctrl;
	  end
	end
    endcase
  end
end


// FSL Slave Output
always @ (posedge pixel_clock_2x or posedge reset) begin
  if (reset == 1'b1) begin
    FSL_S_Read_reg <= 1'b0; 
    S_altlevs <= 1'b1;
    memdata_ctr <= 11'h0;
  end
  else begin
    case (pres_state) 
      `st_reset:
       begin
         FSL_S_Read_reg <= 1'b0; 
         memdata_ctr <= 11'h0;
       end
      `st_ctrl:
       begin
         FSL_S_Read_reg <= 1'b0; 	      
         memdata_ctr <= 11'h0;  // Since we don't go back to st_reset after each line, reset counter here
       end
      `st_addr:
       begin
         FSL_S_Read_reg <= 1'b0; 	      
         memdata_ctr <= memdata_ctr;
       end
      `st_read:
        begin	      
          if ((FSL_S_Exists == 1'b1) && (memdata_ctr < `H_ACTIVE) && (S_altlevs == 1'b1)) begin
            FSL_S_Read_reg <= 1'b1; 	      
            memdata_ctr <= memdata_ctr + 1;
	  end
 	  else begin
            FSL_S_Read_reg <= 1'b0; 
            memdata_ctr <= memdata_ctr;
	  end
        end
      `st_useracc_ok:
	 begin
//        I am assuming that the user_access_ok period is long enough to
//        clear out any backlog in the FSL
          if ((FSL_S_Exists == 1'b1) && (memdata_ctr < `H_ACTIVE) && (S_altlevs == 1'b1)) begin
            FSL_S_Read_reg <= 1'b1; 
            memdata_ctr <= memdata_ctr + 1;
	  end
 	  else begin
            FSL_S_Read_reg <= 1'b0; 
            memdata_ctr <= memdata_ctr;
	  end
        end
    endcase
    S_altlevs <= !S_altlevs;
  end
end
	      
// FSL Master Output, run at 2x pixel clock
always @ (posedge pixel_clock_2x or posedge reset) begin
  if (reset == 1'b1) begin
          FSL_M_Write <= 1'b0;
          FSL_M_Data <= 32'h0000_0000;
          FSL_M_Control <= 1'b0;
	  M_altlevs <= 1'b1;
  end
  else begin
    case (pres_state) 
      `st_reset:
	begin
          FSL_M_Write <= 1'b0;
          FSL_M_Data <= 32'h0000_0000;
          FSL_M_Control <= 1'b0;
	end
      `st_ctrl:
	begin
          FSL_M_Write <= (FSL_M_Full == 1'b1 ? 1'b0 : (1'b1 & M_altlevs));
	  FSL_M_Data <= {5'b00000,num_pixels, 8'h03};
          FSL_M_Control <= 1'b1;
	end
      `st_addr:
	begin
          FSL_M_Write <= (FSL_M_Full == 1'b1 ? 1'b0 : (1'b1 & M_altlevs));
	  FSL_M_Data <= {13'h0000,full_addr};
          FSL_M_Control <= 1'b0;
	end
      `st_read:
	begin
          FSL_M_Write <= 1'b0;
	  FSL_M_Data <= 32'h0;
          FSL_M_Control <= 1'b0;
	end
      `st_useracc_ok:
	begin
          FSL_M_Write <= 1'b0;	  
          FSL_M_Data <= 32'h0;
          FSL_M_Control <= 1'b0;
	end
    endcase 
    M_altlevs <= !M_altlevs;
  end
end


// make the external connections
// shut down the video output during  reset
always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		VGA_HSYNCH_N	 	<= 1'b1;
		VGA_VSYNCH_N 	 	<= 1'b1;
		VGA_COMP_SYNCH_N 		<= 1'b1;
		VGA_OUT_BLANK_N 		<= 1'b0;
		VGA_OUT_RED_P		<= 8'h00;
		VGA_OUT_GREEN_P		<= 8'h00;
		VGA_OUT_BLUE_P		<= 8'h00;

		end
	else begin
		VGA_HSYNCH_N	 	<= !h_synch_delay;
		VGA_VSYNCH_N 	 	<= !v_synch_delay;
		VGA_COMP_SYNCH_N 		<= !comp_synch;
		VGA_OUT_BLANK_N 		<= !blank;
		VGA_OUT_RED_P		<= dac_red_data;
		VGA_OUT_GREEN_P		<= dac_green_data;
		VGA_OUT_BLUE_P		<= dac_blue_data;

		end
	end


//register ram_read_address because of delay with state machine
always @ (posedge pixel_clock or posedge reset) begin
  if (reset) begin
    addr_reg0[18:0] <= 19'h00000;
    addr_reg1[18:0] <= 19'h00000;
    addr_reg2[18:0] <= 19'h00000;
    addr_reg3[18:0] <= 19'h00000;
  end
  else begin
    addr_reg3[18:0] <= addr_reg2[18:0];
    addr_reg2[18:0] <= addr_reg1[18:0];
    addr_reg1[18:0] <= addr_reg0[18:0];
    addr_reg0[18:0] <= vga_ram_read_address[18:0];
  end
end  
    
// instantiate the SVGA timing generator
SVGA_TIMING_GENERATION_532  SVGA_TIMING_GENERATION_532 (
pixel_clock,
reset,
h_synch_delay,
v_synch_delay,
comp_synch,
blank,
vga_ram_read_address,
svga_user_access_ok,
svga_select_user_clock
);


// instantiate the DAC driver
DRIVE_DAC_DATA DRIVE_DAC_DATA(
bm_red_data, 
bm_green_data, 
bm_blue_data, 
8'h00, //char_mode_data,
1'b1, //bm_display_active,
1'b0, //char_display_active,
dac_red_data,
dac_green_data,
dac_blue_data
);


OBUF_F_12 PIXCLK_OUT(
.O(VGA_OUT_PIXEL_CLOCK_P), 
.I(pixel_clock)
);


endmodule //BM_MODE_SVGA_CTRL


