
`timescale  1 ns / 1 ps



module arbiter
  (	
  clk,
  rst,
  ctlrA_using_mem0, //controller A using mem bank 0
  ctlrA_using_mem1, //controller A using mem bank 1
  ctlrB_using_mem0, //controller B using mem bank 0
  ctlrB_using_mem1, //controller B using mem bank 1
  ctlrA_req_mem, //controller A requesting memory 
  ctlrB_req_mem, //controller A requesting memory
  addr_msb_req_A, //controller A requesting a memory bank
  addr_msb_req_B, //controller B requesting a memory bank
  addr_msb_ctrl_A, //controller's A accessed memory bank
  addr_msb_ctrl_B, //controller's B accessed memory bank
  done_0, //mem bank 0 unused (done)
  done_1 //mem bank 1 unused (done)
  );

input clk, rst;
output ctlrA_using_mem0;
output ctlrA_using_mem1;
output ctlrB_using_mem0;
output ctlrB_using_mem1;
input ctlrA_req_mem;
input ctlrB_req_mem;
input [0:12] addr_msb_req_A, addr_msb_req_B;
output [0:12] addr_msb_ctrl_A, addr_msb_ctrl_B;
input done_0, done_1;

wire clk;
wire rst;  
reg ctlrA_using_mem0, ctlrA_using_mem1;
reg ctlrB_using_mem0, ctlrB_using_mem1;
wire ctlrA_req_mem; //, ctlrA_req_mem_1;  
wire ctlrB_req_mem; //, ctlrB_req_mem_1;  
wire [0:12] addr_msb_req_A, addr_msb_req_B;  
reg [0:12] addr_msb_ctrl_A, addr_msb_ctrl_B;  
wire done_0, done_1;
reg prev_done_0, prev_done_1;

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
     prev_done_0 <= 1'b1;
  else
     prev_done_0 <= done_0;
end

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
     prev_done_1 <= 1'b1;
  else
     prev_done_1 <= done_1;
end

//**The next two always blocks do the arbitration (ctlr A has priority)**
//This block controls accesses made by controller A:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
     begin
       ctlrA_using_mem0 <= 1'b0;
       ctlrA_using_mem1 <= 1'b0;
       addr_msb_ctrl_A <= 13'h0000;
     end
  else if ((ctlrA_req_mem == 1'b1) && (addr_msb_req_A[0] == 1'b1) && (done_0 == 1'b1))
     begin
       ctlrA_using_mem0 <= 1'b1;
       ctlrA_using_mem1 <= 1'b0;
       addr_msb_ctrl_A <= 13'h1000;
     end
  else if ((ctlrA_req_mem == 1'b1) && (addr_msb_req_A[1] == 1'b1) && (done_1 == 1'b1))
     begin
       ctlrA_using_mem0 <= 1'b0;
       ctlrA_using_mem1 <= 1'b1;
       addr_msb_ctrl_A <= 13'h0800;
     end
  else if ((ctlrA_using_mem0 == 1'b1) && (done_0 == 1'b1) 
	  				&& (prev_done_0 == 1'b0))
     begin
       ctlrA_using_mem0 <= 1'b0;
       ctlrA_using_mem1 <= 1'b0;
       addr_msb_ctrl_A <= 13'h0000;
     end
  else if ((ctlrA_using_mem1 == 1'b1) && (done_1 == 1'b1) 
	  				&& (prev_done_1 == 1'b0))
     begin
       ctlrA_using_mem0 <= 1'b0;
       ctlrA_using_mem1 <= 1'b0;
       addr_msb_ctrl_A <= 13'h0000;
     end
  else
     begin
       ctlrA_using_mem0 <= ctlrA_using_mem0;
       ctlrA_using_mem1 <= ctlrA_using_mem1;
       addr_msb_ctrl_A <= addr_msb_ctrl_A;
     end
end

//This block controls accesses made by controller B:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
     begin
       ctlrB_using_mem0 <= 1'b0;
       ctlrB_using_mem1 <= 1'b0;
       addr_msb_ctrl_B <= 13'h0000;
     end
  else if ((ctlrB_req_mem == 1'b1) && (addr_msb_req_B[0] == 1'b1) && (done_0 == 1'b1) && ((ctlrA_req_mem == 1'b0) || (addr_msb_req_A[0] == 1'b0)))
     begin
       ctlrB_using_mem0 <= 1'b1;
       ctlrB_using_mem1 <= 1'b0;
       addr_msb_ctrl_B <= 13'h1000;
     end
  else if ((ctlrB_req_mem == 1'b1) && (addr_msb_req_B[1] == 1'b1) && (done_1 == 1'b1) && ((ctlrA_req_mem == 1'b0) || (addr_msb_req_A[1] == 1'b0)))
     begin
       ctlrB_using_mem0 <= 1'b0;
       ctlrB_using_mem1 <= 1'b1;
       addr_msb_ctrl_B <= 13'h0800;
     end
  else if ((ctlrB_using_mem0 == 1'b1) && (done_0 == 1'b1) 
	  				&& (prev_done_0 == 1'b0))
     begin
       ctlrB_using_mem0 <= 1'b0;
       ctlrB_using_mem1 <= 1'b0;
       addr_msb_ctrl_B <= 13'h0000;
     end
  else if ((ctlrB_using_mem1 == 1'b1) && (done_1 == 1'b1) 
	  				&& (prev_done_1 == 1'b0))
     begin
       ctlrB_using_mem0 <= 1'b0;
       ctlrB_using_mem1 <= 1'b0;
       addr_msb_ctrl_B <= 13'h0000;
     end
  else
     begin
       ctlrB_using_mem0 <= ctlrB_using_mem0;
       ctlrB_using_mem1 <= ctlrB_using_mem1;
       addr_msb_ctrl_B <= addr_msb_ctrl_B;
     end
end


endmodule
