

module data_gen_for_fsl_sh_mem_controller
  (	
  core_clk,
  mem_clk,
  reset,

  Dq_0, 
  Addr_0, 
  mem_clk_0,
  CKEb_0, 
  Bwa_n_0, 
  Bwb_n_0, 
  Bwc_n_0, 
  Bwd_n_0,
  WEb_0, 
  ADV_0, 
  OEb_0,                  
  CS1b_0, 
  //CS2_0, 
  //CS2b_0, 
  LBOb_0, 
  ZZ_0,

  Dq_1, 
  Addr_1, 
  mem_clk_1,
  CKEb_1, 
  Bwa_n_1, 
  Bwb_n_1, 
  Bwc_n_1, 
  Bwd_n_1,
  WEb_1, 
  ADV_1, 
  OEb_1,                  
  CS1b_1, 
  //CS2_1, 
  //CS2b_1, 
  LBOb_1, 
  ZZ_1
        );

input core_clk, mem_clk, reset;
inout Dq_0, Dq_1;
output Addr_0, Addr_1;
output mem_clk_0, mem_clk_1;
output CKEb_0, CKEb_1;
output Bwa_n_0, Bwa_n_1;
output Bwb_n_0, Bwb_n_1;
output Bwc_n_0, Bwc_n_1;
output Bwd_n_0, Bwd_n_1;
output WEb_0, WEb_1;
output ADV_0, ADV_1;
output OEb_0, OEb_1;
output CS1b_0, CS1b_1;
//output CS2_0, CS2_1;
//output CS2b_0, CS2b_1;
output LBOb_0, LBOb_1;
output ZZ_0, ZZ_1;


wire core_clk, mem_clk, reset;
wire in_clk;     //clk from testbench to system
wire fsl_rst;
wire rst;  //used to startup the system in a predictable mode

wire [31:0] Dq_0, Dq_1; //Data I/O- 
			//NOTE: the only input/output signal in the module
wire [18:0] Addr_0, Addr_1; //Address
wire mem_clk_0, mem_clk_1;
wire ADV_0, ADV_1;     //Address advance
wire CKEb_0, CKEb_1;    //Slow down mode: acts as a clock enable
wire WEb_0, WEb_1;     //Read is high and Write is Low aka "read, not write"
wire Bwa_n_0, Bwa_n_1;	//Byte Write Enable A
wire Bwb_n_0, Bwb_n_1;	//Byte Write Enable B
wire Bwc_n_0, Bwc_n_1;	//Byte Write Enable C
wire Bwd_n_0, Bwd_n_1;	//Byte Write Enable D
wire CS1b_0, CS1b_1;	//Chip Enable
wire CS2_0, CS2_1;	//Chip Enable
wire CS2b_0, CS2b_1;	//Chip Enable
wire OEb_0, OEb_1;	//Chip Output Enable
wire ZZ_0, ZZ_1;	//Chip Enable vs Sleep Mode
wire LBOb_0, LBOb_1;	//Used to select the autom byte advance mode (ADV) where
			//LBOb = Linear Burst Order bar

wire no_sig; //unused signal

//FSL0 signals writes from video input to memory A:
wire fsl_0_rst;
wire fsl_0_full, fsl_0_has_data;
wire fsl_m_vid_in_clk, fsl_m_vid_in_full;
reg fsl_m_vid_in_write, fsl_m_vid_in_cntrl;
reg[0:31] fsl_m_vid_in_data;
wire fsl_s_memA_clk, fsl_s_memA_read; 
wire fsl_s_memA_cntrl, fsl_s_memA_exists;
wire [0:31] fsl_s_memA_data;

//FSL2 signals writes from video output to memory B:
wire fsl_2_rst;
wire fsl_2_full, fsl_2_has_data;
wire fsl_m_vid_out_clk, fsl_m_vid_out_full;
reg fsl_m_vid_out_write, fsl_m_vid_out_cntrl;
reg[0:31] fsl_m_vid_out_data;
wire fsl_s_memB_clk, fsl_s_memB_read; 
wire fsl_s_memB_cntrl, fsl_s_memB_exists;
wire [0:31] fsl_s_memB_data;

//OUTPUTS FROM SH_MEM CONTROLLER THAT ARE CURRENTLY UNUSED:
wire FSL_M_Clk_B, FSL_M_Write_B, FSL_M_Control_B; 
//reg FSL_M_Full_B;
wire FSL_M_Full_B;
wire [31:0] FSL_M_Data_B;
wire FSL_M_Clk_A, FSL_M_Write_A, FSL_M_Control_A; 
//reg FSL_M_Full_A;
wire FSL_M_Full_A;
wire [31:0] FSL_M_Data_A;


reg [1:0] addr_msb;
reg [2:0] state_cntr;
reg [31:0] data_cntr;
reg [7:0] delay_cntr;
reg [0:2] pres_state, next_state;

//parameter st_reset=0;
parameter st_write_data_instr=0;
parameter st_write_data_addr=1;
parameter st_start_write_data=2;
parameter st_write_data=3;
//parameter st_rx_data_instr=3;
//parameter st_rx_data_addr=4;
//parameter st_rx_data=5;
//parameter st_read_req_data_instr=6;
//parameter st_read_req_data_addr=7;
parameter st_delay=4;

`define wr_dat_instr 32'h00001007
`define wr_dat_0_addr 30'h00000000
`define wr_dat_20_addr 30'h00000020
`define wr_dat_40_addr 30'h00000040
`define wr_dat_60_addr 30'h00000060
`define wr_dat_10_addr 30'h00000010
`define wr_dat_30_addr 30'h00000030
`define wr_dat_50_addr 30'h00000050
`define wr_dat_70_addr 30'h00000070
`define noop 32'h00000004
/*
//Memory model for actual zbt bank (0)
//**NOTE THESE WILL BE ACTUAL SYSTEM INPUTS AND OUTPUTS ON THE BOARD:
   K7N163601A K7N163601A_bank0  (
  .Dq(Dq_0), 
  .Addr(Addr_0), 
  .K(in_clk), 
  .CKEb(CKEb_0), 
  .Bwa_n(Bwa_n_0), 
  .Bwb_n(Bwb_n_0), 
  .Bwc_n(Bwc_n_0), 
  .Bwd_n(Bwd_n_0),
  .WEb(WEb_0), 
  .ADV(ADV_0), 
  .OEb(OEb_0),                  
  .CS1b(CS1b_0), 
  .CS2(CS2_0), 
  .CS2b(CS2b_0), 
  .LBOb(LBOb_0), 
  .ZZ(ZZ_0)
  );

   K7N163601A K7N163601A_bank1 (
  .Dq(Dq_1), 
  .Addr(Addr_1), 
  .K(in_clk), 
  .CKEb(CKEb_1), 
  .Bwa_n(Bwa_n_1), 
  .Bwb_n(Bwb_n_1), 
  .Bwc_n(Bwc_n_1), 
  .Bwd_n(Bwd_n_1),
  .WEb(WEb_1), 
  .ADV(ADV_1), 
  .OEb(OEb_1),                  
  .CS1b(CS1b_1), 
  .CS2(CS2_1), 
  .CS2b(CS2b_1), 
  .LBOb(LBOb_1), 
  .ZZ(ZZ_1)
  );
*/

  fsl_sh_mem_controller fsl_sh_mem_controller (
  .core_clk(core_clk),
  .mem_clk(mem_clk),
  .rst(rst),

  .Dq_0(Dq_0), 
  .Addr_0(Addr_0), 
  .mem_clk_0(mem_clk_0),
  .CKEb_0(CKEb_0), 
  .Bwa_n_0(Bwa_n_0), 
  .Bwb_n_0(Bwb_n_0), 
  .Bwc_n_0(Bwc_n_0), 
  .Bwd_n_0(Bwd_n_0),
  .WEb_0(WEb_0), 
  .ADV_0(ADV_0), 
  .OEb_0(OEb_0),                  
  .CS1b_0(CS1b_0), 
  //.CS2_0(CS2_0), 
  //.CS2b_0(CS2b_0), 
  .LBOb_0(LBOb_0), 
  .ZZ_0(ZZ_0),

  .Dq_1(Dq_1), 
  .Addr_1(Addr_1), 
  .mem_clk_1(mem_clk_1),
  .CKEb_1(CKEb_1), 
  .Bwa_n_1(Bwa_n_1), 
  .Bwb_n_1(Bwb_n_1), 
  .Bwc_n_1(Bwc_n_1), 
  .Bwd_n_1(Bwd_n_1),
  .WEb_1(WEb_1), 
  .ADV_1(ADV_1), 
  .OEb_1(OEb_1),                  
  .CS1b_1(CS1b_1), 
  //.CS2_1(CS2_1), 
  //.CS2b_1(CS2b_1), 
  .LBOb_1(LBOb_1), 
  .ZZ_1(ZZ_1),

  .FSL_S_Clk_A(fsl_s_memA_clk),
  .FSL_S_Read_A(fsl_s_memA_read),
  .FSL_S_Data_A(fsl_s_memA_data),
  .FSL_S_Control_A(fsl_s_memA_cntrl),
  .FSL_S_Exists_A(fsl_s_memA_exists),
  .FSL_M_Clk_A(FSL_M_Clk_A),
  .FSL_M_Write_A(FSL_M_Write_A),
  .FSL_M_Data_A(FSL_M_Data_A),
  .FSL_M_Control_A(FSL_M_Control_A),
  .FSL_M_Full_A(FSL_M_Full_A),

  .FSL_S_Clk_B(fsl_s_memB_clk),
  .FSL_S_Read_B(fsl_s_memB_read),
  .FSL_S_Data_B(fsl_s_memB_data),
  .FSL_S_Control_B(fsl_s_memB_cntrl),
  .FSL_S_Exists_B(fsl_s_memB_exists),
  .FSL_M_Clk_B(FSL_M_Clk_B),
  .FSL_M_Write_B(FSL_M_Write_B),
  .FSL_M_Data_B(FSL_M_Data_B),
  .FSL_M_Control_B(FSL_M_Control_B),
  .FSL_M_Full_B(FSL_M_Full_B)

);

fsl_v20_532 #(.C_FSL_DEPTH(16)) vid_in_to_memA_fsl0 (  
    .FSL_Clk(no_sig), 
    .SYS_Rst(reset), 
    .FSL_Rst(fsl_0_rst), 
    .FSL_M_Clk(fsl_m_vid_in_clk), 
    .FSL_M_Data(fsl_m_vid_in_data), 
    .FSL_M_Control(fsl_m_vid_in_cntrl), 
    .FSL_M_Write(fsl_m_vid_in_write), 
    .FSL_M_Full(fsl_m_vid_in_full), 
    .FSL_S_Clk(fsl_s_memA_clk), 
    .FSL_S_Data(fsl_s_memA_data), 
    .FSL_S_Control(fsl_s_memA_cntrl), 
    .FSL_S_Read(fsl_s_memA_read), 
    .FSL_S_Exists(fsl_s_memA_exists), 
    .FSL_Full(fsl_0_full), 
    .FSL_Has_Data(fsl_0_has_data)
    );
    
fsl_v20_532 #(.C_FSL_DEPTH(16)) vid_out_to_memB_fsl2 (  
    .FSL_Clk(no_sig), 
    .SYS_Rst(reset), 
    .FSL_Rst(fsl_2_rst), 
    .FSL_M_Clk(fsl_m_vid_out_clk), 
    .FSL_M_Data(fsl_m_vid_out_data), 
    .FSL_M_Control(fsl_m_vid_out_cntrl), 
    .FSL_M_Write(fsl_m_vid_out_write), 
    .FSL_M_Full(fsl_m_vid_out_full), 
    .FSL_S_Clk(fsl_s_memB_clk), 
    .FSL_S_Data(fsl_s_memB_data), 
    .FSL_S_Control(fsl_s_memB_cntrl), 
    .FSL_S_Read(fsl_s_memB_read), 
    .FSL_S_Exists(fsl_s_memB_exists), 
    .FSL_Full(fsl_2_full), 
    .FSL_Has_Data(fsl_2_has_data)
    );
    


//assign read_not_write = (pres_state == st_write_data) ? 1'b0 : 1'b1;
assign FSL_M_Full_A = 1'b0;
assign FSL_M_Full_B = 1'b0;
assign fsl_m_vid_in_clk = in_clk;
assign fsl_m_vid_out_clk = in_clk;

assign CS2b_1 = CS1b_1;
assign CS2b_0 = CS1b_0;
assign CS2_1 = 1'b1;
assign CS2_0 = 1'b1;

assign in_clk = core_clk;
assign fsl_rst = fsl_0_rst || fsl_2_rst;
assign rst = reset || fsl_rst;

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
     state_cntr <= 3'b000;
  else if ((pres_state == st_write_data_addr) && (next_state == st_start_write_data))
     state_cntr <= state_cntr +1;
  else
     state_cntr <= state_cntr;
end

always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1)
     pres_state <= st_write_data_instr;
  else
     pres_state <= next_state;
end

always @ (pres_state or state_cntr or delay_cntr or data_cntr or fsl_m_vid_in_write or rst ) begin
  case (pres_state)
     st_write_data_instr:
        begin
           if ((rst == 1'b0) && (fsl_m_vid_in_write == 1'b1))
              next_state = st_write_data_addr;
           else
	      next_state = st_write_data_instr;
        end
     st_write_data_addr:
        begin
           if (fsl_m_vid_in_write == 1'b1)
              next_state = st_start_write_data;
           else
	      next_state = st_write_data_addr;
        end
     st_start_write_data:
        begin
           if (fsl_m_vid_in_write == 1'b1)
              next_state = st_write_data;
           else
	      next_state = st_start_write_data;
        end
     st_write_data:
        begin
           if ((data_cntr[3:0] == 4'hf) && (state_cntr != 3'b000)
		   && (fsl_m_vid_in_write == 1'b1))
	      next_state = st_write_data_instr;
           else if ((data_cntr[3:0] == 4'hf) && (state_cntr == 3'b000)
		   && (fsl_m_vid_in_write == 1'b1))
              next_state = st_delay;
           else
	      next_state = st_write_data;
        end
     st_delay:
        begin
           if (delay_cntr == 8'hFF)
              next_state = st_write_data_instr;
           else
	      next_state = st_delay;
        end
  endcase
end

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
     addr_msb <= 2'b10;
  else if (delay_cntr == 8'hFE)
     addr_msb <= ~addr_msb;
  else
     addr_msb <= addr_msb;
end

always @ (pres_state or data_cntr or state_cntr or fsl_m_vid_in_full) begin
  case (pres_state)
    st_write_data_instr:
       begin
          fsl_m_vid_in_data = `wr_dat_instr;
	  fsl_m_vid_in_cntrl = 1'b1;
	  if (fsl_m_vid_in_full == 1'b0)
	    fsl_m_vid_in_write = 1'b1;
          else
	    fsl_m_vid_in_write = 1'b0;
	  //FSL_S_Exists_A = 1'b1;
       end
    st_write_data_addr:
       begin
       	 case (state_cntr)
	    3'b000:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_0_addr};
	    3'b001:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_20_addr};
	    3'b010:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_40_addr};
	    3'b011:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_60_addr};
	    3'b100:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_10_addr};
	    3'b101:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_30_addr};
	    3'b110:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_50_addr};
	    3'b111:
               fsl_m_vid_in_data = {addr_msb,`wr_dat_70_addr};
         endcase
	  fsl_m_vid_in_cntrl = 1'b0;
	  if (fsl_m_vid_in_full == 1'b0)
	    fsl_m_vid_in_write = 1'b1;
          else
	    fsl_m_vid_in_write = 1'b0;
	  //FSL_S_Exists_A = 1'b1;
       end
    st_start_write_data:
       begin
          fsl_m_vid_in_data = data_cntr;
	  fsl_m_vid_in_cntrl = 1'b0;
	  if (fsl_m_vid_in_full == 1'b0)
	    fsl_m_vid_in_write = 1'b1;
          else
	    fsl_m_vid_in_write = 1'b0;
	  //FSL_S_Exists_A = 1'b1;
       end
    st_write_data:
       begin
          fsl_m_vid_in_data = data_cntr;
	  fsl_m_vid_in_cntrl = 1'b0;
	  if (fsl_m_vid_in_full == 1'b0)
	    fsl_m_vid_in_write = 1'b1;
          else
	    fsl_m_vid_in_write = 1'b0;
	  //FSL_S_Exists_A = 1'b1;
       end
    st_delay:
       begin
          fsl_m_vid_in_data = 32'hFFFFFFFF;
	  fsl_m_vid_in_cntrl = 1'b0;
	  fsl_m_vid_in_write = 1'b0;
	  //FSL_S_Exists_A = 1'b0;
       end
  endcase
end

always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1) 
     delay_cntr <= 8'h00;
  //else if ((pres_state == st_write_data_instr) && (fsl_s_memB_read))
  else if (fsl_s_memB_read)
     delay_cntr <= 8'h00;
  else if (pres_state == st_delay)
     delay_cntr <= delay_cntr + 1;
  else
     delay_cntr <= delay_cntr;
end

always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1)
     begin
	fsl_m_vid_out_data <= 32'hAAAAAAAA;
	fsl_m_vid_out_cntrl <= 1'b0;
	fsl_m_vid_out_write <= 1'b0;
	//FSL_S_Exists_B <= 1'b0;
     end
  else if (delay_cntr == 8'hFF)
     begin
	fsl_m_vid_out_data <= `noop;
	fsl_m_vid_out_cntrl <= 1'b1;
	if (fsl_m_vid_out_full == 1'b0)
	  fsl_m_vid_out_write <= 1'b1;
        else
	  fsl_m_vid_out_write <= 1'b0;
	//FSL_S_Exists_B <= 1'b1;
     end
  else 
     begin
	fsl_m_vid_out_data <= 32'hAAAAAAAA;
	fsl_m_vid_out_cntrl <= 1'b0;
	fsl_m_vid_out_write <= 1'b0;
	//FSL_S_Exists_B <= 1'b0;
     end
end

always @ (posedge rst or posedge in_clk) begin
  //if ((rst == 1'b1) || (pres_state == st_delay))
  if (rst == 1'b1) 
     data_cntr <= 32'h00000000;
  else if (((pres_state == st_write_data) || 
	  (pres_state == st_start_write_data)) && (fsl_m_vid_in_write == 1'b1))
     data_cntr <= data_cntr + 1;
end

endmodule
