

module data_gen_for_fsl_xilinx_mem_controller
  (	
  reset,
  core_clk,
  in_mem_clk,
  //.in_addr(mem_addr),
  //.in_data(mem_in_data),
  //.fifo_empty(fifo_empty),
  //.fifo_full(fifo_full),
  //.read_not_write(read_not_write),
  //.enable(enable),

  ////Outputs from memory controller to fsl_controller:
  //.read_fifo(read_fifo),
  //.write_fifo(write_fifo),
  //.out_data(mem_out_data),
  //.running(running),
  //.done(done),

  //Connections to the memory bank: 
  Dq, 
  Addr, 
  mem_clk, 
  CKEb, 
  Bwa_n, 
  Bwb_n, 
  Bwc_n, 
  Bwd_n,
  WEb, 
  ADV, 
  OEb,                  
  CS1b, 
  //.CS2(CS2), 
  //.CS2b(CS2b), 
  LBOb, 
  ZZ
        );

input reset, core_clk, in_mem_clk;
inout[31:0] Dq; 
output [18:0] Addr; 
output mem_clk; 
output CKEb; 
output Bwa_n; 
output Bwb_n; 
output Bwc_n; 
output Bwd_n;
output WEb; 
output ADV; 
output OEb;                  
output CS1b; 
//output CS2(CS2); 
//output CS2b(CS2b); 
output LBOb; 
output ZZ;

//reg [31:0] inDq; //Data I/O- NOTE: the only input/output signal in the module
//wire [31:0] inDq; //Data I/O- NOTE: the only input/output signal in the module
wire [31:0] Dq; //Data I/O- NOTE: the only input/output signal in the module
//reg [31:0] Dq; //Data I/O- NOTE: the only input/output signal in the module
//reg high_imp_control;  //used to set inout data direction

wire [18:0] Addr; //Address
wire in_clk;     //clk from testbench to system
wire rst;  //used to startup the system in a predictable mode
//wire [18:0] mem_in_addr; //supply the address to the memory bank

//wire mem_ctrlr_clk;     //clk to the memory controller
wire in_mem_clk;     //clk (K on memory module)
wire mem_clk;     //clk (K on memory module)
//reg ADV;     //Address advance
wire ADV;     //Address advance
//reg CKEb;    //Slow down mode: acts as a clock enable
wire CKEb;    //Slow down mode: acts as a clock enable
wire WEb;     //Read is high and Write is Low aka "read, not write"
//reg Bwa_n;	//Byte Write Enable A
wire Bwa_n;	//Byte Write Enable A
//reg Bwb_n;	//Byte Write Enable B
wire Bwb_n;	//Byte Write Enable B
//reg Bwc_n;	//Byte Write Enable C
wire Bwc_n;	//Byte Write Enable C
//reg Bwd_n;	//Byte Write Enable D
wire Bwd_n;	//Byte Write Enable D

wire CS1b;	//Chip Enable
wire CS2;	//Chip Enable
wire CS2b;	//Chip Enable
wire OEb;	//Chip Output Enable
//reg ZZ;		//Chip Enable vs Sleep Mode
wire ZZ;	//Chip Enable vs Sleep Mode
wire LBOb;	//Used to select the automatic byte advance mode (ADV) where
		//LBOb = Linear Burst Order bar

//wire mem_rst;
wire [18:0] mem_addr;
wire [31:0] mem_in_data;
wire fifo_empty, fifo_full, enable; //, read_not_write;
wire read_not_write;
wire read_fifo, write_fifo, running, done;
wire [31:0] mem_out_data;
wire [12:0] addr_msb_req; 
reg [12:0] addr_msb_ctrl;
wire req_mem; 
reg control_mem0;
reg control_mem1;
reg prev_done;

//PROGRAM CONNECTIONS (INTERIOR TO THE FSL_WRAPPER CORE???
wire prog_clk, prog_rst, valid_instr, prog_cbit, prog_instr_read;
wire sbit_using_mem0;
wire [0:31] prog_instr;
wire FSL_S_Clk, FSL_S_Read; 
reg FSL_S_Control, FSL_S_Exists;
reg [31:0] FSL_S_Data;
wire FSL_M_Clk, FSL_M_Write, FSL_M_Control, FSL_M_Full;
wire [31:0] FSL_M_Data;
  
reg [31:0] data_cntr;
reg [11:0] delay_cntr;
reg [0:3] pres_state, next_state;
//reg [31:0] data_cntr;
//reg [18:0] addr_cntr;

//parameter st_reset=0;
parameter st_write_data_instr=0;
parameter st_write_data_addr=1;
parameter st_write_data=2;
parameter st_rx_data_instr=3;
parameter st_rx_data_addr=4;
parameter st_rx_data=5;
parameter st_read_req_data_instr=6;
parameter st_read_req_data_addr=7;
parameter st_delay=8;

`define write_data_instr 32'h00001007
`define write_data_addr 32'h80001230
`define rx_data_instr 32'h00000C05
//`define rx_data_addr 32'h80001270
`define rx_data_addr 32'h80001240
`define read_req_data_instr 32'h00001406
`define read_req_data_addr 32'h80001230
/*
//Memory model for actual zbt bank (0)
//**NOTE THESE WILL BE ACTUAL SYSTEM INPUTS AND OUTPUTS ON THE BOARD:
   K7N163601A K7N163601A (
  .Dq(Dq), 
  .Addr(Addr), 
  .K(mem_clk), 
  .CKEb(CKEb), 
  .Bwa_n(Bwa_n), 
  .Bwb_n(Bwb_n), 
  .Bwc_n(Bwc_n), 
  .Bwd_n(Bwd_n),
  .WEb(WEb), 
  .ADV(ADV), 
  .OEb(OEb),                  
  .CS1b(CS1b), 
  .CS2(CS2), 
  .CS2b(CS2b), 
  .LBOb(LBOb), 
  .ZZ(ZZ)
  );
*/
//My controller for a zbt bank:
  mem_bank_controller mem_bank_controller (
  //Inputs to memory controller from fsl_controller:   
  .reset(rst),
  .core_clk(core_clk),
  .in_mem_clk(in_mem_clk),
  .in_addr(mem_addr),
  .in_data(mem_in_data),
  .fifo_empty(fifo_empty),
  .fifo_full(fifo_full),
  .read_not_write(read_not_write),
  .enable(enable),

  //Outputs from memory controller to fsl_controller:
  .read_fifo(read_fifo),
  .write_fifo(write_fifo),
  .out_data(mem_out_data),
  .running(running),
  .done(done),

  //Connections to the memory bank: 
  .Dq(Dq), 
  .Addr(Addr), 
  .mem_clk(mem_clk), 
  .CKEb(CKEb), 
  .Bwa_n(Bwa_n), 
  .Bwb_n(Bwb_n), 
  .Bwc_n(Bwc_n), 
  .Bwd_n(Bwd_n),
  .WEb(WEb), 
  .ADV(ADV), 
  .OEb(OEb),                  
  .CS1b(CS1b), 
  //.CS2(CS2), 
  //.CS2b(CS2b), 
  .LBOb(LBOb), 
  .ZZ(ZZ)

  );

  fsl_mem_controller fsl_mem_controller (
//Input Connections for controller:
  .clk(core_clk),
  .rst(rst),
//Connections to Memory Controller:
  //Outputs:
  //.mem_clk(mem_ctrlr_clk), //y
  //.mem_rst(mem_rst), //y
  .mem_addr(mem_addr),
  .mem_in_data(mem_in_data), //y
  .fifo_empty(fifo_empty), //y
  .fifo_full(fifo_full), //y
  .read_not_write(read_not_write),
  .enable(enable),
  //Inputs:
  .read_fifo(read_fifo), //y
  .write_fifo(write_fifo), //y
  .mem_out_data(mem_out_data), //y
  .running(running),
  .done(done),

//Connected to arbiter:
  .addr_msb_req(addr_msb_req),
  .req_mem_arb(req_mem),  //status request bit to arbiter
  .control_mem0(control_mem0),  //status bit from arbiter
  .control_mem1(control_mem1),  //status bit from arbiter
  .addr_msb_ctrl(addr_msb_ctrl),
 

//Connections to the FSL Memory Controller Program:
  .prog_clk(prog_clk), //y
  .prog_rst(prog_rst), //y
  .prog_instr(prog_instr),
  .program_cbit(prog_cbit),
  .prog_instr_read(prog_instr_read),
  .valid_instr(valid_instr),
  //.sbit_using_mem0(sbit_using_mem0),  //status bit indicating mem0 in use by controller
  //**mem0_done //***should this be generic "mem" from controller?NECESSARY?


  // Slave FSL Signals
  .FSL_S_Clk(FSL_S_Clk), //y
  .FSL_S_Read(FSL_S_Read), //y
  .FSL_S_Data(FSL_S_Data), //y
  .FSL_S_Control(FSL_S_Control),
  .FSL_S_Exists(FSL_S_Exists), //y

  // Master FSL Signals
  .FSL_M_Clk(FSL_M_Clk), //y
  .FSL_M_Write(FSL_M_Write), //y
  .FSL_M_Data(FSL_M_Data), //y
  .FSL_M_Control(FSL_M_Control),
  .FSL_M_Full(FSL_M_Full) //y
);


fsl_mem0_program fsl_mem0_program (
  // inputs
  .clk(prog_clk), 
  .rst(prog_rst), 
  
  //Program Interface:
  .prog_instr(prog_instr),
  .program_cbit(prog_cbit),
  .prog_instr_read(prog_instr_read),
  .valid_instr(valid_instr),  //only when high should the instruction 
  					//be read (use to stall)
  .sbit_using_mem0(sbit_using_mem0) //Status_bit 0
  );


//assign read_not_write = (pres_state == st_write_data) ? 1'b0 : 1'b1;
assign FSL_M_Full = 1'b0;
assign CS2 = 1'b1;
assign CS2b = 1'b0;
assign rst = reset;
assign in_clk = core_clk;
assign sbit_using_mem0 = 1'b0; //for testing only

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
     prev_done <= 1'b1;
  else
     prev_done <= done;
end

//inside arbiter 
always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
     begin
       control_mem0 <= 1'b0;
       control_mem1 <= 1'b0;
       addr_msb_ctrl <= 13'h0000;
     end
  else if ((req_mem == 1'b1) && (addr_msb_req[12] == 1'b1) && (done == 1'b1))
     begin
       control_mem0 <= 1'b1;
       control_mem1 <= 1'b0;
       addr_msb_ctrl <= 13'h1000;
     end
  else if ((req_mem == 1'b0) && (done == 1'b1) && (prev_done == 1'b0))
     begin
       control_mem0 <= 1'b0;
       control_mem1 <= 1'b0;
       addr_msb_ctrl <= 13'h0000;
     end
  else
     begin
       control_mem0 <= control_mem0;
       control_mem1 <= 1'b0;
       addr_msb_ctrl <= addr_msb_ctrl;
     end
end


always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1)
     pres_state <= st_write_data_instr;
  else
     pres_state <= next_state;
end

always @ (pres_state or delay_cntr or data_cntr or FSL_S_Read) begin
  case (pres_state)
     st_write_data_instr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_write_data_addr;
           else
	      next_state = st_write_data_instr;
        end
     st_write_data_addr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_write_data;
           else
	      next_state = st_write_data_addr;
        end
     st_write_data:
        begin
           if (data_cntr == 32'h00000010)
              next_state = st_rx_data_instr;
           else
	      next_state = st_write_data;
        end
     st_rx_data_instr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_rx_data_addr;
           else
	      next_state = st_rx_data_instr;
        end
     st_rx_data_addr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_rx_data;
           else
	      next_state = st_rx_data_addr;
        end
     st_rx_data:
        begin
           if (data_cntr == 32'h0000001C)
              next_state = st_read_req_data_instr;
           else
	      next_state = st_rx_data;
        end
     st_read_req_data_instr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_read_req_data_addr;
           else
	      next_state = st_read_req_data_instr;
        end
     st_read_req_data_addr:
        begin
           if (FSL_S_Read == 1'b1)
              next_state = st_delay;
           else
	      next_state = st_read_req_data_addr;
        end
     st_delay:
        begin
           if (delay_cntr == 12'hFFF)
              next_state = st_write_data_instr;
           else
	      next_state = st_delay;
        end
  endcase
end
 
always @ (pres_state or data_cntr) begin
  case (pres_state)
    st_write_data_instr:
       begin
          FSL_S_Data = `write_data_instr;
	  FSL_S_Control = 1'b1;
	  FSL_S_Exists = 1'b1;
       end
    st_write_data_addr:
       begin
          FSL_S_Data = `write_data_addr;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b1;
       end
    st_write_data:
       begin
          FSL_S_Data = data_cntr;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b1;
       end
    st_rx_data_instr:
       begin
          FSL_S_Data = `rx_data_instr;
	  FSL_S_Control = 1'b1;
	  FSL_S_Exists = 1'b1;
       end
    st_rx_data_addr:
       begin
          FSL_S_Data = `rx_data_addr;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b1;
       end
    st_rx_data:
       begin
          FSL_S_Data = data_cntr;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b1;
       end
    st_read_req_data_instr:
       begin
          FSL_S_Data = `read_req_data_instr;
	  FSL_S_Control = 1'b1;
	  FSL_S_Exists = 1'b1;
       end
    st_read_req_data_addr:
       begin
          FSL_S_Data = `read_req_data_addr;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b1;
       end
    st_delay:
       begin
          FSL_S_Data = 32'hFFFFFFFF;
	  FSL_S_Control = 1'b0;
	  FSL_S_Exists = 1'b0;
       end
  endcase
end

always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1)
     delay_cntr <= 16'h000;
  else if (pres_state == st_write_data_instr)
     delay_cntr <= 16'h000;
  else if (pres_state == st_delay)
     delay_cntr <= delay_cntr + 1;
  else
     delay_cntr <= delay_cntr;
end

always @ (posedge rst or posedge in_clk) begin
  if (rst == 1'b1) 
     data_cntr <= 32'h00000000;
  else if (pres_state == st_write_data_instr)
     data_cntr <= 32'h00000000;
  else if (((pres_state == st_write_data) || (pres_state == st_rx_data))
	  && (FSL_S_Read == 1'b1))
     data_cntr <= data_cntr + 1;
end

/*
always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
    enable <= 1'b0;
  else if (in_addr[3:0] == 4'b1010)
    enable <= ~enable;
  else if (next_state == st_dead_state)
    enable <= 1'b0;
  else
    enable <= 1'b1;
end*/

/*always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
    fifo_empty <= 1'b0;
  else if ((in_addr[2:0] == 3'b110) || (in_data[2:0] == 3'b111))
    fifo_empty <= ~fifo_empty;
  else
    fifo_empty <= 1'b0;
end

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
    fifo_full <= 1'b0;
  else if ((in_addr[2:0] == 3'b100) || (in_data[2:0] == 3'b011))
    fifo_full <= ~fifo_full;
  else
    fifo_full <= 1'b0;
end

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
    in_data <= 32'h12345670;
  else if ((running == 1'b1) && (read_fifo == 1'b1))
    in_data <= in_data + 1;
  else
    in_data <= in_data;
end

always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
    in_addr <= 32'h00000001;
  else if ((pres_state == st_write_data) && (next_state == st_read_data))
    in_addr <= in_addr - 15;
  else if ((((pres_state == st_write_data) && (next_state == st_write_data)) ||
	  ((pres_state == st_read_data))) &&
	  (running == 1'b1) && (enable == 1'b1))
    in_addr <= in_addr + 1;
  else
    in_addr <= in_addr;
end
*/
/*
//Set present state for FSL_S_Data input
always @ (posedge in_clk or posedge rst) begin
  if (rst == 1'b1)
	  pres_state <= st_reset;
  else
	  pres_state <= next_state;
end

//Set next state for FSL_S_Data input
always @ (in_addr or rst or pres_state) begin
  case (pres_state)
    st_reset:
      begin
        if (rst == 1'b1)
	       next_state = st_reset;
	else 
	       next_state = st_write_data;
      end
    st_write_data:
      begin
        if ((running == 1'b1) && (in_addr[3:0] == 4'b0000))
	       next_state = st_read_data;
        else
	       next_state = st_write_data;
      end
    st_read_data:
      begin
        if ((running == 1'b1) && (in_addr[3:0] == 4'b0000))
	       next_state = st_dead_state;
        else
	       next_state = st_read_data;
      end
    st_dead_state:
      next_state = st_write_data;
  endcase
end
*/

   
endmodule
