//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: fsl_video_output_program.v
//    Used by: fsl_BM_MODE_SVGA_CTRL_532.v
//    Works with ProjectNavigator 6.2.03i
//    Description: Provides the instructions to be executed by the video
//    controller.  It should only be necessary for a user to change the 
//    program to suit the application- the controller should remain
//    unchanged
//
//
//    LLS November 2004
//************************************************************************ 
//
 

module fsl_mem0_program (
  // inputs
  clk, rst, 
  
  //Program Interface:
  prog_instr,
  program_cbit,
  prog_instr_read,
  valid_instr,  //only when high should the instruction be read (use to stall)
  sbit_using_mem0 //Status_bit 0
  );

  input clk, rst;
  output [0:31] prog_instr;  //Next instruction to be executed by the controller
  output program_cbit;
  output valid_instr;
  input prog_instr_read; //Read the instruction on the prog_instr bus
  input sbit_using_mem0; //Status_bit 0
	  
  wire clk, rst;

  //Program interface:
  wire [0:31] prog_instr;
  wire program_cbit;
  wire prog_instr_read; 
  reg valid_instr; 

  wire sbit_using_mem0;
  reg [4:0] pres_state, next_state;
  reg [0:31] int_prog_instr;
  reg int_prog_cbit;

//Test instructions:
//`define write_imm 32'h00000703
//`define read_req_imm 32'h00000702
//`define receive_imm 32'h00000701
`define wr_abs_imm 32'h00000707
`define wr_abs_addr 32'h80001230
`define rd_req_abs_imm 32'h00000706
`define rd_req_abs_addr 32'h80001250
//`define receive_abs_imm 32'h00000705
`define wr_a0 32'h000000B
`define wr_new_a0 32'h80001230
//`define a0_new_addr 32'h001100AA
`define add_a0 32'h00000618
`define sub_a0 32'h00000E08
`define wr_autoinc 32'h0000071F
`define wr_indirect 32'h0000070F
`define rd_autoinc 32'h0000071E
`define rd_indirect 32'h0000070E
`define noop 32'h00000004
`define reset 32'h00000000
`define wait_rx 32'h0000000C
//NEW INSTRUCTION TO ADD??
`define set_clear_status_bits 32'h00700713 //0-11 indicate new status bit values
					   //12-23 indicate bits to be changed
					   //24-31 gives the instr opcode

//TEST STATES
// NOTE: The states reflect the program structure
//parameter  st_wr_imm=0;
parameter  st_wr_abs_imm=0;
parameter  st_wr_abs_addr=1;
//parameter  st_rx_abs_imm=2;
//parameter  st_rx_abs_addr=3;
parameter  st_rd_req_abs_imm=2;
parameter  st_rd_req_abs_addr=3;
parameter  st_wr_a0=4;
parameter  st_wr_new_a0=5;
parameter  st_add_a0=6;
parameter  st_sub_a0=7;
parameter  st_wr_autoinc=8;
parameter  st_wr_indirect=9;
parameter  st_rd_autoinc=10;
parameter  st_rd_indirect=11;
parameter  st_noop=12;
parameter  st_reset=13;
parameter  st_wait_rx=14;
parameter  st_set_clear_status=15;

/*
// NOTE: The states reflect the program structure
parameter  st_wr_addr_a0=0;
parameter  st_wr_addr_bank0=1;
parameter  st_wr_addr_bank1=2;
parameter  st_wr_read_req_a0_inc=3;
parameter  st_wait_rx=4;
parameter  st_stall=5;
*/

////`define line_length 10'd640 
////ACTUAL PROGRAM:
//`define wr_addr_a0 32'h0000000B //Write base address to a0
//`define addr_bank0 32'h00000000 //Odd Base address for bank 0
//`define addr_bank1 32'h80000000 //Even Base address for bank 0
//`define read_req_a0_inc 32'h0002801E //Request data from memory address a0 
					//and post increment
//`define wait_rx 32'h0000000C //Wait until the data comes before
					//processing
assign prog_instr = int_prog_instr;
assign program_cbit = int_prog_cbit;

//Used to indicate when the instruction is valid.  Allows us to stall the
//processor when we don't know for sure the next state
/*always @ (pres_state or prefetch_line) begin
  case(pres_state)
    st_wr_addr_a0:
      valid_instr = 1'b1;
    st_wr_addr_bank0:
      valid_instr = 1'b1;
    st_wr_addr_bank1:
      valid_instr = 1'b1;
    st_wr_read_req_a0_inc:
      //begin
	//if (prefetch_line == 1'b1)
          valid_instr = 1'b1;
	//else 
          //valid_instr = 1'b0;
      //end
    st_wait_rx:
      valid_instr = 1'b1;
    st_stall:
      valid_instr = 1'b0;
    default:
      valid_instr = 1'b0;
  endcase
end*/

// FSM register
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    pres_state <= st_wr_abs_imm;
  else
    pres_state <= next_state;
end

/*//Next-state state machine for the program:
always @ (pres_state or prog_instr_read or prefetch_line or new_frame or choose_bank or rst or prefetch_line_prev or new_frame_prev) 
begin

  case (pres_state)
    st_wr_addr_a0:
      begin
      	if ((prog_instr_read == 1'b1) && (rst == 1'b0)) begin
	  if (choose_bank == 1'b0) 
            next_state = st_wr_addr_bank0;
	  else 
            next_state = st_wr_addr_bank1;
          end
        else
	  next_state = st_wr_addr_a0;
      end
    st_wr_addr_bank0:
      begin 
	if (prog_instr_read == 1'b1) 
          next_state = st_wr_read_req_a0_inc;
	else
          next_state = st_wr_addr_bank0;
      end
    st_wr_addr_bank1:
      begin 
	if (prog_instr_read == 1'b1) 
          next_state = st_wr_read_req_a0_inc;
	else
          next_state = st_wr_addr_bank1;
      end
    st_wr_read_req_a0_inc:
      begin 
        if ((new_frame == 1'b1) && (new_frame_prev == 1'b0)) //**this may not be
	  next_state = st_wr_addr_a0;	//only jump from stall??**
	else if (prog_instr_read == 1'b1) 
	  next_state = st_wait_rx;
        else
          next_state = st_wr_read_req_a0_inc;
      end
    st_wait_rx:
      begin 
	if (prog_instr_read == 1'b1) 
	  next_state = st_stall;
        else
          next_state = st_wait_rx;
      end
    st_stall:
      begin
	if ((prefetch_line == 1'b1) && (prefetch_line_prev == 1'b0))
	  next_state = st_wr_read_req_a0_inc;
        else if ((new_frame == 1'b1) && (new_frame_prev == 1'b0))
          next_state = st_wr_addr_a0;
        else
          next_state = st_stall;
      end
    default:
      next_state = st_wr_addr_a0;
  endcase
end
*/

/*//Sets the current instruction output to the controller based on the state:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1) 
    begin
      int_prog_instr <= `wr_addr_a0;
      int_prog_cbit <= 1'b1;
    end	  
  else
    begin
      case (next_state) 
        st_wr_addr_a0:
          begin
      	    int_prog_instr <= `wr_addr_a0;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_addr_bank0:
          begin
      	    int_prog_instr <= `addr_bank0;
	    int_prog_cbit <= 1'b0;
          end
        st_wr_addr_bank1:
          begin
      	    int_prog_instr <= `addr_bank1;
	    int_prog_cbit <= 1'b0;
          end
        st_wr_read_req_a0_inc:
          begin
      	    int_prog_instr <= `read_req_a0_inc;
	    int_prog_cbit <= 1'b1;
          end
        st_wait_rx:
          begin
      	    int_prog_instr <= `wait_rx;
	    int_prog_cbit <= 1'b1;
          end
        st_stall:
          begin
      	    int_prog_instr <= 32'h00000000;
	    int_prog_cbit <= 1'b0;
          end
        default:
          begin
      	    int_prog_instr <= `wr_addr_a0;
	    int_prog_cbit <= 1'b1;
          end
      endcase
    end
end
*/

//Next state state machine for test program:
always @ (pres_state or prog_instr_read or rst) begin
  case (pres_state) 
    st_wr_abs_imm:
      begin
        if ((prog_instr_read == 1'b1) && (rst == 1'b0))
		next_state <= st_wr_abs_addr;
        else
		next_state <= st_wr_abs_imm;
      end
    st_wr_abs_addr:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_rd_req_abs_imm;
        else
		next_state <= st_wr_abs_addr;
      end
    st_rd_req_abs_imm:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_rd_req_abs_addr; 
        else
		next_state <= st_rd_req_abs_imm; 
      end
    st_rd_req_abs_addr:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_a0;
        else
		next_state <= st_rd_req_abs_addr; 
      end
    st_wr_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_new_a0;
        else
		next_state <= st_wr_a0;
      end
    st_wr_new_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_add_a0;
        else
		next_state <= st_wr_new_a0;
      end
    st_add_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_autoinc;
        else
		next_state <= st_add_a0;
      end
    st_wr_autoinc:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_indirect;
        else
		next_state <= st_wr_autoinc;
      end
    st_wr_indirect:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_sub_a0;
        else
		next_state <= st_wr_indirect;
      end
    st_sub_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_rd_autoinc;
        else
		next_state <= st_sub_a0;
      end
    st_rd_autoinc:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_rd_indirect;
        else
		next_state <= st_rd_autoinc;
      end
    st_rd_indirect:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_noop;
        else
		next_state <= st_rd_indirect;
      end
    st_noop:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_reset;
        else
		next_state <= st_noop;
      end
    st_reset:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wait_rx;
        else
		next_state <= st_reset;
      end
    st_wait_rx:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_abs_imm;
        else
		next_state <= st_wait_rx;
      end
    default:
      next_state <= st_rd_req_abs_imm;
  endcase
end

//TEST PROGRAM:
//Sets the current instruction output to the controller based on the state:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    begin
      int_prog_instr <= `wr_abs_imm;
      int_prog_cbit <= 1'b1;
    end
  else
    begin
      case (next_state) 
        st_wr_abs_imm:
          begin
      	    int_prog_instr <= `wr_abs_imm;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_abs_addr:
          begin
      	    int_prog_instr <= `wr_abs_addr;
	    int_prog_cbit <= 1'b0;
          end
        st_rd_req_abs_imm:
          begin
      	    int_prog_instr <= `rd_req_abs_imm;
	    int_prog_cbit <= 1'b1;
          end
        st_rd_req_abs_addr:
          begin
      	    int_prog_instr <= `rd_req_abs_addr;
	    int_prog_cbit <= 1'b0;
          end
        st_wr_a0:
          begin
      	    int_prog_instr <= `wr_a0;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_new_a0:
          begin
      	    int_prog_instr <= `wr_new_a0;
	    int_prog_cbit <= 1'b0;
          end
        st_add_a0:
          begin
      	    int_prog_instr <= `add_a0;
	    int_prog_cbit <= 1'b1;
          end
        st_sub_a0:
          begin
      	    int_prog_instr <= `sub_a0;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_autoinc:
          begin
      	    int_prog_instr <= `wr_autoinc;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_indirect:
          begin
      	    int_prog_instr <= `wr_indirect;
	    int_prog_cbit <= 1'b1;
          end
        st_rd_autoinc:
          begin
      	    int_prog_instr <= `rd_autoinc;
	    int_prog_cbit <= 1'b1;
          end
        st_rd_indirect:
          begin
      	    int_prog_instr <= `rd_indirect;
	    int_prog_cbit <= 1'b1;
          end
        st_noop:
          begin
      	    int_prog_instr <= `noop;
	    int_prog_cbit <= 1'b1;
          end
        st_reset:
          begin
      	    int_prog_instr <= `reset;
	    int_prog_cbit <= 1'b1;
          end
        st_wait_rx:
          begin
      	    int_prog_instr <= `wait_rx;
	    int_prog_cbit <= 1'b1;
          end
        default:
          begin
      	    int_prog_instr <= `noop; //`write_imm;
	    int_prog_cbit <= 1'b1;
          end
      endcase
    end
end

//FOR TESTING:
//Used to indicate when the instruction is valid.  Allows us to stall the
//processor when we don't know for sure the next state
always @ (posedge rst or posedge clk) begin
  if (rst == 1'b1)
    valid_instr = 1'b0;
  else
    valid_instr = 1'b1;
  /*case(pres_state)
    st_read_req_imm:
      valid_instr = 1'b1;
    st_read_req_abs_imm:
      valid_instr = 1'b1;
    st_wr_addr:
      valid_instr = 1'b1;
    st_wr_a0:
      valid_instr = 1'b1;
    st_wr_new_a0:
      valid_instr = 1'b1;
    st_add_a0:
      valid_instr = 1'b1;
    st_sub_a0:
      valid_instr = 1'b1;
    st_wr_indirect:
      valid_instr = 1'b1;
    st_wr_autoinc:
      valid_instr = 1'b1;
    st_noop:
      valid_instr = 1'b1;
    st_reset:
      valid_instr = 1'b1;
    st_wait_rx:
      valid_instr = 1'b1;
  endcase*/
end

endmodule
