//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: fsl_video_output_program.v
//    Used by: fsl_BM_MODE_SVGA_CTRL_532.v
//    Works with ProjectNavigator 6.2.03i
//    Description: Provides the instructions to be executed by the video
//    controller.  It should only be necessary for a user to change the 
//    program to suit the application- the controller should remain
//    unchanged
//
//
//    LLS November 2004
//************************************************************************ 
//
 

module fsl_mem_ctlrB_program (
  // inputs
  clk, rst, 
  
  //Program Interface:
  prog_instr,
  program_cbit,
  prog_instr_read,
  valid_instr,  //only when high should the instruction be read (use to stall)
  sbitA_using_mem0,  //status bit 0: mem0 used by controller 0
  sbitA_using_mem1,  //status bit 1: mem1 used by controller 0
  sbitB_using_mem0,  //status bit 2: mem0 used by controller 1
  sbitB_using_mem1  //status bit 3: mem1 used by controller 1
  );

  input clk, rst;
  output [0:31] prog_instr;  //Next instruction to be executed by the controller
  output program_cbit;
  output valid_instr;
  input prog_instr_read; //Read the instruction on the prog_instr bus
  input sbitA_using_mem0; //Status_bit 0
  input sbitA_using_mem1; //Status_bit 1
  input sbitB_using_mem0; //Status_bit 2
  input sbitB_using_mem1; //Status_bit 3
	  
  wire clk, rst;

  //Program interface:
  wire [0:31] prog_instr;
  wire program_cbit;
  wire prog_instr_read; 
  reg valid_instr; 

  wire sbitA_using_mem0, sbitA_using_mem1;
  wire sbitB_using_mem0, sbitB_using_mem1;
  reg [2:0] pres_state, next_state;
  //reg pres_state, next_state;
  reg [0:31] int_prog_instr;
  reg int_prog_cbit;

  reg [0:1] addr_msb;
  //reg [0:7] delay_cntr;
  reg [0:7] addr_cntr;
  //wire [0:1] addr_msb;
  //wire [0:7] delay_cntr;
  //wire [0:7] addr_cntr;

//Test instructions:
//`define write_imm 32'h00000703
//`define read_req_imm 32'h00000702
//`define receive_imm 32'h00000701
`define wr_a0 32'h000000B
`define wr_new_a0 30'h00000000
`define wr_autoinc 32'h0000101F
`define wait_rx 32'h0000000C
`define noop 32'h00000004
//`define wr_abs_imm 32'h00000707
//`define wr_abs_addr 32'h80001230
//`define add_a0 32'h00000618
//`define sub_a0 32'h00000d08
//`define wr_indirect 32'h0000070F
//`define rd_autoinc 32'h0000071E
//`define rd_indirect 32'h0000070E
//NEW INSTRUCTION TO ADD??
`define set_clear_status_bits 32'h00700713 //0-11 indicate new status bit values
					   //12-23 indicate bits to be changed
					   //24-31 gives the instr opcode

//TEST STATES
// NOTE: The states reflect the program structure
//parameter  st_wr_imm=0;
//parameter  st_wr_abs_imm=0;
//parameter  st_wr_abs_addr=1;
//parameter  st_rx_abs_imm=2;
//parameter  st_rx_abs_addr=3;
//parameter  st_rd_req_abs_imm=2;
//parameter  st_rd_req_abs_addr=3;
//parameter  st_add_a0=6;
//parameter  st_sub_a0=7;
//parameter  st_wr_indirect=9;
//parameter  st_rd_autoinc=10;
//parameter  st_rd_indirect=11;
//parameter  st_noop=12;
//parameter  st_reset=13;
//parameter  st_set_clear_status=15;


// NOTE: The states reflect a valid program structure
parameter  st_wr_a0=0;
parameter  st_wr_new_a0=1;
parameter  st_wr_autoinc=2;
parameter  st_wait_rx=3;


parameter st_reset=0;
parameter st_stall=1;
////`define line_length 10'd640 
////ACTUAL PROGRAM:
//`define wr_addr_a0 32'h0000000B //Write base address to a0
//`define addr_bank0 32'h00000000 //Odd Base address for bank 0
//`define addr_bank1 32'h80000000 //Even Base address for bank 0
//`define read_req_a0_inc 32'h0002801E //Request data from memory address a0 
					//and post increment
//`define wait_rx 32'h0000000C //Wait until the data comes before
					//processing
//TEMP?? use regs??
//assign addr_msb = 2'b00;
//assign delay_cntr = 8'h00;
//assign addr_cntr = 8'h00;

assign prog_instr = int_prog_instr;
assign program_cbit = int_prog_cbit;

//Used to indicate when the instruction is valid.  Allows us to stall the
//processor when we don't know for sure the next state
always @ (pres_state or sbitA_using_mem0 or sbitA_using_mem1) begin
  case(pres_state)
    //st_reset:
    st_wr_a0:
      begin
        if ((sbitA_using_mem0 == 1'b1) || (sbitA_using_mem1 == 1'b1))
          valid_instr = 1'b1;
        else
          valid_instr = 1'b0;
      end
    //st_stall:
    st_wr_new_a0:
      //valid_instr = 1'b0;
      valid_instr = 1'b1;
    st_wr_autoinc:
      valid_instr = 1'b1;
    st_wait_rx:
      valid_instr = 1'b1;
    default:
      valid_instr = 1'b0;
  endcase
end

// FSM register
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    //pres_state <= st_reset;
    pres_state <= st_wr_a0;
  else
    pres_state <= next_state;
end

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    addr_msb <= 2'b01;
  else if ((pres_state == st_wr_new_a0) && (prog_instr_read == 1'b1))
    addr_msb <=  ~addr_msb;
  else
    addr_msb <= addr_msb;
end

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    addr_cntr <= 8'h00;
  else if (pres_state == st_wr_a0)
    addr_cntr <= 8'h00;
  else if ((pres_state == st_wr_autoinc) && (prog_instr_read))
    addr_cntr <= addr_cntr + 16;
  else
    addr_cntr <= addr_cntr;
end

//Next-state state machine for the program:
always @ (pres_state or prog_instr_read or sbitA_using_mem0 or sbitA_using_mem1  or rst or addr_cntr) 
begin
  case (pres_state)
//    st_reset:
    st_wr_a0:
      begin
      	if ((prog_instr_read == 1'b1) && (rst == 1'b0)) 
            //next_state = st_stall;
          next_state = st_wr_new_a0;
        else 
            //next_state = st_reset;
          next_state = st_wr_a0;
      end
    //st_stall:
    st_wr_new_a0:
      begin 
	if (prog_instr_read == 1'b1) 
          next_state = st_wr_autoinc;
	else
          //next_state = st_stall;
          next_state = st_wr_new_a0;
      end
    st_wr_autoinc:
      begin 
	if (addr_cntr == 8'h80) 
          next_state = st_wait_rx;
	else
          next_state = st_wr_autoinc;
      end
    st_wait_rx:
      begin 
	if (prog_instr_read == 1'b1) 
	  next_state = st_wr_a0;
        else
          next_state = st_wait_rx;
      end
    default:
      next_state = st_wr_a0;
  endcase
end

//Simulation program:
//Sets the current instruction output to the controller based on the state:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1) 
    begin
      //int_prog_instr <= `noop;
      int_prog_instr <= `wr_a0;
      int_prog_cbit <= 1'b1;
    end	  
  else
    begin
      case (next_state) 
        //st_reset:
	st_wr_a0:
          begin
      	    //int_prog_instr <= `noop;
      	    int_prog_instr <= `wr_a0;
	    int_prog_cbit <= 1'b1;
          end
        //st_stall:
	st_wr_new_a0:
          begin
      	    int_prog_instr <= {addr_msb,`wr_new_a0};
	    int_prog_cbit <= 1'b0;
      	    //int_prog_instr <= 32'h00000000;
	    //int_prog_cbit <= 1'b0;
          end
        st_wr_autoinc:
          begin
      	    int_prog_instr <= `wr_autoinc;
	    int_prog_cbit <= 1'b1;
          end
        st_wait_rx:
          begin
      	    int_prog_instr <= `wait_rx;
	    int_prog_cbit <= 1'b1;
          end
        default:
          begin
      	    int_prog_instr <= `wr_a0;
	    int_prog_cbit <= 1'b1;
          end
      endcase
    end
end

endmodule
