
module fsl_sh_mem_controller
  (	
  core_clk,
  mem_clk,
  rst,

  led0,
  led1,

  //Connections to the memory bank 0: 
  Dq_0, 
  Addr_0, 
  mem_clk_0, 
  CKEb_0, 
  Bwa_n_0, 
  Bwb_n_0, 
  Bwc_n_0, 
  Bwd_n_0,
  WEb_0, 
  ADV_0, 
  OEb_0,                  
  CS1b_0, 
  //CS2_0, 
  //CS2b_0, 
  LBOb_0, 
  ZZ_0,

  //Connections to the memory bank 1: 
  Dq_1, 
  Addr_1, 
  mem_clk_1, 
  CKEb_1, 
  Bwa_n_1, 
  Bwb_n_1, 
  Bwc_n_1, 
  Bwd_n_1,
  WEb_1, 
  ADV_1, 
  OEb_1,                  
  CS1b_1, 
  //CS2_1, 
  //CS2b_1, 
  LBOb_1, 
  ZZ_1,

  //FSL A:    
  // Slave FSL Signals
  FSL_S_Clk_A,
  FSL_S_Read_A,
  FSL_S_Data_A,
  FSL_S_Control_A,
  FSL_S_Exists_A,
  // Master FSL Signals
  FSL_M_Clk_A,
  FSL_M_Write_A,
  FSL_M_Data_A,
  FSL_M_Control_A,
  FSL_M_Full_A,

  //FSL B:
  // Slave FSL Signals
  FSL_S_Clk_B,
  FSL_S_Read_B,
  FSL_S_Data_B,
  FSL_S_Control_B,
  FSL_S_Exists_B,
  // Master FSL Signals
  FSL_M_Clk_B,
  FSL_M_Write_B,
  FSL_M_Data_B,
  FSL_M_Control_B,
  FSL_M_Full_B

    );

input  core_clk, mem_clk, rst;

output led0, led1;

inout [31:0]  Dq_0, Dq_1;
output [18:0] Addr_0, Addr_1;
output mem_clk_0, mem_clk_1;
output CKEb_0, CKEb_1;
output Bwa_n_0, Bwa_n_1;
output Bwb_n_0, Bwb_n_1;
output Bwc_n_0, Bwc_n_1;
output Bwd_n_0, Bwd_n_1;
output WEb_0, WEb_1;
output ADV_0, ADV_1;
output OEb_0, OEb_1;
output CS1b_0, CS1b_1;
//output CS2_0, CS2_1;
//output CS2b_0, CS2b_1;
output LBOb_0, LBOb_1;
output ZZ_0, ZZ_1;

output FSL_S_Clk_A, FSL_S_Clk_B;
output FSL_S_Read_A, FSL_S_Read_B;
input [0:31] FSL_S_Data_A, FSL_S_Data_B;
input  FSL_S_Control_A, FSL_S_Control_B;
input  FSL_S_Exists_A, FSL_S_Exists_B;
output FSL_M_Clk_A, FSL_M_Clk_B;
output FSL_M_Write_A, FSL_M_Write_B;
output [0:31] FSL_M_Data_A, FSL_M_Data_B;
output FSL_M_Control_A, FSL_M_Control_B;
input  FSL_M_Full_A, FSL_M_Full_B;

wire core_clk;     //clk from testbench to system
wire mem_clk;     //clk from testbench to system
wire rst;  //used to startup the system in a predictable mode

wire led0, led1;
wire led0_unused, led1_unused;

wire [31:0] Dq_0, Dq_1; //Data I/O- 
			//NOTE: the only input/output signal in the module
wire [18:0] Addr_0, Addr_1; //Address
wire ADV_0, ADV_1;     //Address advance
wire CKEb_0, CKEb_1;    //Slow down mode: acts as a clock enable
wire WEb_0, WEb_1;     //Read is high and Write is Low aka "read, not write"
wire Bwa_n_0, Bwa_n_1;	//Byte Write Enable A
wire Bwb_n_0, Bwb_n_1;	//Byte Write Enable B
wire Bwc_n_0, Bwc_n_1;	//Byte Write Enable C
wire Bwd_n_0, Bwd_n_1;	//Byte Write Enable D
wire CS1b_0, CS1b_1;	//Chip Enable
//wire CS2_0, CS2_1;	//Chip Enable
//wire CS2b_0, CS2b_1;	//Chip Enable
wire OEb_0, OEb_1;	//Chip Output Enable
wire ZZ_0, ZZ_1;	//Chip Enable vs Sleep Mode
wire LBOb_0, LBOb_1;	//Used to select the auto byte advance mode (ADV) where
			//LBOb = Linear Burst Order bar
wire FSL_S_Clk_A, FSL_S_Clk_B;
wire FSL_S_Read_A, FSL_S_Read_B;
wire [0:31] FSL_S_Data_A, FSL_S_Data_B;
wire  FSL_S_Control_A, FSL_S_Control_B;
wire  FSL_S_Exists_A, FSL_S_Exists_B;
wire FSL_M_Clk_A, FSL_M_Clk_B;
wire FSL_M_Write_A, FSL_M_Write_B;
wire [0:31] FSL_M_Data_A, FSL_M_Data_B;
wire FSL_M_Control_A, FSL_M_Control_B;
wire  FSL_M_Full_A, FSL_M_Full_B;

//Internal signals:
wire [18:0] mem_addr_A, mem_addr_B;
wire [31:0] mem_in_data_A, mem_in_data_B;
wire fifo_empty_A, fifo_full_A, enable_A, read_not_write_A;
wire fifo_empty_B, fifo_full_B, enable_B, read_not_write_B;
wire read_fifo_A, write_fifo_A, running_A, done_A;
wire read_fifo_B, write_fifo_B, running_B, done_B;
wire [31:0] mem_out_data_A, mem_out_data_B;

wire [18:0] mem_addr_0, mem_addr_1;
wire [31:0] mem_in_data_0, mem_in_data_1;
wire fifo_empty_0, fifo_full_0, enable_0, read_not_write_0;
wire fifo_empty_1, fifo_full_1, enable_1, read_not_write_1;
wire read_fifo_0, write_fifo_0, running_0, done_0;
wire read_fifo_1, write_fifo_1, running_1, done_1;
wire [31:0] mem_out_data_0, mem_out_data_1;

//arbiter connections
wire [12:0] addr_msb_req_A, addr_msb_req_B; 
wire [12:0] addr_msb_ctrl_A, addr_msb_ctrl_B;
wire req_mem_A, req_mem_B; 

//program connections: 
wire prog_clk_A, prog_rst_A, valid_instr_A, prog_cbit_A, prog_instr_read_A;
wire [0:31] prog_instr_A;
wire prog_clk_B, prog_rst_B, valid_instr_B, prog_cbit_B, prog_instr_read_B;
wire [0:31] prog_instr_B;
wire sbitA_using_mem0, sbitA_using_mem1, sbitB_using_mem0, sbitB_using_mem1;

//***SIGNALS USED TO MULTIPLEX CONNECTIONS BETWEEN FSL_CONTROLLERS AND
//***DEDICATED MEMORY CONTROLLERS:
reg sel_in_mem0;
//reg sel_in_mem1;
reg sel_out_ctlrA;
//reg sel_out_ctlrB;



//My controller for zbt bank 0:
  mem_bank_controller mem_bank_controller_0 (
  //Inputs to memory controller from fsl_controller:   
  .reset(rst),
  .core_clk(core_clk),
  .in_mem_clk(mem_clk),
  .in_addr(mem_addr_0),
  .in_data(mem_in_data_0),
  .fifo_empty(fifo_empty_0),
  .fifo_full(fifo_full_0),
  .read_not_write(read_not_write_0),
  .enable(enable_0),

  //Outputs from memory controller to fsl_controller:
  .read_fifo(read_fifo_0),
  .write_fifo(write_fifo_0),
  .out_data(mem_out_data_0),
  .running(running_0),
  .done(done_0),

  //Connections to the memory bank 0: 
  .Dq(Dq_0), 
  .Addr(Addr_0), 
  .mem_clk(mem_clk_0), 
  .CKEb(CKEb_0), 
  .Bwa_n(Bwa_n_0), 
  .Bwb_n(Bwb_n_0), 
  .Bwc_n(Bwc_n_0), 
  .Bwd_n(Bwd_n_0),
  .WEb(WEb_0), 
  .ADV(ADV_0), 
  .OEb(OEb_0),                  
  .CS1b(CS1b_0), 
  //.CS2(CS2_0), 
  //.CS2b(CS2b_0), 
  .LBOb(LBOb_0), 
  .ZZ(ZZ_0)

  );

//My controller for zbt bank 1:
  mem_bank_controller mem_bank_controller_1 (
  //Inputs to memory controller from fsl_controller:   
  .reset(rst),
  .core_clk(core_clk),
  .in_mem_clk(mem_clk),
  .in_addr(mem_addr_1),
  .in_data(mem_in_data_1),
  .fifo_empty(fifo_empty_1),
  .fifo_full(fifo_full_1),
  .read_not_write(read_not_write_1),
  .enable(enable_1),

  //Outputs from memory controller to fsl_controller:
  .read_fifo(read_fifo_1),
  .write_fifo(write_fifo_1),
  .out_data(mem_out_data_1),
  .running(running_1),
  .done(done_1),

  //Connections to the memory bank: 
  .Dq(Dq_1), 
  .Addr(Addr_1), 
  .mem_clk(mem_clk_1), 
  .CKEb(CKEb_1), 
  .Bwa_n(Bwa_n_1), 
  .Bwb_n(Bwb_n_1), 
  .Bwc_n(Bwc_n_1), 
  .Bwd_n(Bwd_n_1),
  .WEb(WEb_1), 
  .ADV(ADV_1), 
  .OEb(OEb_1),                  
  .CS1b(CS1b_1), 
  //.CS2(CS2_1), 
  //.CS2b(CS2b_1), 
  .LBOb(LBOb_1), 
  .ZZ(ZZ_1)

  );

  //FSL controller for the Video Input:
  fsl_mem_controller fsl_mem_controller_A (
//Input Connections for controller:
  .clk(core_clk),
  .rst(rst),
//Connections to Memory Controller:
  //Outputs:
  .mem_addr(mem_addr_A),
  .mem_in_data(mem_in_data_A), //y
  .fifo_empty(fifo_empty_A), //y
  .fifo_full(fifo_full_A), //y
  .read_not_write(read_not_write_A),
  .enable(enable_A),
  //Inputs:
  .read_fifo(read_fifo_A), //y
  .write_fifo(write_fifo_A), //y
  .mem_out_data(mem_out_data_A), //y
  .running(running_A),
  .done(done_A),

  //debugging:
  .led0(led0),
  .led1(led1),

//Connected to arbiter:
  .addr_msb_req(addr_msb_req_A),
  .req_mem_arb(req_mem_A),  //status request bit to arbiter
  .control_mem0(sbitA_using_mem0),  //status bit from arbiter
  .control_mem1(sbitA_using_mem1),  //status bit from arbiter
  .addr_msb_ctrl(addr_msb_ctrl_A),
 
//Connections to the FSL Memory Controller Program:
//***IN THIS CASE THERE IS NO PROGRAM AS ALL OPERATIONS ARE DRIVEN BY THE
//***THE VIDEO INPUT (SIGNALS ARE GROUNDED IN fsl_mem0_program.v
  .prog_clk(prog_clk_A), //y
  .prog_rst(prog_rst_A), //y
  .prog_instr(prog_instr_A),
  .program_cbit(prog_cbit_A),
  .prog_instr_read(prog_instr_read_A),
  .valid_instr(valid_instr_A),
  /*.sbit0_using_mem0(sbit0_using_mem0),  //mem0 used by controller 0
  .sbit0_using_mem1(sbit1_using_mem1),  //mem1 used by controller 0
  .sbit1_using_mem0(sbit0_using_mem0),  //mem0 used by controller 1
  .sbit1_using_mem1(sbit1_using_mem1),  //mem1 used by controller 1*/


  // Slave FSL Signals
  .FSL_S_Clk(FSL_S_Clk_A), //y
  .FSL_S_Read(FSL_S_Read_A), //y
  .FSL_S_Data(FSL_S_Data_A), //y
  .FSL_S_Control(FSL_S_Control_A),
  .FSL_S_Exists(FSL_S_Exists_A), //y

  // Master FSL Signals
  .FSL_M_Clk(FSL_M_Clk_A), //y
  .FSL_M_Write(FSL_M_Write_A), //y
  .FSL_M_Data(FSL_M_Data_A), //y
  .FSL_M_Control(FSL_M_Control_A),
  .FSL_M_Full(FSL_M_Full_A) //y
);

  fsl_mem_controller fsl_mem_controller_B (
//Input Connections for controller:
  .clk(core_clk),
  .rst(rst),
//Connections to Memory Controller:
  //Outputs:
  .mem_addr(mem_addr_B),
  .mem_in_data(mem_in_data_B), //y
  .fifo_empty(fifo_empty_B), //y
  .fifo_full(fifo_full_B), //y
  .read_not_write(read_not_write_B),
  .enable(enable_B),
  //Inputs:
  .read_fifo(read_fifo_B), //y
  .write_fifo(write_fifo_B), //y
  .mem_out_data(mem_out_data_B), //y
  .running(running_B),
  .done(done_B),

  //debugging:
  .led0(led0_unused),
  .led1(led1_unused),

//Connected to arbiter:
  .addr_msb_req(addr_msb_req_B),
  .req_mem_arb(req_mem_B),  //status request bit to arbiter
  .control_mem0(sbitB_using_mem0),  //status bit from arbiter
  .control_mem1(sbitB_using_mem1),  //status bit from arbiter
  .addr_msb_ctrl(addr_msb_ctrl_B),
 

//Connections to the FSL Memory Controller Program:
  .prog_clk(prog_clk_B), //y
  .prog_rst(prog_rst_B), //y
  .prog_instr(prog_instr_B),
  .program_cbit(prog_cbit_B),
  .prog_instr_read(prog_instr_read_B),
  .valid_instr(valid_instr_B),
  /*.sbit0_using_mem0(sbit0_using_mem0),  //mem0 used by controller 0
  .sbit0_using_mem1(sbit1_using_mem1),  //mem1 used by controller 0
  .sbit1_using_mem0(sbit0_using_mem0),  //mem0 used by controller 1
  .sbit1_using_mem1(sbit1_using_mem1),  //mem1 used by controller 1
  //**mem0_done //***should this be generic "mem" from controller?NECESSARY?*/

  // Slave FSL Signals
  .FSL_S_Clk(FSL_S_Clk_B), //y
  .FSL_S_Read(FSL_S_Read_B), //y
  .FSL_S_Data(FSL_S_Data_B), //y
  .FSL_S_Control(FSL_S_Control_B),
  .FSL_S_Exists(FSL_S_Exists_B), //y

  // Master FSL Signals
  .FSL_M_Clk(FSL_M_Clk_B), //y
  .FSL_M_Write(FSL_M_Write_B), //y
  .FSL_M_Data(FSL_M_Data_B), //y
  .FSL_M_Control(FSL_M_Control_B),
  .FSL_M_Full(FSL_M_Full_B) //y
);



fsl_mem_ctlrA_program fsl_mem_ctlrA_program (
  // inputs
  .clk(prog_clk_A), 
  .rst(prog_rst_A), 
  
  //Program Interface:
  .prog_instr(prog_instr_A),
  .program_cbit(prog_cbit_A),
  .prog_instr_read(prog_instr_read_A),
  .valid_instr(valid_instr_A),  //only when high should the instruction 
  					//be read (use to stall)
  .sbitA_using_mem0(sbitA_using_mem0),  //mem0 used by controller 0
  .sbitA_using_mem1(sbitA_using_mem1),  //mem1 used by controller 0
  .sbitB_using_mem0(sbitB_using_mem0),  //mem0 used by controller 1
  .sbitB_using_mem1(sbitB_using_mem1)  //mem1 used by controller 1
  );

fsl_mem_ctlrB_program fsl_mem_ctlrB_program (
  // inputs
  .clk(prog_clk_B), 
  .rst(prog_rst_B), 
  
  //Program Interface:
  .prog_instr(prog_instr_B),
  .program_cbit(prog_cbit_B),
  .prog_instr_read(prog_instr_read_B),
  .valid_instr(valid_instr_B),  //only when high should the instruction 
  					//be read (use to stall)
  .sbitA_using_mem0(sbitA_using_mem0),  //mem0 used by controller 0
  .sbitA_using_mem1(sbitA_using_mem1),  //mem1 used by controller 0
  .sbitB_using_mem0(sbitB_using_mem0),  //mem0 used by controller 1
  .sbitB_using_mem1(sbitB_using_mem1)  //mem1 used by controller 1
  );


  arbiter arbiter (	
  .clk(core_clk),
  .rst(rst),
  .ctlrA_using_mem0(sbitA_using_mem0), //stat bit: controller 0 using mem bank 0
  .ctlrA_using_mem1(sbitA_using_mem1), //stat bit: controller 0 using mem bank 1
  .ctlrB_using_mem0(sbitB_using_mem0), //stat bit: controller 1 using mem bank 0
  .ctlrB_using_mem1(sbitB_using_mem1), //stat bit: controller 1 using mem bank 1
  .ctlrA_req_mem(req_mem_A), //controller 0 requesting memory 0
  .ctlrB_req_mem(req_mem_B), //controller 1 requesting memory 0
  .addr_msb_req_A(addr_msb_req_A), //controller 0 requesting a memory bank
  .addr_msb_req_B(addr_msb_req_B), //controller 1 requesting a memory bank
  .addr_msb_ctrl_A(addr_msb_ctrl_A), //controller's 0 accessed memory bank
  .addr_msb_ctrl_B(addr_msb_ctrl_B), //controller's 1 accessed memory bank
  .done_0(done_0), //mem bank 0 unused (done)
  .done_1(done_1) //mem bank 1 unused (done)
  );

//assign led0 = sbitA_using_mem0;
//assign led1 = sbitA_using_mem1;

assign read_fifo_A = (sel_out_ctlrA == 1'b1) ? read_fifo_1 : read_fifo_0;
assign read_fifo_B = (sel_out_ctlrA == 1'b0) ? read_fifo_1 : read_fifo_0;
assign write_fifo_A = (sel_out_ctlrA == 1'b1) ? write_fifo_1 : write_fifo_0;
assign write_fifo_B = (sel_out_ctlrA == 1'b0) ? write_fifo_1 : write_fifo_0;
assign mem_out_data_A = (sel_out_ctlrA == 1'b1) ? 
						mem_out_data_1 : mem_out_data_0;
assign mem_out_data_B = (sel_out_ctlrA == 1'b0) ? 
						mem_out_data_1 : mem_out_data_0;
assign running_A = (sel_out_ctlrA == 1'b1) ? running_1 : running_0;
assign running_B = (sel_out_ctlrA == 1'b0) ? running_1 : running_0;
assign done_A = (sel_out_ctlrA == 1'b1) ? done_1 : done_0;
assign done_B = (sel_out_ctlrA == 1'b0) ? done_1 : done_0;

assign mem_addr_0 = (sel_in_mem0 == 1'b1) ? mem_addr_B : mem_addr_A;
assign mem_addr_1 = (sel_in_mem0 == 1'b0) ? mem_addr_B : mem_addr_A;
assign mem_in_data_0 = (sel_in_mem0 == 1'b1) ? mem_in_data_B : mem_in_data_A;
assign mem_in_data_1 = (sel_in_mem0 == 1'b0) ? mem_in_data_B : mem_in_data_A;
assign fifo_empty_0 = (sel_in_mem0 == 1'b1) ? fifo_empty_B : fifo_empty_A;
assign fifo_empty_1 = (sel_in_mem0 == 1'b0) ? fifo_empty_B : fifo_empty_A;
assign fifo_full_0 = (sel_in_mem0 == 1'b1) ? fifo_full_B : fifo_full_A;
assign fifo_full_1 = (sel_in_mem0 == 1'b0) ? fifo_full_B : fifo_full_A;
assign read_not_write_0 = (sel_in_mem0 == 1'b1) ? 
					read_not_write_B : read_not_write_A;
assign read_not_write_1 = (sel_in_mem0 == 1'b0) ? 
					read_not_write_B : read_not_write_A;
assign enable_0 = (sel_in_mem0 == 1'b1) ? enable_B : enable_A;
assign enable_1 = (sel_in_mem0 == 1'b0) ? enable_B : enable_A;


always @ (sbitA_using_mem1 or sbitB_using_mem0) begin
  if ((sbitA_using_mem1 == 1'b1) || (sbitB_using_mem0 == 1'b1))
     begin
        sel_in_mem0 = 1'b1;
	//sel_in_mem1 = 1'b0;
	sel_out_ctlrA = 1'b1;
	//sel_out_ctlrB = 1'b0;
     end
  else
     begin
        sel_in_mem0 = 1'b0;
	//sel_in_mem1 = 1'b1;
	sel_out_ctlrA = 1'b0;
	//sel_out_ctlrB = 1'b1;
     end
end

endmodule
