//************************************************************************
//    This file is a modified version of the version provided by Xilinx
//    File: BM_MODE_SVGA_CTRL_532.v
//    Uses: SVGA_TIMING_GENERATION_532.v
//          DRIVE_DAC_DATA.v (Xilinx original)
//          MEMORY_CTRL_532.v
//          svga_defines_532.v
//    Used by: top_dualzbt_dow.v
//    Modified for: University of Toronto ECE532 project
//                  FSL SVGA controller module
//    Modifications:
//      Added FSL interface
//      Added delay_cycles parameter
//      Added counter to make sure we read everything off FSL
//      For MultiMedia board, using 25MHz pixel clock => 640x480
//
//    Protocol for communication is:
//      control word:  upper 19 bits give # of words to be read
//                     20th bit is read/write_n by SVGA
//      address word:  upper 19 bits give address (19 bits to address into
//                      a ZBT)
//      data:  must read exactly the number of data words you specified
//              you would in the control word, otherwise the FSLs get clogged
//
//****VIDEO OUT STAYS IN RESET UNTIL THERE IS MORE THAN ONE PIECE OF DATA
//TO BE WRITTEN OUT TO THE MONITOR****
//
//
//    Below is the header provided by Xilinx:
//************************************************************************
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

 /*

-------------------------------------------------------------------------------
   Title      : SVGA Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : BM_MODE_SVGA_CTRL.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/20
   Last Update: 2001/06/20
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : SVGA_TIMING_GENERATION.v DRIVE_DAC_DATA.v MEMORY_CTRL.V
-------------------------------------------------------------------------------
   Used by    : 
-------------------------------------------------------------------------------
   Description: This is the top level module for the bit_mapped mode SVGA display.
   	This design supports screen resolutions up to 800 x 600 with 24 bit color.
	This module uses one ZBT RAM as the video memory. The user is granted access to
	the RAM during horizontal and vertical blanking. An additional ZBT RAM will be
	required if the resolution is increased to 1024 x 768 the maximum supported
	by the SVGA_TIMING_GENERATION module.

	Conventions:
		All external port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
*/

`include "svga_defines_532.v"

module fsl_video_output (
// connections to the DAC
VGA_OUT_PIXEL_CLOCK_P,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH_N,
VGA_VSYNCH_N,
VGA_OUT_RED_P,
VGA_OUT_GREEN_P,
VGA_OUT_BLUE_P,

user_access_ok, //unecessary
select_user_clock, //unecessary

pixel_clock,
pixel_clock_2x, // For FSLs
reset,

led0,
led1,

  // Slave FSL Signals
  FSL_S_Clk,
  FSL_S_Read,
  FSL_S_Data,
  FSL_S_Control,
  FSL_S_Exists,

  // Master FSL Signals
  FSL_M_Clk,
  FSL_M_Write,
  FSL_M_Data,
  FSL_M_Control,
  FSL_M_Full

);

//***CAN I REMOVE THESE?***
parameter base_address = 00000; 
parameter delay_cycles = 3;
  
  output VGA_OUT_PIXEL_CLOCK_P;	// pixel clock for the video DAC
  output VGA_COMP_SYNCH_N;	// composite sync for the video DAC
  output VGA_OUT_BLANK_N;	// composite blanking for the video DAC
  output VGA_HSYNCH_N;	// horizontal sync for the VGA output connector
  output VGA_VSYNCH_N;	// vertical sync for the VGA output connector
  output [7:0] 	VGA_OUT_RED_P;		// RED DAC data
  output [7:0] 	VGA_OUT_GREEN_P;		// GREEN DAC data
  output [7:0] 	VGA_OUT_BLUE_P;		// BLUE DAC data

  output user_access_ok;
  output select_user_clock;
  input	pixel_clock;/* synthesis syn_noclockbuf = 1 */ // pixel clock input 
  input pixel_clock_2x;
  input	reset;	// reset

  output led0, led1;
  
// Slave FSL Signals
  output FSL_S_Clk;
  output FSL_S_Read;
  input  [0:31] FSL_S_Data;
  input  FSL_S_Control;
  input  FSL_S_Exists;

  // Master FSL Signals
 output FSL_M_Clk;
 output FSL_M_Write;
 output [0:31] FSL_M_Data;
 output FSL_M_Control;
 input FSL_M_Full;


//Connections for inputs and outputs:
  wire FSL_S_Clk;
  wire FSL_S_Read;
  //reg FSL_S_Read_reg;
  wire [0:31] FSL_S_Data;
  wire FSL_S_Control;
  wire FSL_S_Exists;
  wire FSL_M_Clk;
  //reg FSL_M_Write;
  //reg [0:31] FSL_M_Data;
  wire FSL_M_Write;
  wire [0:31] FSL_M_Data;
  wire FSL_M_Control;
  wire FSL_M_Full;
  wire VGA_OUT_PIXEL_CLOCK_P;
  wire VGA_COMP_SYNCH_N;
  wire VGA_OUT_BLANK_N;
  wire VGA_HSYNCH_N;
  wire VGA_VSYNCH_N;
  wire [7:0] VGA_OUT_RED_P; 
  wire [7:0] VGA_OUT_GREEN_P; 
  wire [7:0] VGA_OUT_BLUE_P;
  
  wire user_access_ok;
  //wire svga_user_access_ok;
  wire select_user_clock;
  //wire svga_select_user_clock;
  wire pixel_clock;
  wire pixel_clock_2x;
  wire reset;

  wire  led0; 
  wire led1; 
  wire led0_unused, led1_unused; 

// For Video out State machine
parameter st_reset=0;
parameter st_display_video=1;
//parameter st_addr=2;
//parameter st_read=3;
//parameter st_useracc_ok=4;

`define line_length 10'd640 

//Create constants for all the instruction opcode:
`define reset 8'h00
`define noop 8'h04

`define readR_Imm 8'h02
`define write_Imm 8'h03
`define readA_Imm 8'h01

`define readR_Abs_addr 8'h06  
`define write_Abs_addr 8'h07  // These first 8 instructions will be 
`define readA_Abs_addr 8'h05  // read and transmitted.

`define read_Areg 8'h0A
`define write_Areg 8'h0B

`define add_imm_Areg 8'h18
`define sub_imm_Areg 8'h08
`define wait_rx 8'h0C

`define read_Areg_indirect 8'h0E
`define write_Areg_indirect 8'h0F

`define read_Areg_autoinc 8'h1E
`define write_Areg_autoinc 8'h1F

//States for overall execution path state machine:
parameter get_next_instr=0;
parameter decode_instr=1;
parameter execute_instr=2;
  
  wire [7:0] bm_red_data;
  wire [7:0] bm_green_data; 
  wire [7:0] bm_blue_data;
  wire [0:23] fifo_in_data; 
  wire [0:23] fifo_out_data; 
  //wire [18:0] num_pixels;
//wire [7:0]		dac_red_data;
//wire [7:0]		dac_green_data;
//wire [7:0]		dac_blue_data;
//wire 			h_synch_delay;
//wire 			v_synch_delay;
//wire 			comp_synch;
//wire 			blank;
//

//state and delay signals:
  reg [2:0] pres_state;
  reg [2:0] next_state;
  //reg [1:0] pres_v_state; //Video output state
  //reg [1:0] next_v_state; //Video output state
  reg pres_v_state; //Video output state
  reg next_v_state; //Video output state
  //reg [18:0]addr_reg0;
  //reg [18:0]addr_reg1;
  //reg [18:0]addr_reg2;
  //reg [18:0]addr_reg3;

  //Address signals: 
  //wire [18:0] full_addr;
  //wire [18:0] base_addr_sig;
  //wire [19:0] vga_ram_read_address;

 //IR signals:
  wire  rx_IR_full;
  wire [0:31] rx_IR;
  wire  rx_cbit;  //receive control bit
  wire prog_IR_full;
  wire [0:31] prog_IR;
  wire prog_cbit;  //program control bit
  reg  ex_cbit;  //executing control bit
  reg exec_prog; //am I executing the program or an rx_IR**only valid when 
  			//ex_IR_full is high
  //Address registers:
  reg [0:31] a0_reg; //Address register 0 
  reg [0:23] a0_operand; //latches the data portion of the instruction
  reg write_a0;

  //Mux control signals:
  wire [0:31] a0_mux_output; //multiplexer output to the a0 reg
  reg [0:31] FSL_out_mux; //fifo_mux_out; //output from multiplexer into buffering fifo
  
  //FSL and FIFO control signals:
  reg fsl_write;
  wire fifo_write; //attaches to either a buffering fifo or the output FSL
  //reg video_write; //used to read the fifo onto the video DAC
  wire video_write; //used to read the fifo onto the video DAC
  //reg fifo_read; //used to read fsl data onto VGA display
  //wire fifo_read; //used to read fsl data onto VGA display
  wire fifo_empty;
  wire fifo_full;
  wire fifo_almost_empty;
  wire fifo_almost_full;
  //wire fifo_exists;

  //Used to generate Computing element resets:
  wire control_reset;  //combines system reset and instruction reset
  reg instr_reset;	//generated via the reset instruction- clock period long
  wire video_out_reset;  //Used to hold video in reset until we start getting
  			//valid frames
  //Used to interface the ex_IR, A0 and data with the fifo: 
  reg [1:0] mux_select_fifo_in; //used to select
  wire [0:7] ex_IR_fifo_in;
  reg [7:0] opcode;  //Used to store the opcode for 2-part instructions
  //reg valid_data_enable;  //Used to ensure only one line gets written to memory
  				//between H_synchs
  //reg valid_data_delay_reg; //Used to delay the fifo read signal for video out
  //reg valid_data;  //Registered to delay the data by another clock cycle

  //Control registers:	
  reg change_a0;  //Flag to indicate when instruction will alter the value of a0
  reg IR_req_W0;  // Indicates when the instruction writes to the fifo
  reg IR_req_W1;  // Indicates when the instruction writes an immed to the fifo
  reg Addr_req_W;  // Indicates when the address register needs to be written to
  				//the fifo
  reg data_req_W;  // Indicates that there is data to be written to the fifo
  reg rx_data_req; // Used to indicate when data should be read in as part of
  			// the received instruction.
  reg rx_addr_req; // Used to indicate when an address is read in as part of
  			// the received instruction.
  reg [0:31] new_a0_total;  //Output from Accumulator;
  reg [23:0] pixel_ct; // Used to keep track of the number of pixels read in

  //Program interface and Instruction Register Read signals:
  wire [0:31] prog_instr;  //connects to the program instrucion input port
  wire program_cbit; //connects to the program's control bit
  wire prog_instr_read; //reading from the port into the program IR
  //wire done_field; //status bit provided to the program for branching
  //wire done_frame; //status bit provided to the program for branching
  //wire video_in_on; //status bit provided to program for initialization
  wire [0:31] ex_IR;  //wire used as mux output from prog and rx IRs
  wire ex_cbit_input; //wire used as mux output from prog and rx cbits
  wire ex_instr_read;  //reads an instruction into the ex_IR
  wire reading_prog_IR;  //reading the prog_IR clears it
  wire reading_rx_IR;  //reading the rx_IR clears it
  wire sel_ex_IR_input; // selects the input to the ex_IR
  //wire rx_req_W;

  //Status bits:
  wire valid_data;  //Status bit 0- latch video_write on this signal
  wire prefetch_line;  //Status bit 1- send to program
  wire new_frame;  //Status bit 2- send to program
 
// Make sure we read off everything we requested to prevent FSL from
// getting backlogged & hanging the ZBT controller
 //reg [11:0] memdata_ctr; // Max pix are 1024, but don't want overflow errors

//****NECESSARY???
 //reg S_altlevs, M_altlevs;
 wire S_altlevs, M_altlevs;

  wire rd_ack;
  wire rd_err;
  wire wr_ack;
  wire wr_err;

  wire clk;

/*reg ex_noop;
reg ex_wr_a0;
reg ex_read_req;
reg a0_val;
reg ex_wait_rx;
reg ex_read_rx;
reg no_dat_wr;
reg drop_data;
reg result, first_switch;
reg [0:31] wr_dat_cntr;*/
reg invalid_opcode;
reg first_bad_wr;
reg stuck_rx;

//****NECESSARY???
assign S_altlevs = 1'b0;
assign M_altlevs = 1'b0;
/*
always @ (posedge clk or posedge reset) begin
  if (reset == 1'b1)
    wr_dat_cntr <= 32'h00000000;
  else if ((fifo_write == 1'b1) && (wr_dat_cntr != 32'h0004a000))
    wr_dat_cntr <= wr_dat_cntr + 1;
  else
    wr_dat_cntr <= wr_dat_cntr;
end

always @(posedge pixel_clock or posedge reset) begin
  if (reset == 1'b1)
    no_dat_wr <= 1'b0;
  else if ((video_write == 1'b1) && (fifo_empty == 1'b1))
    no_dat_wr <= 1'b1;
  else if ((video_write == 1'b1) && (fifo_empty == 1'b0))
    no_dat_wr <= 1'b0;
  else
    no_dat_wr <= no_dat_wr;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    ex_noop <= 1'b0;
  else if (opcode == `noop)
    ex_noop <= 1'b1;
  else
    ex_noop <= ex_noop;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    ex_wr_a0 <= 1'b0;
  else if (opcode == `write_Areg)
    ex_wr_a0 <= 1'b1;
  else
    ex_wr_a0 <= ex_wr_a0;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    ex_read_req <= 1'b0;
  //else if (opcode == `read_Areg_autoinc)
  else if (opcode == 8'h1e)
    ex_read_req <= 1'b1;
  else
    ex_read_req <= ex_read_req;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    a0_val <= 1'b0;
  else if (a0_reg[12:31] == 20'h4b000)
    //a0_val <= 1'b1;
    a0_val <= ~a0_val;
  else
    a0_val <= a0_val;
    //a0_val <= 1'b0;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    ex_wait_rx <= 1'b0;
  else if (opcode == `wait_rx) 
    ex_wait_rx <= 1'b1;
  else
    ex_wait_rx <= ex_wait_rx;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    ex_read_rx <= 1'b0;
  else if (opcode == `readA_Abs_addr ) 
    ex_read_rx <= 1'b1;
  else
    ex_read_rx <= ex_read_rx;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    drop_data <= 1'b0;
  else if ((VGA_OUT_BLANK_N == 1'b0) && (pixel_ct != 24'h000000) &&
  		(VGA_HSYNCH_N == 1'b0) && (VGA_VSYNCH_N == 1'b0)) 
    drop_data <= 1'b1;
  else
    drop_data <= drop_data;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    first_switch <= 1'b0;
  else if (led0 == 1'b1)
    first_switch <= 1'b1;
  else
    first_switch <= first_switch;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    result <= 1'b0;
  else if ((no_dat_wr == 1'b1) && (first_switch == 1'b0))
    result <= 1'b1;
  else
    result <= result;
end
*/
always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    invalid_opcode <= 1'b0;
  else if ((pres_state == execute_instr) && ((opcode == 8'h00) ||
           (opcode == 8'h02) || (opcode == 8'h01)))
    invalid_opcode <= 1'b1;
  else
    invalid_opcode <= invalid_opcode;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    first_bad_wr <= 1'b0;
  else if ((video_write == 1'b1) && (fifo_empty == 1'b1))
    first_bad_wr <= 1'b1;
  else
    first_bad_wr <= first_bad_wr;
end

always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    stuck_rx <= 1'b1;
  else if ((first_bad_wr == 1'b1) && (opcode != 8'h05))
    stuck_rx <= 1'b0;
  else
    stuck_rx <= stuck_rx;
end

assign led0 = ((stuck_rx == 1'b1)) ? 1'b0 :1'b1;
//assign led0 = ((opcode == 8'h06) && (first_bad_wr == 1'b1)) ? 1'b0 :1'b1;
//assign led1 = ~reset;
assign led1 = ((opcode == 8'h05) && (first_bad_wr == 1'b1)) ? 1'b0 :1'b1;
//assign led1 = (invalid_opcode == 1'b1) ? 1'b0 :1'b1;
//assign led1 = (wr_dat_cntr == 32'h0004a000) ? 1'b0 :1'b1;
//assign led1 = (result == 1'b1) ? 1'b0 :1'b1;
//assign led1 = (fifo_write == 1'b1) ? 1'b0 :1'b1;

//assign led0 = ((VGA_OUT_BLANK_N == 1'b0) && (pixel_ct != 24'h000000)) ? 
	//1'b0 : 1'b1;
//assign led0 = (drop_data == 1'b1) ? 1'b0 : 1'b1;
//assign led1 = ((exec_prog == 1'b0)) ? 1'b0 : 1'b1;
//assign led0 = (rx_IR_full == 1'b1) ? 1'b0 : 1'b1;
//assign led1 = (ex_read_rx == 1'b1) ? 1'b0 : 1'b1;
//assign led1 = ((ex_noop == 1'b1) && (ex_wr_a0 == 1'b1) && 
		//(ex_read_req == 1'b1)) ? 1'b0 : 1'b1;
//assign led1 = (video_out_reset == 1'b1) ? 1'b0 : 1'b1;
//assign led1 = ((no_dat_wr == 1'b1) && (first_switch == 1'b0)) ? 1'b0 : 1'b1;
//assign led0 = (a0_val == 1'b1) ? 1'b0 : 1'b1;
//assign led1 = (opcode == `readA_Abs_addr) ? 1'b0 : 1'b1;
//assign led1 = (a0_reg == 32'h80000000) ? 1'b0 : 1'b1;

//assign led1 = (opcode == 8'h0c) ? 1'b0 :1'b1;
//assign led1 = ((opcode == 8'h0c) && (rx_IR_full == 1'b1) 
		//&& (rx_IR == 32'h00000004)) ? 1'b0 :1'b1;

//assign led0 = (video_out_reset == 1'b1) ? 1'b0 : 1'b1;

/*always @(posedge clk or posedge reset) begin
  if (reset == 1'b1)
    led0 <= 1'b1;
  //else if ((rx_IR_full == 1'b1) && (rx_IR == 32'h00000004))
  //else if ((opcode == 8'h0c) && (ex_noop == 1'b1))
  else if ((opcode == 8'h04))
    led0 <= 1'b0;
  //else if (opcode == 8'h04)
    //led0 <= 1'b1;
  else 
    led0 <= led0;
end
*/

assign fifo_write = ((pres_state == execute_instr) && (rx_addr_req == 1'b0)
			&& (rx_data_req == 1'b1) && (fifo_full == 1'b0) &&
			(FSL_S_Exists == 1'b1));

assign video_write = ((valid_data == 1'b1)); // && (opcode[0] == 1'b1)); 
			// && (exec_prog == 1'b0)); //NECESSARY??

assign clk = pixel_clock_2x;
assign FSL_S_Clk = pixel_clock_2x;
assign FSL_M_Clk = pixel_clock_2x;
assign FSL_M_Write = fsl_write;
assign FSL_M_Data = FSL_out_mux;
assign FSL_M_Control = (mux_select_fifo_in == 2'b00) ? IR_req_W0 : 1'b0;

assign FSL_S_Read = ((reading_rx_IR == 1'b1) || (fifo_write == 1'b1));
//assign fifo_read = ((valid_data == 1'b1) && (fifo_empty == 1'b0));
//assign base_addr_sig = base_address;
//assign full_addr = base_addr_sig + addr_reg3;

//// make the internal connections
//// Should I be registering FSL_S_Data?

//Used to buffer data being read out onto the SVGA:
assign fifo_in_data = FSL_S_Data[8:31];

//assign bm_red_data = fifo_out_data[0:7]; //FSL_S_Data[8:15]; 
//assign bm_green_data = fifo_out_data[8:15]; // FSL_S_Data[16:23];  
//assign bm_blue_data = fifo_out_data[16:23]; //FSL_S_Data[24:31]; 

assign bm_red_data = (fifo_empty == 1'b1) ? 8'hff : fifo_out_data[0:7]; //FSL_S_Data[8:15]; 
assign bm_green_data = (fifo_empty == 1'b1) ? 8'h00 : fifo_out_data[8:15]; // FSL_S_Data[16:23];  
assign bm_blue_data = (fifo_empty == 1'b1) ? 8'h00 : fifo_out_data[16:23]; //FSL_S_Data[24:31]; 

//assign num_pixels = `H_ACTIVE;
//// Don't want to allow access between first request for data and
//// last word of that data coming from memory
//assign user_access_ok = svga_user_access_ok;
//assign select_user_clock = svga_select_user_clock;

assign ex_IR_fifo_in[0:4] = 5'b00000;
assign ex_IR_fifo_in[5:7] = opcode[2:0];

//assign valid_data = ((valid_data_reg == 1'b1) && (data_req_W == 1'b1));
assign ex_instr_read = ((pres_state == get_next_instr) 
	 && ((rx_IR_full || prog_IR_full)));  
//Used to mux the outputs from the two different IRs:
assign sel_ex_IR_input = (pres_state == get_next_instr) ? 
				rx_IR_full : ~exec_prog;
assign ex_IR = sel_ex_IR_input ? rx_IR : prog_IR ;
assign ex_cbit_input = sel_ex_IR_input ? rx_cbit : prog_cbit ;
   
assign prog_IR = prog_instr;
assign prog_cbit = program_cbit;

assign rx_IR = FSL_S_Data;
assign rx_cbit = FSL_S_Control;
assign rx_IR_full = FSL_S_Exists;

// Reads the prog_IR into the ex_IR depending on the mux select:
assign reading_prog_IR =  (((ex_instr_read == 1'b1) ||
			((fsl_write == 1'b1) && (mux_select_fifo_in == 2'b01))
			|| ((write_a0 == 1'b1) && (opcode[0] == 1'b1) &&
				(opcode[2] == 1'b0)))
			&& (sel_ex_IR_input == 1'b0) && (prog_IR_full == 1'b1));

////Used to Read the Program Instructions:
assign prog_instr_read = reading_prog_IR;

// Reads the rx_IR into the ex_IR depending on the mux select:
assign reading_rx_IR = (((ex_instr_read == 1'b1) ||
			((fsl_write == 1'b1) && (mux_select_fifo_in == 2'b01))
			|| ((write_a0 == 1'b1) && (opcode[0] == 1'b1) &&
				(opcode[2] == 1'b0)) || (rx_addr_req == 1'b1)) 
			&& (sel_ex_IR_input == 1'b1) && (rx_IR_full == 1'b1));

//Used to Read the Slave fsl data (the rx_IR):
//assign FSL_S_Read = reading_rx_IR; // (~rx_IR_full) && FSL_S_Exists;

assign a0_mux_output = ((opcode[0] == 1'b1) && (opcode[2] == 1'b0)) ? 
			ex_IR : new_a0_total;

//Generates the overall controller reset signal and the video reset signal:
assign control_reset = reset || instr_reset;
assign video_out_reset = (pres_v_state == st_reset) ? 1'b1: 1'b0;

always @ (posedge clk or posedge reset) begin
  if (reset == 1'b1)
	instr_reset <= 1'b0;
  else if ((pres_state == execute_instr) && (opcode == 8'h00) && 
	  (exec_prog == 1'b0)) 
	instr_reset <= 1'b1;
  else
	instr_reset <= 1'b0;
end

always @ (opcode or a0_reg or a0_operand) begin
  if (opcode[4] == 1'b1)
    new_a0_total = a0_reg + a0_operand;//ex_IR[0:23];
  else
    new_a0_total = a0_reg - a0_operand;//ex_IR[0:23];
end

always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    a0_reg <= 32'h00000000;
  else if (write_a0 == 1'b1)
    a0_reg <= a0_mux_output;
  else
    a0_reg <= a0_reg;
end

always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    a0_operand <= 32'h00000000;
  //else if (change_a0 == 1'b1)
  else if ((ex_instr_read == 1'b1) && (pres_state == get_next_instr))
    a0_operand <= ex_IR[0:23];
  //else if ((ex_IR_full == 1'b1) && (pres_state == get_next_instr))
    //a0_operand <= ex_IR[0:23];
  else
    a0_operand <= a0_operand;
end

always @ (control_reset or change_a0 or Addr_req_W) begin
  if (control_reset == 1'b1)
     write_a0 = 1'b0;
  else if ( (change_a0 == 1'b1) && //(two_part_instr == 1'b0) && 
             (Addr_req_W == 1'b0) )
     write_a0 = 1'b1;
  else 
     write_a0 = 1'b0;
end

always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
     change_a0 <= 1'b0;
  else if (write_a0 == 1'b1)
     change_a0 <= 1'b0;
  else if ((pres_state == decode_instr) && ((opcode[3:0] == 4'b1000) ||
	  (opcode[4:1] == 4'b1111) || (opcode[4:0] == 5'b01011)))
     change_a0 <= 1'b1;
  else
     change_a0 <= change_a0;
end

always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    IR_req_W0 <= 1'b0;  
//Set IR_req_W0 when you are in the decode stage and the instruction should
//be sent (examples of NOT sent instructions include: wait_rx, write_a0,
//add_a0, sub_a0
  else if ((pres_state == decode_instr) && 
	  (((exec_prog == 1'b1) && (opcode[3:0] != 4'b1100) &&
	  (opcode[3:0] != 4'b1000) && (opcode[3:0] != 4'b1011)) ||
	  (opcode[1:0] == 2'b10)))
    IR_req_W0 <= 1'b1;  
  else if ((fsl_write == 1'b1) && (mux_select_fifo_in == 2'b00)) 
    IR_req_W0 <= 1'b0;  
  else
    IR_req_W0 <= IR_req_W0;
end

always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    IR_req_W1 <= 1'b0;  
  else if ((pres_state == decode_instr) && (((exec_prog == 1'b1)
	   && (opcode == `write_Abs_addr)) || (opcode == `readR_Abs_addr)))
    IR_req_W1 <= 1'b1;
  else if ((fsl_write == 1'b1) && (mux_select_fifo_in == 2'b01)) 
	  //&& (IR_req_W0 == 1'b0))
    IR_req_W1 <= 1'b0;
  else
    IR_req_W1 <= IR_req_W1;
end

//Instruction includes an address ***FOR NOW DITCH IT***FIX LATER???***
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    rx_addr_req <= 1'b0;  
  else if ((pres_state == decode_instr) && (opcode[2] == 2'b1) &&
	  	((opcode[1] == 1'b1) || (opcode[0] == 1'b1)) &&
  		(exec_prog == 1'b0))
    rx_addr_req <= 1'b1;
  else if ((rx_addr_req == 1'b1) && (reading_rx_IR == 1'b1))
    rx_addr_req <= 1'b0;  
  else
    rx_addr_req <= rx_addr_req;
end

//Controls the request to write an address register to the fifo
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    Addr_req_W <= 1'b0;  
  else if ((pres_state == decode_instr) && (opcode[3:2] == 2'b11) &&
  		(opcode[1:0] != 2'b00))
    Addr_req_W <= 1'b1;
  else if ((fsl_write == 1'b1) && (mux_select_fifo_in == 2'b10))
    Addr_req_W <= 1'b0;  
  else
    Addr_req_W <= Addr_req_W;
end

//Used to delay reading of fifo to video out until control ready:
//always @ (posedge pixel_clock or control_reset) begin
	//if (control_reset == 1'b1)
		//valid_data_delay_reg <= 1'b0;
	//else if ((rx_data_req == 1'b1) && // (fifo_empty == 1'b0) &&
		//(pres_state == execute_instr) && (video_write == 1'b1))
		//valid_data_delay_reg <= 1'b1;
	//else if (fifo_almost_empty == 1'b1) //&& (rx_data_req == 1'b0))
		//valid_data_delay_reg <= 1'b0;
	//else
		//valid_data_delay_reg <= valid_data_delay_reg;
//end

////Used to delay reading of fifo to video out until control ready:
//always @ (posedge pixel_clock or control_reset) begin
	//if (control_reset == 1'b1)
		//valid_data <= 1'b0;
	//else
		//valid_data <= valid_data_delay_reg;
//end

//Should ensure that data is properly read from the fsl to the fifo
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
     rx_data_req <= 1'b0;
  else if ((pres_state == decode_instr) &&  //for sequential execution
		((exec_prog == 1'b0) && (opcode[0] == 1'b1))) 
     rx_data_req <= 1'b1;  
  else if ((pixel_ct == 24'h000001) && (fifo_write == 1'b1)) 
	  //&& (mux_select_fifo_in == 2'b11))
     rx_data_req <= 1'b0;
  else
     rx_data_req <= rx_data_req;
end

//NOT APPLICABLE FOR THIS APPLICATION- no data to write!
//Controls the request to write data to the fifo
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    data_req_W <= 1'b0;  
  /*else if ((pres_state == decode_instr) &&  //for sequential execution
		(((exec_prog == 1'b1) && (opcode[1:0] == 2'b11)) || 
	       ((exec_prog == 1'b0) && (opcode[1:0] == 2'b10))) && 
	       (opcode[3:2] != 2'b10) // && (data_req_W == 1'b0)) 
    data_req_W <= 1'b1;  
  else if ((pixel_ct == 24'h000001) && (fsl_write == 1'b1)) 
	  //&& (mux_select_fifo_in == 2'b11))
    data_req_W <= 1'b0;*/
  else
    data_req_W <= data_req_W;
end

//Combine this with the above code for the complete controller
//which may transmit or receive data (eg a memory)
//Loads the number of data values into the counter
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    pixel_ct <= 24'h000000;  
  /*else if ((pres_state == decode_instr) &&  //for sequential execution
		(((exec_prog == 1'b1) && (opcode[1:0] == 2'b11)) || 
	       ((exec_prog == 1'b0) && (opcode[1:0] == 2'b10))) && 
	       (opcode[3:2] != 2'b10) && (data_req_W == 1'b0)) */
  else if ((pres_state == decode_instr) &&  //for sequential execution
		((exec_prog == 1'b0) && (opcode[0] == 1'b1))) 
    pixel_ct <= a0_operand; //ex_IR[0:23];
  //else if ((mux_select_fifo_in == 2'b11) && (fsl_write == 1'b1)) //data out
  else if (fifo_write == 1'b1)
    pixel_ct <= pixel_ct - 1;
  else
    pixel_ct <= pixel_ct;
end


// Generate the select signal to the mux feeding the internal buffer:
always @ (pres_state or IR_req_W0 or IR_req_W1 or Addr_req_W or data_req_W) begin
	if ((pres_state == execute_instr) && (IR_req_W0 == 1'b1)) // || 
		//(IR_req_W1 == 1'b1))) 
	      mux_select_fifo_in = 2'b00;
	else if ((pres_state == execute_instr) && (IR_req_W1 == 1'b1)) // && 
		//(IR_req_W1 == 1'b0) && (Addr_req_W == 1'b1))
	      mux_select_fifo_in = 2'b01;
        //else if ( (((pres_state == execute_instr) && (IR_req_W0 == 1'b0) && 
		   //(IR_req_W1 == 1'b0) && (Addr_req_W == 1'b0)) || 
		   //(pres_state == get_next_instr) || 
		   //(pres_state == decode_instr)) && (data_req_W == 1'b1))
	      //mux_select_fifo_in <= 2'b10;
	else if ((pres_state == execute_instr) && (Addr_req_W == 1'b1)) // && 
		//(IR_req_W1 == 1'b0) && (IR_req_W0 == 1'b0))
	      mux_select_fifo_in = 2'b10;
	else 
	      mux_select_fifo_in = 2'b11;
end

// Used to update ex_IR with the next instruction to be executed:
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1) //|| (reading_rx_IR == 1'b1))
    //begin
      //ex_IR <= 32'b0;
      ex_cbit <= 1'b0;
    //end
  else if (ex_instr_read == 1'b1) 
    //begin
      //ex_IR <= ex_IR_input;
      ex_cbit <= ex_cbit_input;
    //end
  else
    //begin
      //ex_IR <= ex_IR;
      ex_cbit <= ex_cbit;
    //end
end

// Used to update opcode with the next instruction to be executed:
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
    opcode <= 8'b0;
  else if ((ex_instr_read == 1'b1) && (pres_state == get_next_instr))
    opcode <= ex_IR[24:31];
  //else if ((ex_IR_full == 1'b1) && (pres_state == get_next_instr))
    //opcode <= ex_IR[24:31];
  else
    opcode <= opcode;
end
// Used to update ex_IR with the next instruction to be executed:
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1) //|| 
    exec_prog <= 1'b1;
  //else if ((two_part_instr == 1'b1) && (ex_instr_read == 1'b1))
    //exec_prog <= exec_prog;
  else if ((ex_instr_read == 1'b1) && (rx_IR_full == 1'b1))
    exec_prog <= 1'b0;
  else if ((ex_instr_read == 1'b1) && (rx_IR_full == 1'b0))
    exec_prog <= 1'b1;
  else
    exec_prog <= exec_prog;
end

// Mux the input into the internal buffer:
always @ (mux_select_fifo_in or ex_IR or ex_IR_fifo_in or a0_reg or a0_operand) begin
  case (mux_select_fifo_in) 
    2'b00:
      FSL_out_mux = {a0_operand,ex_IR_fifo_in};
    2'b01:
      FSL_out_mux = ex_IR; //{13'h0,Addr};
    2'b10:
      FSL_out_mux = a0_reg;
    2'b11:
      FSL_out_mux = 32'h00000000;
    //2'b11:
      //FSL_out_mux = FSL_out_mux;
  endcase
end
////
////register ram_read_address because of delay with state machine
//always @ (posedge pixel_clock or posedge reset) begin
  //if (reset) begin
    //addr_reg0[18:0] <= 19'h00000;
    //addr_reg1[18:0] <= 19'h00000;
    //addr_reg2[18:0] <= 19'h00000;
    //addr_reg3[18:0] <= 19'h00000;
  //end
  //else begin
    //addr_reg3[18:0] <= addr_reg2[18:0];
    //addr_reg2[18:0] <= addr_reg1[18:0];
    //addr_reg1[18:0] <= addr_reg0[18:0];
    //addr_reg0[18:0] <= vga_ram_read_address[18:0];
  //end
//end  

/*
//Used to delay the video write signal by a clock signal
always @ (posedge pixel_clock or video_out_reset) begin
       if (video_out_reset == 1'b1)
       	 video_write <= 1'b0;
       else 
	 video_write <= valid_data;
end
*/

// Video output status FSM register
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
  //if (reset == 1'b1)
    pres_v_state <= st_reset;
  else
    pres_v_state <= next_v_state;
end

//Determine next state for Video output FSM:
always @ (pres_v_state or fifo_empty or rx_data_req or control_reset or fifo_almost_empty or instr_reset) begin
  case(pres_v_state)
    st_reset:
      begin
	if (fifo_almost_empty == 1'b0)
		next_v_state = st_display_video;
	else
		next_v_state = st_reset;
      end
    st_display_video:
      begin
	//if ((rx_data_req == 1'b0) && (fifo_empty == 1'b0))
	//if ((instr_reset == 1'b1)) //UNECESSARY? && (fifo_empty == 1'b0))
	if ((control_reset == 1'b1)) //UNECESSARY? && (fifo_empty == 1'b0))
		next_v_state = st_reset;
	else
		next_v_state = st_display_video;
      end
  endcase
end

// FSM register
always @ (posedge clk or posedge control_reset) begin
  if (control_reset == 1'b1)
  //if (reset == 1'b1)
    pres_state <= get_next_instr;
  else
    pres_state <= next_state;
end

// Determine next state
always @ (pres_state or control_reset or exec_prog or opcode or data_req_W or fifo_empty or pixel_ct or ex_instr_read or rx_data_req or IR_req_W0 or IR_req_W1 or Addr_req_W or change_a0 or rx_IR_full or rx_addr_req) begin
  case (pres_state) 
    get_next_instr:
      begin 
	  // Don't attempt to write out this line unless we know for sure
	  // we have space to buffer it. i.e. the svga-line-sized buffer is 
	  // empty
        //if (Vmid == 1'b0 && Hmid_prev[0] == 1'b1 && Hmid == 1'b0 && fifo_exists == 1'b0)  
	if ((ex_instr_read == 1'b1)) // || (ex_IR_full == 1'b1))
	         next_state = decode_instr;
        else
	         next_state = get_next_instr;
      end
    decode_instr:
     /* begin  //Case 1 instr writes out data but the prev instr is using fifo
	if ( ( ((exec_prog == 1'b1) && ((opcode[1:0] == 2'b11) ||
		(opcode[1:0] == 2'b00))) || 
	       ((exec_prog == 1'b0) && (opcode[1:0] == 2'b10)) )
       	    && (opcode[3:2] != 2'b10) && (data_req_W == 1'b1) )
        	  next_state = decode_instr;
	    //Case 2 instr reads in data but the prev instr is using the fsl
	else if ( ((exec_prog == 1'b0) && (opcode[0] == 1'b1))  
       	    && (opcode[3:2] != 2'b10) && (rx_data_req == 1'b1) )
        	  next_state = decode_instr;
	   //There are no shared resource issues so execute
	else*/
		  next_state = execute_instr;
      //end
    execute_instr:
      begin
	  //Continue executing until the instruction/address has been
	  //completed- that data can be written on its own.
	if ((IR_req_W0 == 1'b1) || (IR_req_W1 == 1'b1) || 
	   (Addr_req_W == 1'b1) || (change_a0 == 1'b1) || 
	   //added to remove parallelism
	   (data_req_W == 1'b1) || (rx_data_req == 1'b1)  ||
	   (rx_addr_req == 1'b1) || 
	   ((exec_prog == 1'b0) && (opcode[0] == 1'b1) &&
	   (fifo_empty == 1'b0)) ||  //just fifo empty???
	   ((opcode == `wait_rx) && (rx_IR_full == 1'b0)))
		  next_state = execute_instr;
	else
		  next_state = get_next_instr;
        end
  endcase
end

always @ (mux_select_fifo_in or IR_req_W0 or IR_req_W1 or Addr_req_W or data_req_W or fifo_full or valid_data or FSL_M_Full) begin
  case(mux_select_fifo_in)
     //*****Do not worry about back to back writes******************
     2'b00:
       begin
         if ((IR_req_W0 == 1'b1) && (FSL_M_Full == 1'b0)) 
		 // && ((data_req_W == 1'b0) ||
		//((data_req_W == 1'b1) && (Hmid == 1'b0)))) 
		 //(fifo_exists == 1'b0)) // &&
		 //(fsl_write == 1'b0))
		 // (pres_state == execute_instr) &&
		// || (IR_req_W1 == 1'b1)) && (ex_IR_full == 1'b1) && 
		 //(fifo_full == 1'b0)) 
            fsl_write = 1'b1;
	 else 
            fsl_write = 1'b0;
       end
     2'b01:
       begin
         if ((IR_req_W1 == 1'b1) && (FSL_M_Full == 1'b0)) 
		 // && (fsl_write == 1'b0))
            fsl_write = 1'b1;
	 else 
            fsl_write = 1'b0;
       end
     2'b10:
       begin
         if ((Addr_req_W == 1'b1) && (FSL_M_Full == 1'b0)) // && 
		 //(fsl_write == 1'b0))
            fsl_write = 1'b1;
	 else 
            fsl_write = 1'b0;
       end
     2'b11:
       /*begin
         if ((data_req_W == 1'b1) && (fifo_full == 1'b0) && 
		 (valid_data == 1'b1)) // && (fsl_write == 1'b0))
            fsl_write = 1'b1;
	 else */
            fsl_write = 1'b0;
       //end
  endcase
end


//// FSM register
//always @ (posedge pixel_clock or posedge reset) begin
  //if (reset == 1'b1) begin
    //pres_state <= st_reset; 
  //end
  //else begin
    //pres_state <= next_state;
  //end
//end
//
//// Determine next state
//always @ (negedge pixel_clock or posedge reset) begin
  //if (reset == 1'b1) begin
    //next_state <= st_reset; 
  //end
  //else begin
    //case (pres_state) 
      //st_reset:
	//begin
          //if (reset == 1'b0) begin
            //next_state <= st_ctrl;
	  //end
	  //else begin
            //next_state <= st_reset;
	  //end
	//end
      //st_ctrl:
	//next_state <= st_addr;
      //st_addr:
	//next_state <= st_read;
      //st_read:
	//begin
          //if (user_access_ok == 1'b1) begin  // Keep reading until all data is back
            //next_state <= st_useracc_ok;
	  //end
	  //else begin
            //next_state <= st_read;
	  //end
	//end
      //st_useracc_ok:
	//begin
          //if (user_access_ok == 1'b1) begin  
            //next_state <= st_useracc_ok;
	  //end
	  //else begin
            //next_state <= st_ctrl;
	  //end
	//end
    //endcase
  //end
//end
//

// FSL Slave Output
/*always @ (posedge pixel_clock_2x or posedge reset) begin
  if (reset == 1'b1) begin
    FSL_S_Read_reg <= 1'b0; 
    S_altlevs <= 1'b1;
    memdata_ctr <= 11'h0;
  end
  else begin
    case (pres_state) 
      st_reset:
       begin
         FSL_S_Read_reg <= 1'b0; 
         memdata_ctr <= 11'h0;
       end
      st_ctrl:
       begin
         FSL_S_Read_reg <= 1'b0; 	      
         memdata_ctr <= 11'h0;  // Since we don't go back to st_reset after each line, reset counter here
       end
      st_addr:
       begin
         FSL_S_Read_reg <= 1'b0; 	      
         memdata_ctr <= memdata_ctr;
       end
      st_read:
        begin	      
          if ((FSL_S_Exists == 1'b1) && (memdata_ctr < `H_ACTIVE) && (S_altlevs == 1'b1)) begin
            FSL_S_Read_reg <= 1'b1; 	      
            memdata_ctr <= memdata_ctr + 1;
	  end
 	  else begin
            FSL_S_Read_reg <= 1'b0; 
            memdata_ctr <= memdata_ctr;
	  end
        end
      st_useracc_ok:
	 begin
////        I am assuming that the user_access_ok period is long enough to
////        clear out any backlog in the FSL
          if ((FSL_S_Exists == 1'b1) && (memdata_ctr < `H_ACTIVE) && (S_altlevs == 1'b1)) begin
            FSL_S_Read_reg <= 1'b1; 
            memdata_ctr <= memdata_ctr + 1;
	  end
 	  else begin
            FSL_S_Read_reg <= 1'b0; 
            memdata_ctr <= memdata_ctr;
	  end
        end
    endcase
    S_altlevs <= !S_altlevs;
  end
end
	      
// FSL Master Output, run at 2x pixel clock
always @ (posedge pixel_clock_2x or posedge reset) begin
  if (reset == 1'b1) begin
          FSL_M_Write <= 1'b0;
          FSL_M_Data <= 32'h0000_0000;
          FSL_M_Control <= 1'b0;
	  M_altlevs <= 1'b1;
  end
  else begin
    case (pres_state) 
      st_reset:
	begin
          FSL_M_Write <= 1'b0;
          FSL_M_Data <= 32'h0000_0000;
          FSL_M_Control <= 1'b0;
	end
      st_ctrl:
	begin
          FSL_M_Write <= (FSL_M_Full == 1'b1 ? 1'b0 : (1'b1 & M_altlevs));
	  FSL_M_Data <= {5'b00000,num_pixels, 8'h06};
          FSL_M_Control <= 1'b1;
	end
      st_addr:
	begin
          FSL_M_Write <= (FSL_M_Full == 1'b1 ? 1'b0 : (1'b1 & M_altlevs));
	  FSL_M_Data <= {13'h0000,full_addr};
          FSL_M_Control <= 1'b0;
	end
      st_read:
	begin
          FSL_M_Write <= 1'b0;
	  FSL_M_Data <= 32'h0;
          FSL_M_Control <= 1'b0;
	end
      st_useracc_ok:
	begin
          FSL_M_Write <= 1'b0;	  
          FSL_M_Data <= 32'h0;
          FSL_M_Control <= 1'b0;
	end
    endcase 
    M_altlevs <= !M_altlevs;
  end
end*/


//// make the external connections
//// shut down the video output during  reset
//always @ (posedge pixel_clock or posedge reset) begin
	//if (reset) begin
		//VGA_HSYNCH_N	 	<= 1'b1;
		//VGA_VSYNCH_N 	 	<= 1'b1;
		//VGA_COMP_SYNCH_N 		<= 1'b1;
		//VGA_OUT_BLANK_N 		<= 1'b0;
		//VGA_OUT_RED_P		<= 8'h00;
		//VGA_OUT_GREEN_P		<= 8'h00;
		//VGA_OUT_BLUE_P		<= 8'h00;
//
		//end
	//else begin
		//VGA_HSYNCH_N	 	<= !h_synch_delay;
		//VGA_VSYNCH_N 	 	<= !v_synch_delay;
		//VGA_COMP_SYNCH_N 		<= !comp_synch;
		//VGA_OUT_BLANK_N 		<= !blank;
		//VGA_OUT_RED_P		<= dac_red_data;
		//VGA_OUT_GREEN_P		<= dac_green_data;
		//VGA_OUT_BLUE_P		<= dac_blue_data;
//
		//end
	//end
//
//
////register ram_read_address because of delay with state machine
//always @ (posedge pixel_clock or posedge reset) begin
  //if (reset) begin
    //addr_reg0[18:0] <= 19'h00000;
    //addr_reg1[18:0] <= 19'h00000;
    //addr_reg2[18:0] <= 19'h00000;
    //addr_reg3[18:0] <= 19'h00000;
  //end
  //else begin
    //addr_reg3[18:0] <= addr_reg2[18:0];
    //addr_reg2[18:0] <= addr_reg1[18:0];
    //addr_reg1[18:0] <= addr_reg0[18:0];
    //addr_reg0[18:0] <= vga_ram_read_address[18:0];
  //end
//end  

fsl_video_output_program fsl_video_output_program (
  // inputs
  .clk(pixel_clock_2x), 
  .rst(control_reset), 

  //.led0(led0_unused),
  //.led1(led1_unused), 
  //Program Interface:
  .prog_instr(prog_instr),
  .program_cbit(program_cbit),
  .prog_instr_read(prog_instr_read),
  .valid_instr(prog_IR_full),
  .valid_data(valid_data),  //Status bit 0- latch video_write on this signal
  .prefetch_line(prefetch_line),  //Status bit 1- send to program
  .new_frame(new_frame)   //Status bit 2- send to program
  );

  /***SAVE THIS AS WHAT I PLAN TO USE
coregen_asynch_fifo coregen_asynch_fifo (
	.din(fifo_in_data),
	.wr_en(fifo_write),
	.wr_clk(pixel_clock_2x),
	.rd_en(video_write),
	.rd_clk(pixel_clock),
	.ainit(control_reset),
	.dout(fifo_out_data),
	.full(fifo_full),
	.empty(fifo_empty),
	.almost_full(fifo_almost_full),
	.almost_empty(fifo_almost_empty)
);

***/

coregen_asynch_fifo_test coregen_asynch_fifo_test (
	.din(fifo_in_data),
	.wr_en(fifo_write),
	.wr_clk(pixel_clock_2x),
	.rd_en(video_write),
	.rd_clk(pixel_clock),
	.ainit(control_reset),
	.dout(fifo_out_data),
	.full(fifo_full),
	.empty(fifo_empty),
	.almost_full(fifo_almost_full),
	.almost_empty(fifo_almost_empty),
	.rd_ack(test_rd_ack),
	.rd_err(test_rd_err),
	.wr_ack(test_wr_ack),
	.wr_err(test_wr_err)
);


BM_MODE_SVGA_CTRL_532 BM_MODE_SVGA_CTRL_532 (
// connections to the DAC
.VGA_OUT_PIXEL_CLOCK_P(VGA_OUT_PIXEL_CLOCK_P),
.VGA_COMP_SYNCH_N(VGA_COMP_SYNCH_N),
.VGA_OUT_BLANK_N(VGA_OUT_BLANK_N),
.VGA_HSYNCH_N(VGA_HSYNCH_N),
.VGA_VSYNCH_N(VGA_VSYNCH_N),
.VGA_OUT_RED_P(VGA_OUT_RED_P),
.VGA_OUT_GREEN_P(VGA_OUT_GREEN_P),
.VGA_OUT_BLUE_P(VGA_OUT_BLUE_P),

.user_access_ok(user_access_ok),
.select_user_clock(select_user_clock),
.pixel_clock(pixel_clock),
.pixel_clock_2x(pixel_clock_2x), // For FSL
.reset(video_out_reset),
//.vga_ram_read_address(vga_ram_read_address),

  //// Slave FSL Signals
  //.FSL_S_Clk(FSL_S_Clk),
  //.FSL_S_Read(FSL_S_Read),
  //.FSL_S_Data(FSL_S_Data),
  //.FSL_S_Control(FSL_S_Control),
  //.FSL_S_Exists(FSL_S_Exists),
  .bm_red_data(bm_red_data),
  .bm_green_data(bm_green_data),
  .bm_blue_data(bm_blue_data),

  .valid_data(valid_data),  //Status bit 0
  .prefetch_line(prefetch_line),  //Status bit 1
  .new_frame(new_frame)  //Status bit 2
  //// Master FSL Signals
  //.FSL_M_Clk(FSL_M_Clk),
  //.FSL_M_Write(FSL_M_Write),
  //.FSL_M_Data(FSL_M_Data),
  //.FSL_M_Control(FSL_M_Control),
  //.FSL_M_Full(FSL_M_Full)

);
    
//// instantiate the SVGA timing generator
//SVGA_TIMING_GENERATION_532  SVGA_TIMING_GENERATION_532 (
//pixel_clock,
//reset,
//h_synch_delay,
//v_synch_delay,
//comp_synch,
//blank,
//vga_ram_read_address,
//svga_user_access_ok,
//svga_select_user_clock
//);
//
//
//// instantiate the DAC driver
//DRIVE_DAC_DATA DRIVE_DAC_DATA(
//bm_red_data, 
//bm_green_data, 
//bm_blue_data, 
//8'h00, //char_mode_data,
//1'b1, //bm_display_active,
//1'b0, //char_display_active,
//dac_red_data,
//dac_green_data,
//dac_blue_data
//);
//
//
//OBUF_F_12 PIXCLK_OUT(
//.O(VGA_OUT_PIXEL_CLOCK_P), 
//.I(pixel_clock)
//);
//
endmodule //BM_MODE_SVGA_CTRL
