//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: fsl_video_output_program.v
//    Used by: fsl_BM_MODE_SVGA_CTRL_532.v
//    Works with ProjectNavigator 6.2.03i
//    Description: Provides the instructions to be executed by the video
//    controller.  It should only be necessary for a user to change the 
//    program to suit the application- the controller should remain
//    unchanged
//
//
//    LLS November 2004
//************************************************************************ 
//
 

module fsl_video_output_program (
  // inputs
  clk, rst, 
 
  //Debugging outputs:
  led0, led1,

  //Program Interface:
  prog_instr,
  program_cbit,
  prog_instr_read,
  valid_instr,  //only when high should the instruction be read (use to stall)
  valid_data, //Status_bit 0
  prefetch_line, //Status_bit 1
  new_frame //Status_bit 2
  );

  input clk, rst;

  output led0, led1;

  output [0:31] prog_instr;  //Next instruction to be executed by the controller
  output program_cbit;
  output valid_instr;
  input prog_instr_read; //Read the instruction on the prog_instr bus
  input valid_data; //Status_bit 0
  input prefetch_line; //Status_bit 1
  input new_frame; //Status_bit 2
	  
  wire clk, rst;

  wire led1;
  reg  led0;

  //Program interface:
  wire [0:31] prog_instr;
  wire program_cbit;
  wire prog_instr_read; 
  reg valid_instr; 
  wire valid_data, prefetch_line, new_frame;

  reg prefetch_line_prev;
  reg new_frame_prev;
  reg choose_bank;
  //reg [3:0] pres_state, next_state;
  reg [2:0] pres_state, next_state;
  reg [0:31] int_prog_instr;
  reg int_prog_cbit;

/*//Test instructions:
`define write_imm 32'h00000703
`define read_req_imm 32'h00000702
`define receive_imm 32'h00000701
`define write_abs_imm 32'h00000707
`define read_req_abs_imm 32'h00000706
`define receive_abs_imm 32'h00000705
`define abs_addr 32'h00440033
`define write_a0 32'h000000B
`define a0_new_addr 32'h001100AA
`define add_a0 32'h00220018
`define sub_a0 32'h00007708
`define write_indirect 32'h0000070F
`define write_autoinc 32'h0000071F
`define noop 32'h00000004
`define reset 32'h00000000
`define wait_rx 32'h0000000C
//NEW INSTRUCTION TO ADD??
`define set_clear_status_bits 32'h00700713 //0-11 indicate new status bit values
					   //12-23 indicate bits to be changed
					   //24-31 gives the instr opcode
*/
/*//TEST STATES
// NOTE: The states reflect the program structure
//parameter  st_wr_imm=0;
//parameter  st_wr_abs_imm=1;
parameter  st_read_req_imm=0;
parameter  st_read_req_abs_imm=1;
parameter  st_wr_addr=2;
parameter  st_wr_a0=3;
parameter  st_wr_new_a0=4;
parameter  st_add_a0=5;
parameter  st_sub_a0=6;
parameter  st_wr_indirect=7;
parameter  st_wr_autoinc=8;
parameter  st_noop=9;
parameter  st_reset=10;
parameter  st_wait_rx=11;
parameter  st_set_clear_status=11;
*/
// NOTE: The states reflect the program structure
parameter  st_reset=0; //wait for a noop to start up
parameter  st_wr_addr_a0=1;
parameter  st_wr_addr_bank0=2;
parameter  st_wr_addr_bank1=3;
parameter  st_wr_read_req_a0_inc=4;
parameter  st_wait_rx=5;
parameter  st_stall=6;

////`define line_length 10'd640 
/*//ACTUAL PROGRAM:
`define wr_addr_a0 32'h0000000B //Write base address to a0
`define addr_bank0 32'h80000000 //Odd Base address for bank 0
`define addr_bank1 32'h40000000 //Even Base address for bank 0
`define read_req_a0_inc 32'h0002801E //Request data from memory address a0 
					//and post increment
`define wait_rx 32'h0000000C //Wait until the data comes before processing
*/
//**FOR TESTING PROGRAM:
`define wr_addr_a0 32'h0000000B //Write base address to a0
`define addr_bank0 32'h80000000 //Base address for bank 0
`define addr_bank1 32'h40000000 //Base address for bank 1
//`define read_req_a0_inc 32'h0000071E //Write 7 words to a0 and post increment
`define read_req_a0_inc 32'h0002801E //Request data from memory address a0 
					//and post increment
`define wait_rx 32'h0000000C //Wait for a receive instruction before continuing
`define noop 32'h00000004 //Wait for a receive instruction before continuing

reg bank_change;

assign prog_instr = int_prog_instr;
assign program_cbit = int_prog_cbit;

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    led0 <= 1'b0;
  //else if ((new_frame == 1'b1) && (new_frame_prev == 1'b0))
  else if ((pres_state == st_reset))
    //led0 <= ~led0;
    led0 <= 1'b0;
  else
    //led0 <= led0;
    led0 <= 1'b1;
end 

//assign led0 = ((pres_state == st_wr_read_req_a0_inc) || 
		//(pres_state == st_wr_addr_a0)) ? 1'b0 : 1'b1;
assign led1 = ((pres_state == st_wait_rx) || (pres_state ==st_stall)) ? 1'b0 :1'b1;

//assign led0 = (bank_change == 1'b1) ? 1'b0 : 1'b1;
//assign led0 = (choose_bank == 1'b1) ? 1'b0 : 1'b1; //choose_bank; 
//assign led0 = (rst == 1'b1) ? 1'b0 : 1'b1;
//assign led0 = (pres_state == st_wr_addr_bank1) ? 1'b0 : 1'b1;
//assign led1 = (pres_state == st_reset) ? 1'b0 : 1'b1;
//assign led1 = (choose_bank == 1'b1) ? 1'b0 : 1'b1; //choose_bank; 
//assign led1 = (pres_state == st_wr_addr_a0) ? 1'b0 : 1'b1;

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    bank_change <= 1'b0;
  else if (choose_bank == 1'b1)
    bank_change <= 1'b1;
  else
    bank_change <= bank_change;
end 

/*
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    led0 <= 1'b1;
  //else if (pres_state == st_reset)
  else if (pres_state == st_wr_addr_a0)
    led0 <= ~led0;
  //else if ((pres_state == st_wr_addr_bank0) || (pres_state == st_wr_addr_bank1))
    //led1 <= 1'b1;
  else
    led0 <= led0;
    //led1 <= led1;
end
*/

// Stores the current frame:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    //choose_bank <= 1'b1;
    choose_bank <= 1'b0;
  //else if ((pres_state == st_wr_addr_bank0) && (prog_instr_read == 1'b1))
    //choose_bank <= 1'b1;
  //else if ((pres_state == st_wr_addr_bank1) && (prog_instr_read == 1'b1))
    //choose_bank <= 1'b0;
  else if ((pres_state == st_wr_addr_a0) && (prog_instr_read == 1'b1))
    choose_bank <= ~choose_bank;
  else
    choose_bank <= choose_bank;
end

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    prefetch_line_prev <= 1'b0;
  else
    prefetch_line_prev <= prefetch_line;
end

always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    new_frame_prev <= 1'b0;
  else
    new_frame_prev <= new_frame;
end

//Used to indicate when the instruction is valid.  Allows us to stall the
//processor when we don't know for sure the next state
always @ (pres_state or prefetch_line) begin
  case(pres_state)
    st_reset:
      valid_instr = 1'b1;
    st_wr_addr_a0:
      valid_instr = 1'b1;
    st_wr_addr_bank0:
      valid_instr = 1'b1;
    st_wr_addr_bank1:
      valid_instr = 1'b1;
    st_wr_read_req_a0_inc:
      //begin
	//if (prefetch_line == 1'b1)
          valid_instr = 1'b1;
	//else 
          //valid_instr = 1'b0;
      //end
    st_wait_rx:
      valid_instr = 1'b1;
    st_stall:
      valid_instr = 1'b0;
    default:
      valid_instr = 1'b0;
  endcase
end

// FSM register
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    //pres_state <= st_wr_addr_a0;
    pres_state <= st_reset;
  else
    pres_state <= next_state;
end

//Next-state state machine for the program:
always @ (pres_state or prog_instr_read or prefetch_line or new_frame or choose_bank or rst or prefetch_line_prev or new_frame_prev) 
begin

  case (pres_state)
    st_reset:
      begin
        if ((prog_instr_read == 1'b1) && (rst == 1'b0))
          next_state = st_wr_addr_a0;
        else
          next_state = st_reset;
      end
    st_wr_addr_a0:
      begin
      	//if ((prog_instr_read == 1'b1) && (rst == 1'b0)) begin
	  if ((choose_bank == 1'b0) && (prog_instr_read == 1'b1))
            next_state = st_wr_addr_bank0;
	  else if ((choose_bank == 1'b1) && (prog_instr_read == 1'b1))
            //next_state = st_wr_addr_bank1;
            next_state = st_wr_addr_bank0;
          //end
          else
            next_state = st_wr_addr_a0;
      end
    st_wr_addr_bank0:
      begin 
	if (prog_instr_read == 1'b1) 
          next_state = st_wr_read_req_a0_inc;
	else
          next_state = st_wr_addr_bank0;
      end
    st_wr_addr_bank1:
      begin 
	if (prog_instr_read == 1'b1) 
          next_state = st_wr_read_req_a0_inc;
	else
          next_state = st_wr_addr_bank1;
      end
    st_wr_read_req_a0_inc:
      begin 
        if ((new_frame == 1'b1) && (new_frame_prev == 1'b0)) //**this may not be
	  next_state = st_wr_addr_a0;	//only jump from stall??**
	else if (prog_instr_read == 1'b1) 
	  next_state = st_wait_rx;
        else
          next_state = st_wr_read_req_a0_inc;
      end
    st_wait_rx:
      begin 
	if (prog_instr_read == 1'b1) 
	  next_state = st_stall;
        else
          next_state = st_wait_rx;
      end
    st_stall:
      begin
	if ((prefetch_line == 1'b1) && (prefetch_line_prev == 1'b0))
	  next_state = st_wr_read_req_a0_inc;
        else if ((new_frame == 1'b1) && (new_frame_prev == 1'b0))
          next_state = st_wr_addr_a0;
        else
          next_state = st_stall;
      end
    default:
      next_state = st_reset;
  endcase
end

//Sets the current instruction output to the controller based on the state:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1) 
    begin
      //int_prog_instr <= `wait_rx;
      int_prog_instr <= `noop;
      int_prog_cbit <= 1'b1;
    end	  
  else
    begin
      case (next_state) 
        st_reset:
          begin
      	    //int_prog_instr <= `wait_rx;
      	    int_prog_instr <= `noop;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_addr_a0:
          begin
      	    int_prog_instr <= `wr_addr_a0;
	    int_prog_cbit <= 1'b1;
          end
        st_wr_addr_bank0:
          begin
      	    int_prog_instr <= `addr_bank0;
	    int_prog_cbit <= 1'b0;
          end
        st_wr_addr_bank1:
          begin
      	    int_prog_instr <= `addr_bank1;
	    int_prog_cbit <= 1'b0;
          end
        st_wr_read_req_a0_inc:
          begin
      	    int_prog_instr <= `read_req_a0_inc;
	    int_prog_cbit <= 1'b1;
          end
        st_wait_rx:
          begin
      	    int_prog_instr <= `wait_rx;
	    int_prog_cbit <= 1'b1;
          end
        st_stall:
          begin
      	    int_prog_instr <= 32'h00000000;
	    int_prog_cbit <= 1'b0;
          end
        default:
          begin
      	    //int_prog_instr <= `wait_rx;
      	    int_prog_instr <= `noop;
	    int_prog_cbit <= 1'b1;
          end
      endcase
    end
end

/*//Next state state machine for test program:
always @ (pres_state or prog_instr_read or rst) begin
  case (pres_state) 
    st_read_req_imm:
      begin
        if ((prog_instr_read == 1'b1) && (rst == 1'b0))
		next_state <= st_read_req_abs_imm;
        else
		next_state <= st_read_req_imm;
      end
    st_read_req_abs_imm:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_addr;
        else
		next_state <= st_read_req_abs_imm;
      end
    st_wr_addr:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_read_req_imm; // st_wr_a0;
        else
		next_state <= st_wr_addr;
      end
    /*st_wr_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_new_a0;
        else
		next_state <= st_wr_a0;
      end
    st_wr_new_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_add_a0;
        else
		next_state <= st_wr_new_a0;
      end
    st_add_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_sub_a0;
        else
		next_state <= st_add_a0;
      end
    st_sub_a0:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_indirect;
        else
		next_state <= st_sub_a0;
      end
    st_wr_indirect:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_autoinc;
        else
		next_state <= st_wr_indirect;
      end
    st_wr_autoinc:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_noop;
        else
		next_state <= st_wr_autoinc;
      end
    st_noop:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_reset;
        else
		next_state <= st_noop;
      end
    st_reset:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wait_rx;
        else
		next_state <= st_reset;
      end
    st_wait_rx:
      begin
        if (prog_instr_read == 1'b1)
		next_state <= st_wr_imm;
        else
		next_state <= st_wait_rx;
      end*/
    /*default:
      next_state <= st_read_req_imm;
  endcase
end*/

/*//TEST PROGRAM:
//Sets the current instruction output to the controller based on the state:
always @ (posedge clk or posedge rst) begin
  case (next_state) 
    st_read_req_imm:
      begin
      	int_prog_instr = `read_req_imm;
	int_prog_cbit = 1'b1;
      end
    st_read_req_abs_imm:
      begin
      	int_prog_instr = `read_req_abs_imm;
	int_prog_cbit = 1'b1;
      end
    st_wr_addr:
      begin
      	int_prog_instr = `abs_addr;
	int_prog_cbit = 1'b0;
      end
    /*st_wr_a0:
      begin
      	int_prog_instr = `write_a0;
	int_prog_cbit = 1'b1;
      end
    st_wr_new_a0:
      begin
      	int_prog_instr = `a0_new_addr;
	int_prog_cbit = 1'b0;
      end
    st_add_a0:
      begin
      	int_prog_instr = `add_a0;
	int_prog_cbit = 1'b1;
      end
    st_sub_a0:
      begin
      	int_prog_instr = `sub_a0;
	int_prog_cbit = 1'b1;
      end
    st_wr_indirect:
      begin
      	int_prog_instr = `write_indirect;
	int_prog_cbit = 1'b1;
      end
    st_wr_autoinc:
      begin
      	int_prog_instr = `write_autoinc;
	int_prog_cbit = 1'b1;
      end
    st_noop:
      begin
      	int_prog_instr = `noop;
	int_prog_cbit = 1'b1;
      end
    st_reset:
      begin
      	int_prog_instr = `reset;
	int_prog_cbit = 1'b1;
      end
    st_wait_rx:
      begin
      	int_prog_instr = `wait_rx;
	int_prog_cbit = 1'b1;
      end*/
   /* default:
      begin
      	int_prog_instr = `read_req_imm; //`write_imm;
	int_prog_cbit = 1'b1;
      end
  endcase
end*/

/*//FOR TESTING:
//Used to indicate when the instruction is valid.  Allows us to stall the
//processor when we don't know for sure the next state
always @ (pres_state ) begin
  case(pres_state)
    st_read_req_imm:
      valid_instr = 1'b1;
    st_read_req_abs_imm:
      valid_instr = 1'b1;
    st_wr_addr:
      valid_instr = 1'b1;
    /*st_wr_a0:
      valid_instr = 1'b1;
    st_wr_new_a0:
      valid_instr = 1'b1;
    st_add_a0:
      valid_instr = 1'b1;
    st_sub_a0:
      valid_instr = 1'b1;
    st_wr_indirect:
      valid_instr = 1'b1;
    st_wr_autoinc:
      valid_instr = 1'b1;
    st_noop:
      valid_instr = 1'b1;
    st_reset:
      valid_instr = 1'b1;
    st_wait_rx:
      valid_instr = 1'b1;*/
 // endcase
//end

endmodule
