--************************************************************************
--    This file was written for University of Toronto, ECE532 
--    File: fsl_zbt_control_core_halffull.vhd
--    Uses: SRL_FIFO_532.vhd
--    Used by: fsl_zbt_control.vhd
--             fsl_dual_zbt_control.v
--    Works with ProjectNavigator 6.2.03i
--    Description: Module to wrap zbt controllers for use with 1  
--    FSL connection.  
--
--  This version differs from the fsl_zbt_control_core.vhd in that it 
--  only stops requesting ZBT reads when the internal FIFO is half full
--  This gives less of those "dropped pixels" in the SVGA output, but 
--  makes the output look a little shakier.
--
--    NCKC August 2004
--************************************************************************ 
--
-- Make sure Logic_Clk is running at same frequency as user_memory_ram_clock
-- The RAM clock should be generated by the DCM with the feedback loop that
-- is connected to the feedback pins in the FPGA.  See CLOCKGEN_532 module.
-- fsl_zbt_control_core.vhd


library IEEE;
use IEEE.std_logic_1164.all;
-- use ieee.numeric_std.all;
use IEEE.STD_Logic_arith.all;
use IEEE.STD_Logic_unsigned.all;

entity FSL_ZBT_CONTROL_CORE is
  generic (
    C_DWIDTH      : integer := 32;
    ADDR_BITS     : integer := 19;
    DATA_BITS     : integer := 32;
    ZBT_READ_LATENCY  : integer := 4
  );
  port (
    FSL_Clk : in std_logic;
    Logic_Clk : in std_logic;
    Reset : in std_logic;
    Can_Switch_FSLs_o : out std_logic;

    -- Slave FSL Signals
    FSL_S_Read_o    : out std_logic;
    FSL_S_Data_i    : in  std_logic_vector(0 to C_DWIDTH-1);
    FSL_S_Control_i : in  std_logic;
    FSL_S_Exists_i  : in  std_logic;

    -- Master FSL Signals
    FSL_M_Write_o   : out std_logic;
    FSL_M_Data_o    : out std_logic_vector(0 to C_DWIDTH-1);
    FSL_M_Control_o : out std_logic;
    FSL_M_Full_i    : in  std_logic;

    -- ZBT signals
    MEMORY0_CLK_P_o   : out std_logic;
    MEMORY0_CLKEN_N_o : out std_logic;
    MEMORY0_WEN_N_o   : out std_logic;
    MEMORY0_WENA_N_o  : out std_logic;
    MEMORY0_WENB_N_o  : out std_logic;
    MEMORY0_WENC_N_o  : out std_logic;
    MEMORY0_WEND_N_o  : out std_logic;
    MEMORY0_ADV_LD_N_o : out std_logic;
    MEMORY0_OEN_N_o   : out std_logic; 
    MEMORY0_CEN_N_o   : out std_logic;
    MEMORY0_ADDR_P_o  : out std_logic_vector(ADDR_BITS-1 downto 0);
    MEMORY0_DATA_A_P_io : inout std_logic_vector(7 downto 0);
    MEMORY0_DATA_B_P_io : inout std_logic_vector(7 downto 0);
    MEMORY0_DATA_C_P_io : inout std_logic_vector(7 downto 0);
    MEMORY0_DATA_D_P_io : inout std_logic_vector(7 downto 0);
    
--    user_memory_clock -- Clock for control logic is now Logic_Clk

    user_memory_ram_clock_zbt_i : in std_logic;
    user_memory_clocken_zbt_i   : in std_logic;  -- I assume we'd just want this to be '1'
    user_memory_load_addr_zbt_i : in std_logic;  -- I assume we'd just want this to be '1'

--    user_memory_read -- This should be handled by this modules' logic
--    user_memory_write -- This should be handled by this modules' logic
--    user_memory_wea -- This should be handled by this modules' logic
--    user_memory_web -- This should be handled by this modules' logic
--    user_memory_wec -- This should be handled by this modules' logic
--    user_memory_wed -- This should be handled by this modules' logic

    user_memory_ce_zbt_i        : in std_logic  -- I assume we'd just want this to be '1'

--    user_memory_addr -- This should be handled by this modules' logic
--    user_memory_read_data -- This should be handled by this modules' logic
--    user_memory_write_data  -- This should be handled by this modules' logic


  );
end entity FSL_ZBT_CONTROL_CORE;

library unisim;
use unisim.all;


architecture IMP of FSL_ZBT_CONTROL_CORE is
  
  component SRL_FIFO_old is
    generic (
      C_DATA_BITS : integer := 32;
      C_DEPTH     : integer := 16 );
    port (
      Clk             : in  std_logic;
      Reset           : in  std_logic;
      FIFO_Write      : in  std_logic;
      Data_In         : in  std_logic_vector(0 to C_DATA_BITS-1);
      FIFO_Read       : in  std_logic;
      Data_Out        : out std_logic_vector(0 to C_DATA_BITS-1);
      FIFO_Full       : out std_logic;
      FIFO_Half_Full  : out std_logic;
      FIFO_Half_Empty : out std_logic;
      Data_Exists     : out std_logic);
  end component SRL_FIFO_old;

	COMPONENT MEMORY_CTRL_532
	PORT(
		user_memory_clock : IN std_logic;
		user_memory_ram_clock : IN std_logic;
		user_memory_clocken : IN std_logic;
		user_memory_load_addr : IN std_logic;
		user_memory_read : IN std_logic;
		user_memory_write : IN std_logic;
		user_memory_wea : IN std_logic;
		user_memory_web : IN std_logic;
		user_memory_wec : IN std_logic;
		user_memory_wed : IN std_logic;
		user_memory_ce : IN std_logic;
		user_memory_addr : IN std_logic_vector(18 downto 0);
		user_memory_write_data : IN std_logic_vector(31 downto 0);    
		MEMORY_DATA_A_P : INOUT std_logic_vector(7 downto 0);
		MEMORY_DATA_B_P : INOUT std_logic_vector(7 downto 0);
		MEMORY_DATA_C_P : INOUT std_logic_vector(7 downto 0);
		MEMORY_DATA_D_P : INOUT std_logic_vector(7 downto 0);      
		MEMORY_CLK_P : OUT std_logic;
		MEMORY_CLKEN_N : OUT std_logic;
		MEMORY_WEN_N : OUT std_logic;
		MEMORY_WENA_N : OUT std_logic;
		MEMORY_WENB_N : OUT std_logic;
		MEMORY_WENC_N : OUT std_logic;
		MEMORY_WEND_N : OUT std_logic;
		MEMORY_ADV_LD_N : OUT std_logic;
		MEMORY_OEN_N : OUT std_logic;
		MEMORY_CEN_N : OUT std_logic;
		MEMORY_ADDR_P : OUT std_logic_vector(18 downto 0);
		user_memory_read_data : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;

  type state_values is (st_reset, st_wait1, st_wait2, st_wait3, st_get_ctrl, st_get_addr, st_read, st_write, st_done);
  signal pres_state, next_state: state_values;

  signal FIFO_Write_sig      : std_logic;
  signal FIFO_Read_sig       : std_logic;
  signal FIFO_Full_sig       : std_logic;  -- Shouldn't need this
  signal FIFO_Half_Full_sig  : std_logic;  -- Shouldn't need this
  signal FIFO_Half_Empty_sig : std_logic;  -- Shouldn't need this
  signal Data_Exists_sig     : std_logic;

--  signal FIFO_Read_sigd : std_logic;
--  signal FIFO_Write_sigd : std_logic;

  signal r_datavalid : std_logic_vector(0 to ZBT_READ_LATENCY-1);

  signal user_memory_read_sig     	:  std_logic;
  signal user_memory_write_sig     	:  std_logic;
  signal user_memory_addr_sig    	:  std_logic_vector((ADDR_BITS-1) downto 0);
  signal user_memory_read_data_sig 	:  std_logic_vector((DATA_BITS-1) downto 0); 
  signal user_memory_read_data_sigd 	:  std_logic_vector((DATA_BITS-1) downto 0);  
  signal user_memory_read_data_sigd2 	:  std_logic_vector((DATA_BITS-1) downto 0);  

  signal user_memory_read_sigd : std_logic;
  signal user_memory_write_sigd : std_logic;
  signal in_datad : std_logic_vector((DATA_BITS-1) downto 0);


  -- For next state logic
  signal FSL_S_Ctrl_Exists : std_logic;

  -- components of the control word
  signal num_words : std_logic_vector(0 to (ADDR_BITS-1));
  signal rwnreg : std_logic;
  signal base_addr : std_logic_vector(0 to (ADDR_BITS-1));

  signal zbt_word_count : std_logic_vector(0 to (ADDR_BITS-1));
  signal fsl_word_count : std_logic_vector(0 to (ADDR_BITS-1));

  signal Clock : std_logic;

  -- Debug signals
  --signal logic0, logic1, debug_write : std_logic;
--  signal debug_addr : std_logic_vector((DATA_BITS-1) downto 0);

begin


  Clock <= Logic_Clk;

  -- FSM state register
  statereg: process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      pres_state <= st_reset;
    elsif (Logic_Clk'event and Logic_Clk ='1') then
      pres_state <= next_state;
    end if;
  end process statereg;
  
  -- Capture parts of control word
  ctrlregs: process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      rwnreg <= '1';
      num_words <= (others => '0');
    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      if (pres_state = st_get_ctrl)  then
        num_words <= FSL_S_Data_i(5 to 23);    --Change later from 19bits
        rwnreg <= not FSL_S_Data_i(31);
      else
        num_words <= num_words;
        rwnreg <= rwnreg;
      end if;
    end if;
  end process ctrlregs;

  -- Capture starting address
  addrreg: process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      base_addr <= (others => '0');
    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      if (pres_state = st_get_addr) then
        base_addr <= FSL_S_Data_i(13 to 31);    
      else
        base_addr <= base_addr;
      end if;
    end if;
  end process addrreg;

------------------------ State Machine Outputs --------------------------
  -- We'll never be outputting control, just data
  FSL_M_Control_o <= '0';

  -- Allow switching between FSLs only while in reset state
  Can_Switch_FSLs_o <= '1' when (pres_state = st_reset) and (next_state = st_reset) else -- maybe just next_state =  st_reset is good enough?
                       '1' when (pres_state = st_done) else
                       '0';

  user_mem_proc : process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      -- Disable ZBT read/write signals
      user_memory_read_sig     	<= '0';
      user_memory_write_sig    	<= '0';
      user_memory_addr_sig    	<= (others => '0');

    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      case pres_state is
      when st_reset | st_get_ctrl | st_get_addr | st_done =>
        -- Disable ZBT read/write signals
        user_memory_read_sig     	<= '0';
        user_memory_write_sig    	<= '0';
        user_memory_addr_sig    	<= (others => '0');
      when st_read =>
        --if (FSL_M_Full_i = '0') and (zbt_word_count < num_words) then
        if (FIFO_Half_Full_sig = '0') and (zbt_word_count < num_words) then
          -- Keep requesting more reads
          user_memory_read_sig <= '1';
          user_memory_write_sig <= '0';
          user_memory_addr_sig <= (base_addr + zbt_word_count);
        else
          -- No more reads to request from zbt or FSL is full and FIFO is half full
          user_memory_read_sig <= '0';
          user_memory_write_sig <= '0';
          user_memory_addr_sig <= (others => '0');
        end if;
      when st_write =>
        if (FSL_S_Exists_i = '1') and (zbt_word_count < num_words) then
          -- Write the data since it is available and we need to
          user_memory_read_sig <= '0';
          user_memory_write_sig <= '1';
          user_memory_addr_sig <= (base_addr + zbt_word_count);
        else
          -- No data or done writing
          user_memory_read_sig <= '0';
          user_memory_write_sig <= '0';
          user_memory_addr_sig <= (base_addr + zbt_word_count);
        end if;
      when others =>  -- should never get here
        user_memory_read_sig     	<= '0';
        user_memory_write_sig    	<= '0';
        user_memory_addr_sig    	<= (others => '0');
      end case;

       ---------------------- Is read data valid? -----------------------------
      -- Because it takes MEMORY_CTRL $ZBT_READ_LATENCY (4?) cycles for data to 
      -- be ready on a read
      -- r_datavalid(ZBT_READ_LATENCY) has the valid bit

      r_datavalid(1 to ZBT_READ_LATENCY-1) <= r_datavalid(0 to ZBT_READ_LATENCY-2);
      if (user_memory_read_sig = '0') then
        r_datavalid(0) <= '0';
      else
        r_datavalid(0) <= '1';
      end if;
    end if;
  end process user_mem_proc; 

  --- Need to delay read, write & data sigs so that addr gets out in time ---
  delay_user_mem_proc : process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
       user_memory_read_sigd <= '0';
       user_memory_write_sigd <= '0';
       in_datad <= (others => '0');
    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
       user_memory_read_sigd <= user_memory_read_sig;
       user_memory_write_sigd <= user_memory_write_sig;
       in_datad <= FSL_S_Data_i;
    end if;
  end process delay_user_mem_proc;
       

  counter_proc : process (Logic_Clk, Reset) 
  begin
    if (Reset = '1') then
      -- Re-initialize counters
      zbt_word_count <= (others => '0');
      fsl_word_count <= (others => '0');

    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      case pres_state is
      when st_reset | st_get_ctrl | st_get_addr | st_done =>
        -- Re-initialize counters
        zbt_word_count <= (others => '0');
        fsl_word_count <= (others => '0');
      when st_read =>
        if (FSL_M_Full_i = '0') and (zbt_word_count < num_words) then
          -- Keep requesting more reads
          zbt_word_count <= zbt_word_count + 1;
        else 
          -- no more reads to do or output FSL full
          zbt_word_count <= zbt_word_count;
        end if;
  
        if (FSL_M_Full_i = '0') and (Data_Exists_sig = '1') then
          -- FSL can take more and there's data on the internal FIFO
          fsl_word_count <= fsl_word_count + 1;
        else
          fsl_word_count <= fsl_word_count;
        end if;  

      when st_write =>
        if (FSL_S_Exists_i = '1') and (zbt_word_count < num_words) then
          -- Write the data since it is available and we need to
          zbt_word_count <= zbt_word_count + 1;
        else
          -- No data or done writing
          zbt_word_count <= zbt_word_count;
        end if;
  
        fsl_word_count <= fsl_word_count;

      when others =>  -- shouldn't get here
        zbt_word_count <= (others => '0');
        fsl_word_count <= (others => '0');
      end case;
    end if;
  end process counter_proc;
 
  fsl_proc : process (FSL_Clk, Reset)
    variable FSL_alt_levs : std_logic;
    variable FSL_M_Write_var : std_logic;
    variable FSL_S_Read_var : std_logic;
  begin
    if (Reset = '1') then
      -- Disable writes/reads to/from FSL
      FSL_alt_levs := '0';
      FSL_M_Write_var   := '0';
      FSL_S_Read_var    := '0';
    elsif (FSL_Clk'event) and (FSL_Clk = '1') then
      case pres_state is
      when st_reset | st_done =>
        FSL_M_Write_var   := '0';
        FSL_S_Read_var    := '0';
      when st_get_ctrl | st_get_addr =>
        FSL_M_Write_var   := '0';
        FSL_S_Read_var    := '1';  -- retrieve ctrl/addr word
      when st_read =>
        FSL_S_Read_var := '0';
        if (FSL_M_Full_i = '0') and (Data_Exists_sig = '1') then
          FSL_M_Write_var := '1';
        else
          FSL_M_Write_var := '0';
        end if;  
      when st_write =>
        FSL_M_Write_var := '0';
        if (FSL_S_Exists_i = '1') and (zbt_word_count < num_words) then
          -- Write the data since it is available and we need to
          FSL_S_Read_var := '1';
        else
          -- No data or done writing
          FSL_S_Read_var := '0';
        end if;
      when others =>  -- should never get here
        FSL_M_Write_var  := '0';
        FSL_S_Read_var    := '0';
      end case;
      FSL_alt_levs := not FSL_alt_levs;
    end if;

    FSL_M_Write_o   <= FSL_M_Write_var and not FSL_alt_levs;
    FSL_S_Read_o    <= FSL_S_Read_var and not FSL_alt_levs;


  end process fsl_proc; 

  FIFO_proc : process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      -- Disable FIFO
      FIFO_Read_sig <= '0';
      FIFO_Write_sig <= '0';
    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      case pres_state is
      when st_reset | st_get_ctrl | st_get_addr | st_write | st_done =>
        -- Disable FIFO
        FIFO_Read_sig <= '0';
        FIFO_Write_sig <= '0';
      when st_read =>
        if (r_datavalid(ZBT_READ_LATENCY-1) = '1') then
          -- Throw it on the FIFO, FSL will pick it up
          FIFO_Write_sig <= '1';
        else 
          -- Data not valid yet so don't write to FIFO
          -- OR Done servicing reads, so don't write to FIFO
          FIFO_Write_sig <= '0';
        end if;
  
        if (FSL_M_Full_i = '0') and (Data_Exists_sig = '1') then
          FIFO_Read_sig <= '1';
        else
          FIFO_Read_sig <= '0';
        end if;
      when others =>  -- should never get here
        FIFO_Read_sig <= '0';
        FIFO_Write_sig <= '0';
      end case;
    end if;
  end process FIFO_proc; 

  --- Need to delay fifo input data so that control is timed right ---
  delay_FIFO_proc : process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      user_memory_read_data_sigd <= (others => '0');
      user_memory_read_data_sigd2 <= (others => '0');
    elsif (Logic_Clk'event) and (Logic_Clk = '1') then
      user_memory_read_data_sigd <= user_memory_read_data_sig;
      user_memory_read_data_sigd2 <= user_memory_read_data_sigd;
    end if;
  end process delay_FIFO_proc;

---------------------- FSM Next State Logic---------------------
  -- Should make sure that the first word in the FSL FIFO is a control word
  -- Otherwise, won't know what to do with it
  FSL_S_Ctrl_Exists <= FSL_S_Exists_i and FSL_S_Control_i;

  -- Take care of next_state when clock goes low
  fsm: process (Logic_Clk, Reset)
  begin
    if (Reset = '1') then
      next_state <= st_reset;
    elsif (Logic_Clk'event) and (Logic_Clk = '0') then
      case pres_state is
        when st_reset =>		
          if FSL_S_Ctrl_Exists = '1' then
            next_state <= st_wait1; 
          else
            next_state <= st_reset;
          end if;
        when st_wait1 =>
          next_state <= st_wait2;
        when st_wait2 => 
          next_state <= st_wait3;
        when st_wait3 =>
          if FSL_S_Ctrl_Exists = '1' then
            next_state <= st_get_ctrl; 
          else
            next_state <= st_reset;
          end if;
        when st_get_ctrl =>
          next_state <= st_get_addr;
        when st_get_addr =>
          if rwnreg = '1' then
            next_state <= st_read; 
          else 
            next_state <= st_write;
          end if;
        when st_read =>
          if fsl_word_count >= num_words then
            next_state <= st_done;
          else
            next_state <= st_read;
          end if;
        when st_write =>
          if zbt_word_count >= num_words then
            next_state <= st_done;
          else 
            next_state <= st_write;
          end if;
        when st_done => 
          next_state <= st_reset;
        when others => next_state <= st_reset;
      end case;   
   end if;
  end process fsm;




-----------------------------------------------------------------------------
-- Instantiating the MEMORY_CTRL core
-----------------------------------------------------------------------------
	Inst_MEMORY_CTRL: MEMORY_CTRL_532 PORT MAP(
		MEMORY_CLK_P => MEMORY0_CLK_P_o,
		MEMORY_CLKEN_N => MEMORY0_CLKEN_N_o,
		MEMORY_WEN_N => MEMORY0_WEN_N_o,
		MEMORY_WENA_N => MEMORY0_WENA_N_o,
		MEMORY_WENB_N => MEMORY0_WENB_N_o,
		MEMORY_WENC_N => MEMORY0_WENC_N_o,
		MEMORY_WEND_N => MEMORY0_WEND_N_o,
		MEMORY_ADV_LD_N => MEMORY0_ADV_LD_N_o,
		MEMORY_OEN_N => MEMORY0_OEN_N_o,
		MEMORY_CEN_N => MEMORY0_CEN_N_o,
		MEMORY_ADDR_P => MEMORY0_ADDR_P_o,
		MEMORY_DATA_A_P => MEMORY0_DATA_A_P_io,
		MEMORY_DATA_B_P => MEMORY0_DATA_B_P_io,
		MEMORY_DATA_C_P => MEMORY0_DATA_C_P_io,
		MEMORY_DATA_D_P => MEMORY0_DATA_D_P_io,
		user_memory_clock => Clock,
		user_memory_ram_clock => user_memory_ram_clock_zbt_i,
		user_memory_clocken => user_memory_clocken_zbt_i,
		user_memory_load_addr => user_memory_load_addr_zbt_i,
		user_memory_read => user_memory_read_sigd,
		user_memory_write => user_memory_write_sigd,
		user_memory_wea => user_memory_write_sigd,
		user_memory_web => user_memory_write_sigd,
		user_memory_wec => user_memory_write_sigd,
		user_memory_wed => user_memory_write_sigd,
		user_memory_ce => user_memory_ce_zbt_i,
		user_memory_addr => user_memory_addr_sig,
		user_memory_read_data => user_memory_read_data_sig,
		user_memory_write_data => in_datad
	);


----------------------------------------------------------------------------
-- Instantiating the SRL_FIFO Xilinx core
----------------------------------------------------------------------------
  SRL_FIFO_0 : SRL_FIFO_old
  generic map (
    C_DATA_BITS => 32,
    C_DEPTH     => 8
    )
  port map(
    Clk => Clock,            -- in  std_logic;
    Reset => Reset,          -- in  std_logic;
    FIFO_Write => FIFO_Write_sig,     -- in  std_logic; -- Controlled by st_read logic
    Data_In => user_memory_read_data_sigd2,        -- in  std_logic_vector(0 to C_DATA_BITS-1);
    FIFO_Read => FIFO_Read_sig,       -- in  std_logic;
    Data_Out => FSL_M_Data_o,        -- out std_logic_vector(0 to C_DATA_BITS-1);
    FIFO_Full => FIFO_Full_sig,       -- out std_logic;
    FIFO_Half_Full => FIFO_Half_Full_sig,  -- out std_logic;
    FIFO_Half_Empty => FIFO_Half_Empty_sig, -- out std_logic;
    Data_Exists => Data_Exists_sig     -- out std_logic
    );

end architecture IMP;
