--************************************************************************
--    This file is the same as the version provided by Xilinx in their
--    IP for FSL v2.00a
--
--    File: gen_async_bram.vhd
--
--    Below is the header provided by Xilinx:
--************************************************************************
-------------------------------------------------------------------------------
-- $Id: gen_async_bram.vhd,v 1.1 2004/08/18 20:29:03 chankin Exp $
-------------------------------------------------------------------------------
-- gen_async_bram.vhd - Entity and architecture
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Author:          satish
-- Revision:        $Revision: 1.1 $
-- Date:            $Date: 2004/08/18 20:29:03 $
--
-- History:
--   satish  2004-03-24    New Version
--
-- Description:
-- Code to infer asynchronous dual port bram and separate read/write clock dual
-- port bram
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee; 
use ieee.std_logic_1164.all; 
use ieee.std_logic_unsigned.all; 

entity Async_BRAM is
  generic (
    C_AWIDTH : integer := 8;
    C_DWIDTH : integer := 16
    );
  port (
    clk1 : in  std_logic;
    clk2 : in  std_logic;
    we   : in  std_logic;
    add1 : in  std_logic_vector(C_AWIDTH-1 downto 0);
    add2 : in  std_logic_vector(C_AWIDTH-1 downto 0);
    di   : in  std_logic_vector(C_DWIDTH-1 downto 0);
    do1  : out std_logic_vector(C_DWIDTH-1 downto 0);
    do2  : out std_logic_vector(C_DWIDTH-1 downto 0)
    ); 
end Async_BRAM;

architecture syn of Async_BRAM is 
  type ram_type is array ((2**C_AWIDTH)-1 downto 0) of std_logic_vector (C_DWIDTH-1 downto 0); 
  signal RAM : ram_type; 
  signal read_add1 : std_logic_vector(C_AWIDTH-1 downto 0); 
  signal read_add2 : std_logic_vector(C_AWIDTH-1 downto 0); 
begin 
  process (clk1) 
    begin 
      if (clk1'event and clk1 = '1') then 
          if (we = '1') then 
              RAM(conv_integer(add1)) <= di; 
          end if; 
          read_add1 <= add1; 
      end if; 
  end process; 
  do1 <= RAM(conv_integer(read_add1)); 
  process (clk2) 
    begin 
      if (clk2'event and clk2 = '1') then 
          read_add2 <= add2; 
      end if; 
    end process; 
  do2 <= RAM(conv_integer(read_add2)); 
end syn; 
