--************************************************************************
--    This file is the same as the version provided by Xilinx in their
--    IP for FSL v2.00a
--
--    File: gen_async_dpram.vhd
--
--    Below is the header provided by Xilinx:
--************************************************************************
-------------------------------------------------------------------------------
-- $Id: gen_async_dpram.vhd,v 1.1 2004/08/18 20:29:03 chankin Exp $
-------------------------------------------------------------------------------
-- Gen_DpRAM.vhd - Entity and architecture
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        Gen_DpRAM.vhd
--
-- Description:
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              Gen_DpRAM.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision: 1.1 $
-- Date:            $Date: 2004/08/18 20:29:03 $
--
-- History:
--   goran  2003-10-27    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.numeric_std.all;

entity Gen_DpRAM is
  generic (
    Use_Muxes : Boolean := False;
    Mem_Size  : Integer := 36;
    Addr_Size : Integer := 6;
    Data_Size : Integer := 16
    );
  port (
    Reset    : in Std_Logic;
    -- Read/Write port 1
    Addr1    : in  Std_Logic_Vector(Addr_Size-1 downto 0);
    WrClk    : in  Std_Logic;
    WE       : in  Std_Logic;
    DataIn   : in  Std_Logic_Vector(Data_Size-1 downto 0);
    DataOut1 : out Std_Logic_Vector(Data_Size-1 downto 0);
    -- Read port 2
    Addr2    : in  Std_Logic_Vector(Addr_Size-1 downto 0);
    DataOut2 : out Std_Logic_Vector(Data_Size-1 downto 0)
    );
end Gen_DpRAM;    

library Unisim;
use Unisim.all;

architecture XILINX_RTL of Gen_DpRAM is

  component RAM16x1D is
    port (
      WE    : in  std_logic;
      D     : in  std_logic;
      WClk  : in  std_logic;
      A0    : in  std_logic;
      A1    : in  std_logic;
      A2    : in  std_logic;
      A3    : in  std_logic;
      DPRA0 : in  std_logic;
      DPRA1 : in  std_logic;
      DPRA2 : in  std_logic;
      DPRA3 : in  std_logic;
      SPO   : out std_logic;
      DPO   : out std_logic);
  end component RAM16x1D;

  constant RAM_Banks : integer := 1 + ((Mem_Size - 1) / 16);

  signal Write : Std_Logic_Vector(RAM_Banks-1 downto 0);
  
  subtype Word is Std_Logic_Vector(Data_Size-1 downto 0);
  
  type Mux_Level_Array is array(natural range <>) of Word;
  
  signal RAM_Bank_Out1 : Mux_Level_Array(Ram_Banks-1 downto 0);
  signal RAM_Bank_Out2 : Mux_Level_Array(Ram_Banks-1 downto 0);

  signal RAM_Sel1 : Integer range Ram_Banks-1 downto 0;
  signal RAM_Sel2 : Integer range Ram_Banks-1 downto 0;

  signal Read1 : Std_Logic_Vector(Ram_Banks-1 downto 0);
  signal Read2 : Std_Logic_Vector(Ram_Banks-1 downto 0);

  signal Ram_Addr1 : Std_Logic_Vector(3 downto 0);
  signal Ram_Addr2 : Std_Logic_Vector(3 downto 0);

  function Mux_Tree(Datas   : in Mux_Level_Array;
                    Selects : in Std_Logic_Vector) return Word is
    variable Len,Middle : Integer;
    variable Left,Right : Word;
  begin
    Len := Datas'LENGTH;
    Middle := Len/2 + Datas'Right;
    if Len = 1 then
      return Datas(Datas'Left);
      -- TBD Check that Selects'Length is ....
    elsif Len = 2 then
      if Selects(Selects'Left) = '0' then
        return Datas(Datas'Left);
      else
        return Datas(Datas'Right);
      end if;
    else
      Left  := Mux_Tree(Datas(Datas'Left downto Middle),
                        Selects(Selects'Left-1 downto Selects'Right));
      Right := Mux_Tree(Datas(Middle-1 downto Datas'Right),
                       Selects(Selects'Left-1 downto Selects'Right));
      if Selects(Selects'Left) = '0' then
        return Left;
      else
        return Right;
      end if;
    end if;
  end Mux_Tree;

begin

  RAM_Sel_Gen : if Ram_Banks > 1 generate
    Ram_Sel1 <= to_integer(unsigned(Addr1(Addr1'Left downto 4)));
    Ram_Sel2 <= to_integer(unsigned(Addr2(Addr2'Left downto 4)));
  end generate;

  Write_Select : process(WE,Ram_Sel1)
  begin
    Write <= (others => '0');
    if (WE = '1') then
      if Ram_Banks > 1 then
        Write(Ram_Sel1) <= '1';
      else
        Write(0) <= '1';
      end if;      
    end if;
  end process Write_Select; 

  Read1_Select : process(Ram_Sel1)
  begin
    Read1 <= (others => '0');
    if Ram_Banks > 1 then
      Read1(Ram_Sel1) <= '1';
    else
      Read1(0) <= '1';
    end if;
  end process Read1_Select;

  Read2_Select : process(Ram_Sel2)
  begin
    Read2 <= (others => '0');
    if Ram_Banks > 1 then
      Read2(Ram_Sel2) <= '1';
    else
      Read2(0) <= '1';
    end if;
  end process Read2_Select;

  Fixed_Addr_Bits : if Mem_Size > 8 generate
    Ram_Addr1(3 downto 0) <= Addr1(3 downto 0);
    Ram_Addr2(3 downto 0) <= Addr2(3 downto 0);
  end generate Fixed_Addr_Bits;

  Align_to_4_Addr_Bits : if Mem_Size <= 8 generate
    Ram_Addr1(3 downto Addr1'Left+1) <= (others => '0');
    Ram_Addr1(Addr1'Range) <= Addr1;
    Ram_Addr2(3 downto Addr2'Left+1) <= (others => '0');
    Ram_Addr2(Addr2'Range) <= Addr2;
  end generate Align_to_4_Addr_Bits;
 
  FIFO_RAM : for I in 0 to RAM_Banks-1 generate
    RAMBANKx : for J in 0 to Data_Size-1 generate

      RAM16x1D_I1: RAM16x1D
        port map (
          WE    => Write(I),                  -- [in  std_logic]
          D     => DataIn(J),                   -- [in  std_logic]
          WClk  => WrClk,                -- [in  std_logic]
          A0    => Ram_Addr1(0),                  -- [in  std_logic]
          A1    => Ram_Addr1(1),                  -- [in  std_logic]
          A2    => Ram_Addr1(2),                  -- [in  std_logic]
          A3    => Ram_Addr1(3),                  -- [in  std_logic]
          DPRA0 => Ram_Addr2(0),               -- [in  std_logic]
          DPRA1 => Ram_Addr2(1),               -- [in  std_logic]
          DPRA2 => Ram_Addr2(2),               -- [in  std_logic]
          DPRA3 => Ram_Addr2(3),               -- [in  std_logic]
          SPO   => RAM_Bank_Out1(I)(J),                 -- [out std_logic]
          DPO   => RAM_Bank_Out2(I)(J));                -- [out std_logic]
      TBuffs : if not Use_Muxes generate
--         Must be used if the syntesis tool can not handle Z assignments
--         TBuf1x : TBuf port map ( RAM_Bank_Out1(I)(J), Read1(I), DataOut1(J));
--         TBuf2x : TBuf port map ( RAM_Bank_Out2(I)(J), Read2(I), DataOut2(J));
        DataOut1(J) <= RAM_Bank_Out1(I)(J) when Read1(I) = '1' else 'Z';
        DataOut2(J) <= RAM_Bank_Out2(I)(J) when Read2(I) = '1' else 'Z';
      end generate;
    end generate;
  end generate;

  Muxes : if Use_Muxes generate
    Need_Muxes: if (Addr1'left > 3) generate
      DataOut1 <= Mux_Tree(RAM_Bank_Out1,Addr1(Addr1'Left downto 4));
      DataOut2 <= Mux_Tree(RAM_Bank_Out2,Addr2(Addr2'Left downto 4));      
    end generate Need_Muxes;

    No_Muxes: if (Addr1'left < 4) generate
      DataOut1 <= RAM_Bank_Out1(0);
      DataOut2 <= RAM_Bank_Out2(0);
    end generate No_Muxes;

  end generate;

end XILINX_RTL;
