//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: input2rgb.v
//    Uses: lf_decode_532.v
//          v422to444_v2mult_24t_532.v
//          ycrcb2rgb_v2mult_532.v
//    Used by: fsl_video_input.v
//    Works with ProjectNavigator 6.2.03i
//    Description: 
//      Combine video input processing blocks that were in the Xilinx
//      example design for video input
//
//    NCKC August 2004
//************************************************************************ 

module input2rgb (video_bits,clk,rst,R,G,B,Fmid,Vmid,Hmid);
    input [7:0] video_bits;
    input clk;
    input rst;
    output [7:0] R;
    output [7:0] G;
    output [7:0] B;
    output Fmid, Vmid, Hmid;

wire F_lf_422, V_lf_422, H_lf_422, F_444_rgb, V_444_rgb, H_444_rgb;
wire Fo, Vo, Ho;
wire Fmid, Vmid, Hmid;
wire ceo, ycc_ceo_clk;

wire [7:0] video_bits, ycc_lf_422, Ysig, Crsig, Cbsig, R, G, B;

wire [18:0] Addr;

wire [7:0] Ysel, Crsel, Cbsel;

assign Ysel = ((H_444_rgb == 1 | V_444_rgb == 1) ? 8'h10 : Ysig);
assign Crsel = ((H_444_rgb == 1 | V_444_rgb == 1) ? 8'h80 : Crsig);
assign Cbsel = ((H_444_rgb == 1 | V_444_rgb == 1) ? 8'h80 : Cbsig);

assign Fmid = F_444_rgb; //Nathalie's
assign Vmid = V_444_rgb; //Nathalie's
assign Hmid = H_444_rgb; //Nathalie's
//assign Fmid = Fo;  //Lesley's
//assign Vmid = Vo;  //Lesley's
//assign Hmid = Ho;  //Lesley's


lf_decode_532 lfdec (rst, clk, video_bits,
		      ycc_lf_422, //YCrCb_out
			 F_lf_422, // Fo
			 V_lf_422,  // Vo
			 H_lf_422); //Ho

v422to444_v2mult_24t_532 formatconv ( rst, clk,
					         F_lf_422, //Fi
						    V_lf_422, //Vi
						    H_lf_422, //Hi
				            F_444_rgb,//Fo
						    V_444_rgb, //Vo
						    H_444_rgb, //Ho
						    ycc_ceo_clk, //ceo
						      ycc_lf_422, //YCrCb_in
				              Ysig, //Y_out
						    Crsig, // Cr_out
						    Cbsig); // Cb_out

ycrcb2rgb_v2mult_532 ycc2rgb ( rst, 
					  clk, //clk
				F_444_rgb, //Fin
				V_444_rgb, //Vin
				H_444_rgb, //Hin
				Fo, Vo, Ho,
				ycc_ceo_clk, //cei
				ceo,
					  Ysel, //Y
                    Crsel, // Cr
				Cbsel, //Cb
		              R, G, B);


endmodule

