
module mem_bank_controller (	
 
  //Inputs from fsl_controller to memory controller:
  reset,
  core_clk,
  in_mem_clk,

  //in_clk, //y
  //rst, //y
  in_addr, //y
  in_data, //y
  fifo_empty, //y
  fifo_full, //y
  read_not_write, 
  enable, 

  //Outputs from memory controller to fsl_controller:
  read_fifo, 
  write_fifo, 
  out_data, //connected to fsl //y
  running, 
  done,
  

  //****THESE NAMES MAY HAVE TO CHANGE TO MATCH UCF FILE****
  Dq,  //Data input/output from memory bank  //y

  //Control outputs to memory bank
  Addr, //y
  mem_clk, //y
  CKEb, //y
  Bwa_n, //y
  Bwb_n, //y
  Bwc_n, //y
  Bwd_n, //y
  WEb, //y
  ADV, //y
  OEb, //y
  CS1b, //y
  //CS2, //y
  //CS2b, //y
  LBOb, //y
  ZZ //y

  //MEMORY_CLK_P,
  //MEMORY_CLKEN_N,
  //MEMORY_WEN_N,
  //MEMORY_WENA_N,
  //MEMORY_WENB_N,
  //MEMORY_WENC_N,
  //MEMORY_WEND_N,
  //MEMORY_ADV_LD_N,
  //MEMORY_OEN_N,
  //MEMORY_CEN_N,
  //MEMORY_ADDR_P,
  //MEMORY_DATA_A_P,
  //MEMORY_DATA_B_P,
  //MEMORY_DATA_C_P,
  //MEMORY_DATA_D_P

  );

input  reset;
input  core_clk;
input  in_mem_clk;
//output  MEMORY_CLK_P;
//output  MEMORY_CLKEN_N;
//output  MEMORY_WEN_N;
//output  MEMORY_WENA_N;
//output  MEMORY_WENB_N;
//output  MEMORY_WENC_N;
//output  MEMORY_WEND_N;
//output  MEMORY_ADV_LD_N;
//output  MEMORY_OEN_N;
//output  MEMORY_CEN_N;
//output  MEMORY_ADDR_P;
//inout  MEMORY_DATA_A_P;
//inout  MEMORY_DATA_B_P;
//inout  MEMORY_DATA_C_P;
//inout  MEMORY_DATA_D_P;



//Signals from the fsl controller:
input enable, read_not_write, fifo_empty, fifo_full;
input [18:0] in_addr;
input [31:0] in_data; 
//inout [31:0] in_data; 

//Signals to the fsl controller:
output read_fifo, write_fifo, running, done;
output [31:0] out_data; //to fsl

//Signals to the memory bank:
inout [31:0] Dq;     // Data I/O
output [18:0] Addr;  // Address
output mem_clk;     // Memory Clock
output ADV;   // Address Advance (byte by byte)
output CKEb;  // Slow Down Mode
output WEb;  // Read(="H")/Write(="L")
output Bwa_n, Bwb_n, Bwc_n, Bwd_n; // Byte Write Enables
output CS1b;  // Used chip enable
//output CS2, CS2b;  // other Chip Enables
output OEb;   // Chip Output Enable
output ZZ;    // Sleep Mode
output LBOb;  // Linear Burst Output (used in conjunction with ADV)

wire enable, read_not_write, fifo_empty, fifo_full;
wire mem_read_not_write; //used to ensure that mem defaults to read mode

wire [18:0] in_addr;
wire [31:0] in_data;

//reg read_fifo, write_fifo;
wire read_fifo, write_fifo;
wire running, done;
wire [31:0] out_data;
wire [31:0] mem_out_data;

//reg [31:0] inDq; //Data I/O- NOTE: the only input/output signal in the module
wire [31:0] Dq; //Data I/O- NOTE: the only input/output signal in the module
//reg high_imp_control, high_imp_control_0;  //used to set inout data direction
			//output delayed by a clock cycle

//reg [18:0] Addr; //Address
wire [18:0] Addr; //Address
wire mem_clk;     //clk (K on memory module)
wire ADV;     //Address advance
//reg CKEb;    //Slow down mode: acts as a clock enable
wire CKEb;    //Slow down mode: acts as a clock enable
wire WEb;     //Read is high and Write is Low aka "read, not write"
wire Bwa_n;	//Byte Write Enable A
wire Bwb_n;	//Byte Write Enable B
wire Bwc_n;	//Byte Write Enable C
wire Bwd_n;	//Byte Write Enable D

wire CS1b;	//Chip Enable
//wire CS2;	//Chip Enable
//wire CS2b;	//Chip Enable
//************OUTPUT ENABLED ONLY DURING READ? THEREFORE NOT A WIRE*****
//reg OEb, OEb_0;	//Chip Output Enable and the delay cycle
//***NEED BUFFERING SO SEE THE INTERNAL SIGNALS: ******************
wire OEb; //, OEb_0;	//Chip Output Enable and the delay cycle
wire ZZ;	//Chip Enable vs Sleep Mode
wire LBOb;	//Used with ADV to dictate order bitwise order of outputs
		//eg LBOb = Linear Burst Order bar

`define zbt_read_latency 4 

//parameter st_reset=0;
//parameter st_write0=1;
//parameter st_stall0=2;
//parameter st_write1=3;
//parameter st_read0=4;
//parameter st_stall1=5;
//parameter st_read1=6;
//parameter st_done=7;
////parameter st_assert_datab_read=8;


wire  reset;
wire  core_clk, in_mem_clk;
wire  clk;
//wire  MEMORY_CLK_P;
//wire  MEMORY_CLKEN_N;
//wire  MEMORY_WEN_N;
//wire  MEMORY_WENA_N;
//wire  MEMORY_WENB_N;
//wire  MEMORY_WENC_N;
//wire  MEMORY_WEND_N;
//wire  MEMORY_ADV_LD_N;
//wire  MEMORY_OEN_N;
//wire  MEMORY_CEN_N;
//wire[18:0]  MEMORY_ADDR_P;
//wire[7:0]  MEMORY_DATA_A_P;
//wire[7:0]  MEMORY_DATA_B_P;
//wire[7:0]  MEMORY_DATA_C_P;
//wire[7:0]  MEMORY_DATA_D_P;

  // For Clockgen module
//wire  MASTER_CLOCK_P;

//reg [0:3] pres_state, next_state;
wire rst;  //used to startup the system in a predictable mode

reg[`zbt_read_latency-1:0] valid_data;
wire valid_out;
//reg [18:0] Addr; //Address
//wire [18:0] Addr; //Address

wire MEM_CS2, MEM_CS2b; // LBOb, ZZ; //UNUSED??
wire clk_en, ce1, ce2;
wire adv_ld, wea, web, wec, wed, lbo, zz_in;
//reg read_not_write; 
wire oe;
//reg [31:0] fifo_in_data;
//wire [31:0] fifo_in_data;
//wire[31:0] fifo_out_data;
//reg [8:0] cntr, data_cntr;

wire buffer_fifo_write, buffer_fifo_read;
wire buffer_fifo_full, buffer_fifo_data_exists;
wire buffer_fifo_half_full, buffer_fifo_half_empty;
wire [0:31] buffer_fifo_data_in, buffer_fifo_data_out;


ZBT_CONTROL_532 xilinx_mem_controller(
  .ram_clock(mem_clk), //y
  //.ram_clock(MEMORY_CLK_P), //y
  .ram_clocken_n(CKEb), //y
  //.ram_clocken_n(MEMORY_CLKEN_N), //y
  .ram_we_n(WEb), //y
  //.ram_we_n(MEMORY_WEN_N), //y
  .ram_oe_n(OEb), //y
  //.ram_oe_n(MEMORY_OEN_N), //y
  .ram_ce0_n(CS1b), //y
  //.ram_ce0_n(MEMORY_CEN_N), //y
  .ram_ce1(MEM_CS2), //y
  //.ram_ce1(MEM_CS2), //y
  .ram_ce2_n(MEM_CS2b), //y
  //.ram_ce2_n(MEM_CS2b), //y
  .ram_adv_ld(ADV), //y
  //.ram_adv_ld(MEMORY_ADV_LD_N), //y
  .ram_wea_n(Bwa_n), //y
  //.ram_wea_n(MEMORY_WENA_N), //y
  .ram_web_n(Bwb_n), //y
  //.ram_web_n(MEMORY_WENB_N), //y
  .ram_wec_n(Bwc_n), //y
  //.ram_wec_n(MEMORY_WENC_N), //y
  .ram_wed_n(Bwd_n), //y
  //.ram_wed_n(MEMORY_WEND_N), //y
  .ram_lbo(LBOb), //y
  //.ram_lbo(LBOb), //y
  .ram_zz(ZZ), //y
  //.ram_zz(ZZ), //y
  .ram_addr(Addr), //y
  //.ram_addr(MEMORY_ADDR_P), //y
  .user_read_data(mem_out_data),
  
  .ram_data_bus(Dq), //y
  //.ram_data_bus({MEMORY_DATA_D_P,MEMORY_DATA_C_P,MEMORY_DATA_B_P,MEMORY_DATA_A_P}), //y
  
  .user_ram_clock(in_mem_clk), //y
  .user_clocken(clk_en), 
  .user_read_writez(mem_read_not_write), 
  .user_oe(oe), 
  .user_ce0(ce1), //y
  .user_ce1(ce2), //y
  .user_ce2(ce2), //y
  .user_adv_ld(adv_ld), //y
  .user_wea(wea), //y
  .user_web(web), //y
  .user_wec(wec), //y
  .user_wed(wed), //y
  .user_lbo(lbo), //y
  .user_zz(zz_in), //y
  .clock(clk), //y
  .user_write_data(in_data), //y
  .user_ram_addr(in_addr) //y
);


//----------------------------------------------------------------------------
// Instantiating the SRL_FIFO Xilinx core
//----------------------------------------------------------------------------
 SRL_FIFO_old SRL_FIFO (
  //generic map (
    //C_DATA_BITS => 32,
    //C_DEPTH     => 8
    //)
    .Clk(clk), //in  std_logic;
    .Reset(rst), //in  std_logic;
    .FIFO_Write(buffer_fifo_write), //in  std_logic; 
    						// Controlled by st_read logic
    .Data_In(buffer_fifo_data_in), //in  std_logic_vector(0 to C_DATA_BITS-1);
    .FIFO_Read(buffer_fifo_read), //in  std_logic;
    .Data_Out(buffer_fifo_data_out), //out std_logic_vector(0 to C_DATA_BITS-1);
    .FIFO_Full(buffer_fifo_full), //out std_logic;
    .FIFO_Half_Full(buffer_fifo_half_full), //out std_logic;
    .FIFO_Half_Empty(buffer_fifo_half_empty), //out std_logic;
    .Data_Exists(buffer_fifo_data_exists) //out std_logic
    );

//signals used to connect buffering fifo to the memory output and the 
//FSL input:
assign buffer_fifo_write = valid_out;
assign buffer_fifo_data_in = mem_out_data;
assign buffer_fifo_read = write_fifo;
assign out_data = buffer_fifo_data_out;

assign valid_out = valid_data[`zbt_read_latency-1];

//FOR TESTING ONLY:
//assign fifo_in_data[7:0] = data_cntr;
//assign fifo_in_data[15:8] = data_cntr;
//assign fifo_in_data[23:16] = data_cntr;
//assign fifo_in_data[31:24] = data_cntr;
//assign Addr = {11'h000,data_cntr};

assign wea = 1'b1;
assign web = 1'b1;
assign wec = 1'b1;
assign wed = 1'b1;
assign adv_ld = 1'b1;
assign lbo = 1'b0;
assign zz_in = 1'b0;
assign ce2 = 1'b1;
assign ce1 = 1'b1;
assign clk_en = 1'b1;

assign mem_read_not_write = (read_not_write) || (~enable);
assign clk = core_clk;
assign rst = reset;
assign running = oe || read_fifo;
assign done = (~enable) && (~buffer_fifo_data_exists) && 
	(~valid_data[0]) && (~valid_data[1]) && 
	(~valid_data[2]) && (~valid_data[3]); //used when zbt_read_latency ==3

assign oe = ((buffer_fifo_half_full == 1'b0) && (enable == 1'b1) 
		&& (read_not_write == 1'b1));
assign read_fifo = ((fifo_empty == 1'b0) && (enable == 1'b1)
		&& (read_not_write == 1'b0));
assign write_fifo = ((buffer_fifo_data_exists == 1'b1) && (fifo_full == 1'b0));

//Used to indicate whether there is valid data in the pipeline to be 
//written from memory to the buffering fifo:
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    valid_data[`zbt_read_latency-1:0] <= 4'h0;
  else if (oe == 1'b1)
    begin
      valid_data[`zbt_read_latency-1:1] <= valid_data[`zbt_read_latency-2:0];
      valid_data[0] <= 1'b1;
    end
  else
    begin
      valid_data[`zbt_read_latency-1:1] <= valid_data[`zbt_read_latency-2:0];
      valid_data[0] <= 1'b0;
    end
end

//FOR TESTING ONLY:
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //cntr <= 9'h000;
  //else if (cntr[8] == 1'b1)
    //cntr <= cntr;
  //else
    //cntr <= cntr + 1;
//end

//FOR TESTING ONLY:
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //data_cntr <= 8'h00;
  //else if ((pres_state == st_read0) && (read_not_write == 1'b0))
    //data_cntr <= 8'h00;
  //else if ((read_not_write == 1'b0) || (oe == 1'b1))
    //data_cntr <= data_cntr + 1;
  //else
    //data_cntr <= data_cntr;
//end

//FOR TESTING ONLY:
//Set present state for FSL_S_Data input
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
	  //pres_state <= st_reset;
  //else
	  //pres_state <= next_state;
//end

//FOR TESTING ONLY:
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //read_not_write <= 1'b1;
  //else if ((pres_state == st_write0) || (pres_state == st_write1))
    //read_not_write <= 1'b0;
  //else
    //read_not_write <= 1'b1;
//end

//FOR TESTING ONLY:
//always @ (posedge clk or posedge rst) begin
  //if (rst == 1'b1)
    //oe <= 1'b0;
  //else if ((pres_state == st_read0) || (pres_state == st_read1))
    //oe <= 1'b1;
  //else
    //oe <= 1'b0;
//end

//FOR TESTING ONLY:
//Set next state for FSL_S_Data input
//always @ (rst or pres_state or cntr) begin
  //case (pres_state)
    //st_reset:
      //begin
        //if (rst == 1'b1)
	       //next_state = st_reset;
	//else 
	       //next_state = st_write0;
      //end
    //st_write0:
      //begin
        //if (cntr == 9'h01B)
	       //next_state = st_stall0;
        //else
	       //next_state = st_write0;
      //end
    //st_stall0:
      //begin
        //if (cntr == 9'h028)
	       //next_state = st_write1;
        //else
	       //next_state = st_stall0;
      //end
    //st_write1:
      //begin
        //if (cntr == 9'h063)
	       //next_state = st_read0;
        //else
	       //next_state = st_write1;
      //end
    //st_read0:
      //begin
        //if (cntr == 9'h07E)
	       //next_state = st_stall1;
        //else
	       //next_state = st_read0;
      //end
    //st_stall1:
      //begin
        //if (cntr == 9'h0A3)
	       //next_state = st_read1;
        //else
	       //next_state = st_stall1;
      //end
    //st_read1:
      //begin
        //if (cntr == 9'h0D4)
	       //next_state = st_done;
        //else
	       //next_state = st_read1;
      //end
    //st_done:
      //next_state = st_done;
  //endcase
//end

/*always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
    fifo_in_data <= 32'h00000000;
  else

end
*/
/*
//Set address used to read/write memory locations
always @ (posedge clk or posedge rst) begin
  if (rst == 1'b1)
	  Addr <= 32'h00000000;
  else if (((next_state == st_assert_addrb_write) || 
	  (next_state == st_assert_addrb_write))) // && (CS1b == 1'b0))
	  Addr <= 32'h00007654;
  else if (((next_state == st_assert_addr_write) || 
	  (next_state == st_assert_addr_read))) // && (CS1b == 1'b0))
	  Addr <= 32'h00003210;
  else
	  Addr <= Addr;
end
*/

endmodule
