//************************************************************************
//    This file was written for University of Toronto, ECE532 
//    File: top_dualzbt_dow.v
//    Uses: fsl_v20_532.v
//          fsl_dual_zbt_control.v
//          BM_MODE_SVGA_CTRL_532.v
//          fsl_video_input.v
//          CLOCKGEN_532.v
//          dual_zbt_memctrl.ucf (Remember to associate w/ module if in
//          ISE!)
//    Used by:
//
//    Works with ProjectNavigator 6.2.03i, except for when ZBT behavioral
//    model is included for simulation.  See important note at bottom of
//    file re: use of ZBT model
//    
//    Description: Module to combine FSL video input, FSL dual-zbt bank ZBT
//    controller, and FSL SVGA output modules.  This is the top level file
//    for an FSL "streaming video" design
//
//    NCKC August 2004
//************************************************************************ 

module top_dualzbt_dow (
	reset,
	//sw1,  // For debugging
// remove memory stuff for simulation
  MEMORY0_CLK_P,
  MEMORY0_CLKEN_N,
  MEMORY0_WEN_N,
  MEMORY0_WENA_N,
  MEMORY0_WENB_N,
  MEMORY0_WENC_N,
  MEMORY0_WEND_N,
  MEMORY0_ADV_LD_N,
  MEMORY0_OEN_N,
  MEMORY0_CEN_N,
  MEMORY0_ADDR_P,
  MEMORY0_DATA_A_P,
  MEMORY0_DATA_B_P,
  MEMORY0_DATA_C_P,
  MEMORY0_DATA_D_P,

  MEMORY1_CLK_P,
  MEMORY1_CLKEN_N,
  MEMORY1_WEN_N,
  MEMORY1_WENA_N,
  MEMORY1_WENB_N,
  MEMORY1_WENC_N,
  MEMORY1_WEND_N,
  MEMORY1_ADV_LD_N,
  MEMORY1_OEN_N,
  MEMORY1_CEN_N,
  MEMORY1_ADDR_P,
  MEMORY1_DATA_A_P,
  MEMORY1_DATA_B_P,
  MEMORY1_DATA_C_P,
  MEMORY1_DATA_D_P,

  // For Clockgen module
MASTER_CLOCK_P,
ALTERNATE_CLOCK_P,
MEM_CLK_FBOUT_P,
MEM_CLK_FBIN_P,
EXTEND_DCM_RESET_P,

  // For SVGA
/*  VGA_OUT_PIXEL_CLOCK_P,
  VGA_COMP_SYNCH_N,
  VGA_OUT_BLANK_N,
  VGA_HSYNCH_N,
  VGA_VSYNCH_N,
  VGA_OUT_RED_P,
  VGA_OUT_GREEN_P,
  VGA_OUT_BLUE_P,

  // For Video
  VIDEO_INPUT,
*/
  led0,
  led1
);

//input sw1; // for debugging
input reset;

// remove memory stuf for simulation
output MEMORY0_CLK_P;
output MEMORY0_CLKEN_N;
output MEMORY0_WEN_N;
output MEMORY0_WENA_N;
output MEMORY0_WENB_N;
output MEMORY0_WENC_N;
output MEMORY0_WEND_N;
output MEMORY0_ADV_LD_N;
output MEMORY0_OEN_N;
output MEMORY0_CEN_N;
output [18:0] MEMORY0_ADDR_P;
inout [7:0] MEMORY0_DATA_A_P;
inout [7:0] MEMORY0_DATA_B_P;
inout [7:0] MEMORY0_DATA_C_P;
inout [7:0] MEMORY0_DATA_D_P;
output MEMORY1_CLK_P;
output MEMORY1_CLKEN_N;
output MEMORY1_WEN_N;
output MEMORY1_WENA_N;
output MEMORY1_WENB_N;
output MEMORY1_WENC_N;
output MEMORY1_WEND_N;
output MEMORY1_ADV_LD_N;
output MEMORY1_OEN_N;
output MEMORY1_CEN_N;
output [18:0] MEMORY1_ADDR_P;
inout [7:0] MEMORY1_DATA_A_P;
inout [7:0] MEMORY1_DATA_B_P;
inout [7:0] MEMORY1_DATA_C_P;
inout [7:0] MEMORY1_DATA_D_P;


input		MASTER_CLOCK_P;		// master clock input (27MHz) 
input		ALTERNATE_CLOCK_P;	// alternate clock input (50MHz)
output		MEM_CLK_FBOUT_P;	// memory DCM feedback loop output
input		MEM_CLK_FBIN_P;		// memory DCM feedback loop input
input		EXTEND_DCM_RESET_P;	// signal to extend the DCM reset

//input [7:0] VIDEO_INPUT;  // Incoming NTSC video from comp video input

output led0;
output led1;

//output VGA_OUT_PIXEL_CLOCK_P;
//output VGA_COMP_SYNCH_N;
//output VGA_OUT_BLANK_N;
//output VGA_HSYNCH_N;
//output VGA_VSYNCH_N;
//output [7:0] VGA_OUT_RED_P;
//output [7:0] VGA_OUT_GREEN_P;
//output [7:0] VGA_OUT_BLUE_P;

//wire VGA_OUT_PIXEL_CLOCK_P;
//wire VGA_COMP_SYNCH_N;
//wire VGA_OUT_BLANK_N;
//wire VGA_HSYNCH_N;
//wire VGA_VSYNCH_N;
//wire [7:0] VGA_OUT_RED_P;
//wire [7:0] VGA_OUT_GREEN_P;
//wire [7:0] VGA_OUT_BLUE_P;

wire reset;  //external reset
wire core_rst; //used to reset cores (combines external reset switch, 
		//dcm reset and reset from fsls
wire 			MEMORY0_CLK_P;		// ZBT RAM clock
wire 			MEMORY0_CLKEN_N;	// ZBT RAM clock enable
wire 			MEMORY0_WEN_N;		// ZBT RAM write enable
wire 			MEMORY0_WENA_N;	// ZBT RAM byte A write enable
wire 			MEMORY0_WENB_N;	// ZBT RAM byte B write enable
wire 			MEMORY0_WENC_N;	// ZBT RAM byte C write enable
wire 			MEMORY0_WEND_N;	// ZBT RAM byte D write enable
wire 			MEMORY0_ADV_LD_N;	// ZBT RAM address load-advance
wire 			MEMORY0_OEN_N;		// ZBT RAM output enable
wire 			MEMORY0_CEN_N;		// ZBT RAM chip enable
wire [18:0]		MEMORY0_ADDR_P;// ZBT RAM address bus
wire [7:0] 		MEMORY0_DATA_A_P;	// ZBT RAM data bus byte "A"
wire [7:0] 		MEMORY0_DATA_B_P;	// ZBT RAM data bus byte "B"
wire [7:0] 		MEMORY0_DATA_C_P;	// ZBT RAM data bus byte "C"
wire [7:0] 		MEMORY0_DATA_D_P;	// ZBT RAM data bus byte "D"
//wire inv_MEMORY0_CEN_N;

wire 			MEMORY1_CLK_P;		
wire 			MEMORY1_CLKEN_N;	
wire 			MEMORY1_WEN_N;		
wire 			MEMORY1_WENA_N;	
wire 			MEMORY1_WENB_N;	
wire 			MEMORY1_WENC_N;	
wire 			MEMORY1_WEND_N;	
wire 			MEMORY1_ADV_LD_N;	
wire 			MEMORY1_OEN_N;		
wire 			MEMORY1_CEN_N;		
wire [18:0]		MEMORY1_ADDR_P;
wire [7:0] 		MEMORY1_DATA_A_P;	
wire [7:0] 		MEMORY1_DATA_B_P;	
wire [7:0] 		MEMORY1_DATA_C_P;	
wire [7:0] 		MEMORY1_DATA_D_P;	
//wire inv_MEMORY1_CEN_N;

wire led0, led1;
//wire [7:0] VIDEO_INPUT; 

// Clockgen stuff
wire clk_27mhz;
wire clk_54mhz;
wire clk_108mhz;
wire clk_25mhz;
wire clk_50mhz;
wire pixel_clock;
wire memory_27mhz;
wire memory_54mhz;
wire memory_108mhz;
wire fpga_reset;

wire		MASTER_CLOCK_P;
wire		ALTERNATE_CLOCK_P;
wire		MEM_CLK_FBOUT_P;
wire		MEM_CLK_FBIN_P;
wire 		EXTEND_DCM_RESET_P;

wire LBOb_0, LBOb_1, ZZ_0, ZZ_1;

wire int_rst;
wire logic0_sig; 
wire logic1_sig; 

//wire frame_batch;
/*
wire FSL_svgaM_Rst; 
wire FSL_svgaM_M_Clk; 
wire [0:31] FSL_svgaM_M_Data; 
wire FSL_svgaM_M_Control; 
wire FSL_svgaM_M_Write; 
wire FSL_svgaM_M_Full; 
wire FSL_svgaM_S_Clk; 
wire [0:31] FSL_svgaM_S_Data; 
wire FSL_svgaM_S_Control; 
wire FSL_svgaM_S_Read; 
wire FSL_svgaM_S_Exists; 
wire FSL_svgaM_Full; 
wire FSL_svgaM_Has_Data;

wire FSL_svgaS_Rst; 
wire FSL_svgaS_M_Clk; 
wire [0:31] FSL_svgaS_M_Data; 
wire FSL_svgaS_M_Control; 
wire FSL_svgaS_M_Write; 
wire FSL_svgaS_M_Full; 
wire FSL_svgaS_S_Clk; 
wire [0:31] FSL_svgaS_S_Data; 
wire FSL_svgaS_S_Control; 
wire FSL_svgaS_S_Read; 
wire FSL_svgaS_S_Exists; 
wire FSL_svgaS_Full; 
wire FSL_svgaS_Has_Data;

wire FSL_noM_Rst; 
wire FSL_noM_M_Clk; 
wire [0:31] FSL_noM_M_Data; 
wire FSL_noM_M_Control; 
wire FSL_noM_M_Write; 
wire FSL_noM_M_Full; 
wire FSL_noM_S_Clk; 
wire [0:31] FSL_noM_S_Data; 
wire FSL_noM_S_Control; 
wire FSL_noM_S_Read; 
wire FSL_noM_S_Exists; 
wire FSL_noM_Full; 
wire FSL_noM_Has_Data;

wire FSL_noS_Rst; 
wire FSL_noS_M_Clk; 
wire [0:31] FSL_noS_M_Data; 
wire FSL_noS_M_Control; 
wire FSL_noS_M_Write; 
wire FSL_noS_M_Full; 
wire FSL_noS_S_Clk; 
wire [0:31] FSL_noS_S_Data; 
wire FSL_noS_S_Control; 
wire FSL_noS_S_Read; 
wire FSL_noS_S_Exists; 
wire FSL_noS_Full; 
wire FSL_noS_Has_Data;
*/
wire no_sig,empty_signal;
//wire Can_switch_n;
//reg did_it_write, did_it_read, did_exist;  // LED debugging
//wire ctrl_exist, write_exist;

//added for use with the video's input fsl:
//wire unused_clk;
//wire unused_read;
//wire unused_rd_output;
//wire unused_stat_bit0;

//assign inv_MEMORY0_CEN_N = !MEMORY0_CEN_N;
assign logic0_sig = 1'b0;
assign logic1_sig = 1'b1;

// Only count having left reset as when the dcm's are locked
assign int_rst = reset || fpga_reset;
assign core_rst = int_rst; // || FSL_rst;
//assign FSL_rst = FSL_svgaM_Rst | FSL_svgaS_Rst | FSL_noM_Rst | FSL_noS_Rst;

assign led0 = reset || fpga_reset;  // lit when out of reset
assign led1 = 1'b1;
//assign led1 = !did_exist;
//assign led0 = !did_it_write;  // lit when write_module writing
//assign led1 = !FSL_noM_M_Write;
//assign led1 = !did_it_read;


//assign ctrl_exist = ((FSL_noM_S_Exists == 1'b1 && FSL_noM_S_Control == 1'b1 && FSL_noM_S_Read == 1'b1) ? 1'b1 : 1'b0);
//assign write_exist = ((FSL_noM_M_Control == 1'b1 && FSL_noM_M_Write == 1'b1) ? 1'b1 : 1'b0);
/*
always @ (posedge clk_27mhz or posedge reset)
begin
  if (reset)
          did_it_write <= 1'b0;
  else begin
          if (write_exist == 1'b1)
 	    did_it_write <= 1'b1;
          else
            did_it_write <= did_it_write;
  end
end

always @ (posedge clk_27mhz or posedge reset)
begin
  if (reset)
          did_exist <= 1'b0;
  else begin
          if (ctrl_exist == 1'b1)
 	    did_exist <= 1'b1;
          else
            did_exist <= did_exist;
  end
end
*/
/*always @ (posedge FSL_svgaS_M_Write or posedge reset)
begin
	if (reset)
          did_it_write <= 1'b0;
        else
	  did_it_write <= 1'b1;
end
*/


/*
always @ (posedge FSL_noM_S_Read or posedge reset)
begin
	if (reset)
          did_it_read <= 1'b0;
        else
	  did_it_read <= 1'b1;
end
*/

/*always @ (posedge FSL_svgaM_S_Read or posedge reset)
begin
	if (reset)
          did_it_read <= 1'b0;
        else
	  did_it_read <= 1'b1;
end
*/

CLOCKGEN_532 clockgen_532 (
    .MASTER_CLOCK_P(MASTER_CLOCK_P), 
    .ALTERNATE_CLOCK_P(ALTERNATE_CLOCK_P), 
    .MEM_CLK_FBOUT_P(MEM_CLK_FBOUT_P), 
    .MEM_CLK_FBIN_P(MEM_CLK_FBIN_P), 
    .EXTEND_DCM_RESET_P(EXTEND_DCM_RESET_P), 
    .clk_27mhz(clk_27mhz), 
    .clk_54mhz(clk_54mhz), 
    .clk_108mhz(clk_108mhz), 
    .clk_25mhz(clk_25mhz), 
    .clk_50mhz(clk_50mhz), 
    .pixel_clock(pixel_clock), // Ignore this, useless
    .memory_27mhz(memory_27mhz), 
    .memory_54mhz(memory_54mhz), 
    .memory_108mhz(memory_108mhz), 
    .pal_mode(logic0_sig), 
    .fpga_reset(fpga_reset)
    );

    //memory_controller memory_controller (	
    data_gen_for_fsl_sh_mem_controller data_generator (	
  .reset(core_rst),
  //.core_clk(clk_27mhz),
  //.mem_clk(memory_27mhz),
  .core_clk(clk_54mhz),
  .mem_clk(memory_54mhz),
  //.Dq({MEMORY0_DATA_A_P,MEMORY0_DATA_B_P,MEMORY0_DATA_C_P,MEMORY0_DATA_D_P}),
  .Dq_0({MEMORY0_DATA_D_P,MEMORY0_DATA_C_P,MEMORY0_DATA_B_P,MEMORY0_DATA_A_P}),
  .Addr_0(MEMORY0_ADDR_P),
  .mem_clk_0(MEMORY0_CLK_P),
  .CKEb_0(MEMORY0_CLKEN_N),
  .Bwa_n_0(MEMORY0_WENA_N),
  .Bwb_n_0(MEMORY0_WENB_N),
  .Bwc_n_0(MEMORY0_WENC_N),
  .Bwd_n_0(MEMORY0_WEND_N),
  .WEb_0(MEMORY0_WEN_N),
  .ADV_0(MEMORY0_ADV_LD_N),
  .OEb_0(MEMORY0_OEN_N),
  .CS1b_0(MEMORY0_CEN_N),
  .LBOb_0(LBOb_0),
  .ZZ_0(ZZ_0),

  //.Dq({MEMORY1_DATA_A_P,MEMORY1_DATA_B_P,MEMORY1_DATA_C_P,MEMORY1_DATA_D_P}),
  .Dq_1({MEMORY1_DATA_D_P,MEMORY1_DATA_C_P,MEMORY1_DATA_B_P,MEMORY1_DATA_A_P}),
  .Addr_1(MEMORY1_ADDR_P),
  .mem_clk_1(MEMORY1_CLK_P),
  .CKEb_1(MEMORY1_CLKEN_N),
  .Bwa_n_1(MEMORY1_WENA_N),
  .Bwb_n_1(MEMORY1_WENB_N),
  .Bwc_n_1(MEMORY1_WENC_N),
  .Bwd_n_1(MEMORY1_WEND_N),
  .WEb_1(MEMORY1_WEN_N),
  .ADV_1(MEMORY1_ADV_LD_N),
  .OEb_1(MEMORY1_OEN_N),
  .CS1b_1(MEMORY1_CEN_N),
  .LBOb_1(LBOb_1),
  .ZZ_1(ZZ_1)



  //.MEMORY_CLK_P(MEMORY0_CLK_P),
  //.MEMORY_CLKEN_N(MEMORY0_CLKEN_N),
  //.MEMORY_WEN_N(MEMORY0_WEN_N),
  //.MEMORY_WENA_N(MEMORY0_WENA_N),
  //.MEMORY_WENB_N(MEMORY0_WENB_N),
  //.MEMORY_WENC_N(MEMORY0_WENC_N),
  //.MEMORY_WEND_N(MEMORY0_WEND_N),
  //.MEMORY_ADV_LD_N(MEMORY0_ADV_LD_N),
  //.MEMORY_OEN_N(MEMORY0_OEN_N),
  //.MEMORY_CEN_N(MEMORY0_CEN_N),
  //.MEMORY_ADDR_P(MEMORY0_ADDR_P),
  //.MEMORY_DATA_A_P(MEMORY0_DATA_A_P),
  //.MEMORY_DATA_B_P(MEMORY0_DATA_B_P),
  //.MEMORY_DATA_C_P(MEMORY0_DATA_C_P),
  //.MEMORY_DATA_D_P(MEMORY0_DATA_D_P)

  
  // For Clockgen module
  //MASTER_CLOCK_P
  //ALTERNATE_CLOCK_P,
  //MEM_CLK_FBOUT_P,
  //MEM_CLK_FBIN_P,
  //EXTEND_DCM_RESET_P
  );

endmodule

  // Can uncomment this part if you want to try synthesis in ISE to check
  // syntax.  This would have to be used as a synthesis black box because
  // including the actual module in the project will cause lots of errors.
  // ISE *will* complain about the inout lines being connected to the ZBT
  // model instead of to inout pins.  At that point, you will have to
  // switch to compiling in ModelSim and compile the actual ZBT verilog
  // model.  NOTE:  WHEN YOU MOVE TO COMPILING IN MODELSIM MAKE SURE TO
  // COMMENT THIS PART OUT IF YOU WERE USING IT BEFORE!

/*module K7N163601A ( Dq, Addr, K, CKEb, 
	   	  Bwa_n, Bwb_n, Bwc_n, Bwd_n,
	   	  WEb, ADV, OEb,                  
	   	  CS1b, CS2, CS2b, LBOb, ZZ ); 


inout   [31 : 0] Dq;                            // Data I/O
input   [18 : 0] Addr;                          // Address

input   K;                                                  // Clock
input   ADV;                                                  // Address Advance
input   CKEb;                                                  // Slow Down Mode
input   WEb;                                                  // Read(="H")/Write(="L")

input   Bwa_n;                                                // Byte Write Enable A
input   Bwb_n;                                                // Byte Write Enable B
input   Bwc_n;                                                // Byte Write Enable C
input   Bwd_n;                                                // Byte Write Enable D

input   CS1b;                                                  // Chip Enable
input   CS2;                                                   // Chip Enable
input   CS2b;                                                   // Chip Enable
input   OEb;                                                   // Chip Enable
input   ZZ;                                                   // Chip Enable
input   LBOb;                                                   // Chip Enable
endmodule
*/
